package domain;

import java.io.Serializable;
import java.rmi.RemoteException;
import java.rmi.server.UnicastRemoteObject;
import java.util.ArrayList;
import java.util.Observable;
import java.util.UUID;

import communications.IClient;
import ea.domain.ListGames;
import ea.domain.stateObservable;
import ea.libs.Constants;
import exceptions.GameNotFoundException;
import exceptions.InvalidArsenalException;
import exceptions.InvalidTerritoryException;
import exceptions.NotCurrentPlayerGameException;
public class GameListener extends UnicastRemoteObject implements IClient, Serializable {
	
	private static final long serialVersionUID = -6470872431147212072L;
	private stateObservable state;
	
	public GameListener(stateObservable state) throws RemoteException
	{
		this.state = state;
	}
	
	private stateObservable getState()
	{
		return state;
	}
	
	@Override
	public void attackAccepted(UUID game) throws RemoteException,
			GameNotFoundException {
		
		if(!ListGames.checkGame(game))
		{
			throw new GameNotFoundException("Este juego no lo esta jugando este cliente.");
		}	
		//Not there errors
		this.getState().setAction(Constants.notifications.ResolveAttack);
		this.getState().setGame(game);
		this.getState().notifyObserver();
	}

	@Override
	public void negotiationRequested(UUID game, int money, int soldiers)
			throws RemoteException, GameNotFoundException,
			InvalidArsenalException 
	{
		this.getState().setAction(Constants.notifications.Negotiate);
		this.getState().setSoldier(soldiers);
		this.getState().setMoney(money);
		this.getState().notifyObserver();
	}

	@Override
	public void territoryUnderAttack(UUID game, Territory src, Territory dst,
			Arsenal arsenal) throws RemoteException, GameNotFoundException,
			InvalidTerritoryException {
		if(!ListGames.checkGame(game))
		{
			throw new GameNotFoundException("Este juego no lo esta jugando este cliente.");
		}
		Game g = ListGames.getGame(game);
		int pos = g.getMap().indexOf(src);
		if(pos==-1){
			throw new InvalidTerritoryException();
		}
		else
		{
			Territory aux = g.getMap().get(pos);
			if(aux.getOwner().equals(this.getState().getNamePlayer()))
			{
				throw new InvalidTerritoryException();
			}
		}	
		
		pos = g.getMap().indexOf(dst);
		if(pos==-1)
		{
			throw new InvalidTerritoryException();
		}
		else
		{
			Territory aux = g.getMap().get(pos);
			if(!aux.getOwner().equals(this.getState().getNamePlayer()))
			{
				throw new InvalidTerritoryException();
			}
		}
		//Not there errors
		
		this.getState().setSrc(src);
		this.getState().setDst(dst);
		this.getState().setArsenal(arsenal);
		this.getState().setGame(game);
		this.getState().setAction(Constants.notifications.Defender);
		this.getState().notifyObserver();
	}

	@Override
	public void timeExpired(UUID game, TimeType whatTime)
			throws RemoteException, GameNotFoundException 
	{
		switch(whatTime)
		{
			case TurnExpired:
				if(!ListGames.checkGame(game))
				{
					throw new GameNotFoundException("Este juego no lo esta jugando este cliente.");
				}	
				//Not there errors
				this.getState().setAction(Constants.notifications.TimeTurnExpired);
				this.getState().setGame(game);
				this.getState().notifyObserver();
				break;
			case DefenseExpired:
				if(!ListGames.checkGame(game))
				{
					throw new GameNotFoundException("Este juego no lo esta jugando este cliente.");
				}	
				//Not there errors
				this.getState().setAction(Constants.notifications.TimeDefenseExpired);
				this.getState().setGame(game);
				this.getState().notifyObserver();
				break;
			case NegotiationExpired:
				if(!ListGames.checkGame(game))
				{
					throw new GameNotFoundException("Este juego no lo esta jugando este cliente.");
				}	
				//Not there errors
				this.getState().setAction(Constants.notifications.TimeNegotiationExpired);
				this.getState().setGame(game);
				this.getState().notifyObserver();
				break;
		}
	}

	@Override
	public void updateClient(UUID game, ArrayList<Player> playerUpdate,
			ArrayList<Territory> territoryUpdate, EventType event) throws RemoteException,
																		  GameNotFoundException,
																		  NotCurrentPlayerGameException
	{
		if(!ListGames.checkGame(game))
		{
			throw new NotCurrentPlayerGameException("Este juego no lo esta jugando este cliente.");
		}	
		//Not there are errors
		Game g = ListGames.getGame(game);
		g.setPlayers(playerUpdate);
		g.setMap(territoryUpdate);
		if(event == EventType.TurnChanged)
			this.getState().setAction(Constants.notifications.TurnChanged);
		else
			this.getState().setAction(Constants.notifications.UpdateGame);
		
		this.getState().setGame(game);
		this.getState().notifyObserver();
	}
}
