package ea.communications;

import java.net.ConnectException;
import java.rmi.Naming;
import java.rmi.RemoteException;
import java.rmi.server.UnicastRemoteObject;
import java.util.ArrayList;
import java.util.UUID;

import communications.IClient;
import communications.IServer;

import domain.*;

import ea.domain.Listener;
import ea.exceptions.NullInformationException;
import ea.presentation.Alert;
import ea.presentation.CommonConstants;
import exceptions.*;

@SuppressWarnings("serial")
public class Client extends UnicastRemoteObject 
{
	private IServer server;
	@SuppressWarnings("unused")
	private String host;
	
//Probado	
    public Client(String host) throws RemoteException
    {
    	this.host = host;
        try
        {
        	if(CommonConstants.FAKESERVER)
        		this.server = (IServer)Naming.lookup("rmi://" + host + ":1099/Server");
        	else 
        		this.server = (IServer)Naming.lookup("rmi://" + host + ":1099/Server");
        }
        catch(java.rmi.ConnectException e)
        {
            System.err.println("Error en la conexion: " + e.getLocalizedMessage() + e.getMessage());
            // [TODO] Define by server?
            // throw new ServerRefusedException()
            new Alert("Error fatal", 
            			"Servidor no iniciado.\nException:\n" + e.getLocalizedMessage() +
            			"\n\nCausa:\n" + e.getCause()); 
        }
        catch(Exception e)
        {
            System.err.println("Client exception: " + e.toString());
            e.printStackTrace();
        }
    }
    
//Probado    
    private IServer getServer()throws NullInformationException
    {
    	if(server == null)
    		throw new NullInformationException("El objeto servidor RMI es nulo.");
    	return this.server;
    }

//Probado    
    public UUID loginPlayer(String name, String password) throws RemoteException, 
    															 WrongLoginException, ConnectException, 
    															 NullInformationException
    {
    	UUID response;
    	try
    	{
    		Listener.initializeListener(name);
    			
    		response = this.getServer().loginUser(name, password,((IClient)Listener.getListener()));
    	}
    	catch(java.lang.NullPointerException e)
    	{
    		throw new NullInformationException("El servidor devolvio un valor nulo en la funcion loginPlayer().");
    	}
			
		if(CommonConstants.DEBUG)
        	System.out.println("response: " + response);
        return response;
    }
    
//Probado    
	public void logoutUser(UUID session) throws RemoteException,
												InvalidSessionException,
												NullInformationException
	{
		try
		{
			this.getServer().logoutUser(session);
		}
    	catch(RemoteException e) 
    	{
			e.printStackTrace();
		} 
    	catch(InvalidSessionException e)
    	{
    		new Alert("Error", "La sesin es invlida");
    	}
	}

//Probado
    public void joinGame(UUID session, UUID game) throws RemoteException,
    													 FullGameException, 
    													 GameNotFoundException, 
    													 InvalidSessionException, 
    													 AlreadyInGameException,
    													 NullInformationException
    {
    	this.getServer().joinGame(session, game);
    }

//Probado   
	public ArrayList<GameInfo> listGames(UUID session) throws RemoteException, InvalidSessionException, 
														ConnectException, NullInformationException
	{
		ArrayList<GameInfo> allGamesInfo;
		try
		{
			allGamesInfo = this.getServer().listGames(session);
		}
		catch(java.lang.NullPointerException e)
		{
			throw new NullInformationException("El servidor devolvio una lista de partidas nula.");
		}
		return allGamesInfo;
	}
	
	public void updateGame(UUID session, UUID game, 
					ArrayList<Player> playerUpdate,
					ArrayList<Territory> territoryUpdate, EventType event) 
	{
		/*try
		{                            
			this.getServer().updateGame(session, game, playerUpdate, territoryUpdate, event);
		}
		catch(RemoteException e)
		{
			e.printStackTrace();
		}
		catch(GameNotFoundException e)
		{
			new Alert("Error", "Objeto game no encontrado");
		}
		catch(InvalidSessionException e)
		{
			new Alert("Error", "La sesin es invlida");
		}
		catch(NotCurrentPlayerGameException e)
		{
			new Alert("Error", "El jugador actual no se encuentra en la partida");
		}*/
	}
	
//Probado	
	public void registerPlayer(String user, String pass, String email) throws UserAlreadyExistsException, 
																		RemoteException, ConnectException,
																		NullInformationException
	{
		if(CommonConstants.DEBUG)
			System.out.println("User:");
		this.getServer().registerUser(user, pass, email);  
	}

//Probado	
	public Game playGame(UUID session, UUID game) throws RemoteException,
														 GameNotFoundException, 
														 InvalidSessionException,
														 InvalidTimeException,
														 NullInformationException, 
														 NotCurrentPlayerGameException, 
														 AlreadyInGameException									 
	{
		Game resgame;
		try{
			resgame = this.getServer().playGame(session, game);
		}
		catch(java.lang.NullPointerException e)
		{
			throw new NullInformationException("El servidor devolvio el juego nulo.");
		} 
		return resgame;
	}

	public void resignGame(UUID session, UUID game, ArrayList<Player> players,
			ArrayList<Territory>territories, EventType type)
   {
		/*try{
			this.getServer().updateGame(session, game, players, territories, type);
    		
			try 
    		{
    			this.getServer().resignGame(session, game);
    		} 
    		catch(RemoteException e) 
    		{
    			e.printStackTrace();
    		} 
    		catch(GameNotFoundException e) 
    		{
    			new Alert("Error", "El objeto Game no se encuentra");
    		} 
    		catch(InvalidSessionException e) 
    		{
    			new Alert("Error", "La sesin es invalida");
    		}
    		catch(NotCurrentPlayerGameException e) 
    		{
    			new Alert("Error", "No es permitido un jugador concurrente en una partida");
    		}
   		}
		catch(RemoteException e)
		{
			e.printStackTrace();
		}
		catch(GameNotFoundException e)
		{
			new Alert("Error","El objeto Game no se encuentra");
		}
		catch(NotCurrentPlayerGameException e)
		{
			new Alert("Error", "No es permitido un jugador concurrente en una partida");
		}
		catch(InvalidSessionException e)
		{
			new Alert("Error", "La sesin es invalida");
		}
		
		if(CommonConstants.DEBUG) System.out.println("User resign game");*/
	}

	public void quitGame(UUID session, UUID game,  ArrayList<Player> players,
			ArrayList<Territory>territories, EventType type)
	{
		/*try{
			this.getServer().updateGame(session, game, players, territories, type);
			try
			{
				this.getServer().quitGame(session, game);
			}
			catch(RemoteException e) 
			{
				e.printStackTrace();
			} 
			catch(GameNotFoundException e) 
			{
				new Alert("Error", "El objeto Game no se encuentra");
			} 
			catch(InvalidSessionException e) 
			{
				new Alert("Error", "La sesin es invalida");
			}
			catch(InvalidTimeException e) 
			{
				new Alert("Error", "El tiempo es invalido");
			}
			catch(NotCurrentPlayerGameException e) 
			{
				new Alert("Error", "No es permitido un jugador concurrente en una partida");
			}
		}
		catch(RemoteException e)
		{
			e.printStackTrace();
		}
		catch(GameNotFoundException e)
		{
			new Alert("Error","El objeto Game no se encuentra");
		}
		catch(NotCurrentPlayerGameException e)
		{
			new Alert("Error", "No es permitido un jugador concurrente en una partida");
		}
		catch(InvalidSessionException e)
		{
			new Alert("Error", "La sesin es invalida");
		}
		if(CommonConstants.DEBUG) System.out.println("User quit game");*/
	}

	public void attackTerritory(UUID session, UUID game, Territory src,
			Territory dst, Arsenal arsenal) throws RemoteException,
			GameNotFoundException, InvalidSessionException,
			InvalidTerritoryException 
	{
		/*try
		{
			this.getServer().attackTerritory(session, game, 
											 src, dst, 
											 arsenal);
		}
		catch(RemoteException e)
		{
			e.printStackTrace();
		}
		catch(GameNotFoundException e)
		{
			new Alert("Error", "Objeto game no encontrado");
		}
		catch(InvalidSessionException e)
		{
			new Alert("Error", "La sesin es invlida");
		}
		catch(InvalidTerritoryException  e)
		{
			new Alert("Error", "Territorio no vlido");
		
		} 
		catch(InvalidTimeException e) 
		{
			new Alert("Error", "Tiempo no vlido");
		}*/
	}
	
	public void acceptAttack(UUID session, UUID game) throws RemoteException,
											   GameNotFoundException, InvalidSessionException
	{
		/*try
		{
			this.getServer().acceptAttack(session, game);
		}
		catch(RemoteException e)
		{
			e.printStackTrace();
		}
		catch(GameNotFoundException e)
		{
			new Alert("Error", "Objeto game no encontrado");
		}
		catch(InvalidSessionException e)
		{
			new Alert("Error", "La sesin es invlida");
		}
		catch(InvalidTimeException e) 
		{
			new Alert("Error", "Tiempo no vlido");
		}				*/	   
	}

	public void requestedNegotiation(UUID session, UUID game, int money,
			int soldiers) throws RemoteException, GameNotFoundException,
			InvalidSessionException 
	{
		/*try
		{
			this.getServer().requestedNegotiation(session, game, 
												  money, soldiers);
		}
		catch(RemoteException e)
		{
			e.printStackTrace();
		}
		catch(GameNotFoundException e)
		{
			new Alert("Error", "Objeto game no encontrado");
		}
		catch(InvalidSessionException e)
		{
			new Alert("Error", "La sesin es invlida");
		} 
		catch (InvalidTimeException e) 
		{
			new Alert("Error", "Tiempo no vlido");
		}	*/
	}

	public void endTurn(UUID session, UUID game, ArrayList<Player> players,
			ArrayList<Territory>territories, EventType type) 
	{
		/*try{
			this.getServer().updateGame(session, game, players, territories, type);
			try
			{
				this.getServer().endTurn(session, game);
			}
			catch(RemoteException e) 
			{
				e.printStackTrace();
			} 
			catch(InvalidSessionException e) 
			{
				new Alert("Error", "La sesin es invalida");
			}
			catch(InvalidTimeException e) 
			{
				new Alert("Error", "El tiempo es invalido");
			}
		}
		catch(RemoteException e)
		{
			e.printStackTrace();
		}
		catch(GameNotFoundException e)
		{
			new Alert("Error","El objeto Game no se encuentra");
		}
		catch(NotCurrentPlayerGameException e)
		{
			new Alert("Error", "No es permitido un jugador concurrente en una partida");
		}
		catch(InvalidSessionException e)
		{
			new Alert("Error", "La sesin es invalida");
		}
		if(CommonConstants.DEBUG) System.out.println("User end turn");*/
	}
	
//Probado	
	public void createGame(UUID session, GameInfo info) throws RemoteException, ConnectException,
												               InvalidGameInfoException, 
												               InvalidSessionException,
												               NullInformationException
	{
		this.getServer().createGame(session, info);
	}
}