package ea.domain;
import java.net.ConnectException;
import java.rmi.RemoteException;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.UUID;

import domain.EventType;
import domain.Game;
import domain.GameInfo;

import ea.communications.Client;
import ea.exceptions.*;
import ea.libs.Constants;
import exceptions.AlreadyInGameException;
import exceptions.FullGameException;
import exceptions.GameNotFoundException;
import exceptions.InvalidGameInfoException;
import exceptions.InvalidSessionException;
import exceptions.InvalidTimeException;
import exceptions.NotCurrentPlayerGameException;
import exceptions.UserAlreadyExistsException;
import exceptions.WrongLoginException;

public class Connection 
{
	private static Client client = null;
	private static String host; 
	
//Probado	
	public Connection()
	{
	}
	
//Probado	
	public static void setHost(String host)throws NullInformationException, EmptyInformationException
	{
		if(host == null){
			throw new NullInformationException("The host is null.");
		}
		else{
			if(host.equals("")){
				throw new EmptyInformationException("The host is empty.");
			}
			else{
				Connection.host = host;
				Connection.client = null;
			}
		}
	}
	
//Probado	
	private static String getHost()throws NullInformationException, EmptyInformationException
	{
		if(Connection.host == null)
			throw new NullInformationException("The host is null.");
		if(Connection.host.equals(""))
			throw new EmptyInformationException("The host is empty.");
		return Connection.host;
	}
	
//Probado	
	public static Client getConnection()throws NullInformationException, EmptyInformationException
	{
		if(client == null)
		{
			try 
			{
				String currentHost = getHost();
				Connection.client = new Client(currentHost);
			} 
			catch(RemoteException e) 
			{
				e.printStackTrace();
			}
		}
		return Connection.client;
	}

//Probado	
	public static UUID loginPlayer(String name, String pswd)throws RemoteException, 
	 															   WrongLoginException, 
	 															   ConnectException, 
	 															   NullInformationException,
	 															   EmptyInformationException
	{
		return Connection.getConnection().loginPlayer(name, pswd);
	}
	
//Probado	
	public static void registerPlayer(String user, String pass,String email)throws 
																	UserAlreadyExistsException, 
																	RemoteException, 
																	ConnectException,
																	NullInformationException,
																	EmptyInformationException
	{
		Connection.getConnection().registerPlayer(user, pass, email);
	}

//Probado	
	public static ArrayList<GameInfo> listGames(UUID session)throws RemoteException,
												InvalidSessionException, 
												ConnectException, NullInformationException, 
												EmptyInformationException
	{
		if(session == null)
			throw new NullInformationException("El identificador de la sesin es nulo.");
		return Connection.getConnection().listGames(session); 
	}
	
//Probado
	public static Game playGame(UUID session, UUID idgame)throws RemoteException,
	 														GameNotFoundException, 
	 														InvalidSessionException,
	 														InvalidTimeException,
	 														NullInformationException, 
	 														NotCurrentPlayerGameException, 
	 														AlreadyInGameException,
	 														EmptyInformationException
	{
		if(session == null)
			throw new NullInformationException("El identificador de la sesin es nulo.");
		
		if(idgame == null)
			throw new NullInformationException("El identificador del juego es nulo.");
		
		return Connection.getConnection().playGame(session, idgame);
	}
	
//Probado	
	public static void joinGame(UUID session,UUID game)throws RemoteException,
	 													FullGameException, 
	 													GameNotFoundException, 
	 													InvalidSessionException, 
	 													AlreadyInGameException,
	 													EmptyInformationException,
	 													NullInformationException
	 													
	{
		if(session == null)
			throw new NullInformationException("El identificador de la sesin es nulo.");
		
		if(game == null)
			throw new NullInformationException("El identificador del juego es nulo.");
		
		Connection.getConnection().joinGame(session, game);
	}
	
//Probado	
	public static void logoutPlayer(UUID idSession)throws RemoteException,
													InvalidSessionException,
													NullInformationException,
													EmptyInformationException
	{
		if(idSession == null)
			throw new NullInformationException("El identificador de la sesin es nulo.");
		Connection.getConnection().logoutUser(idSession);
	}
	
//Probado
	public static void createGame(UUID idSession, String nameGame, String description,
									ArrayList<String> players, ArrayList<Calendar> listDays,
									int turnTime, int defenseTime, int negotiationTime)
															throws RemoteException, 
															ConnectException,
															InvalidGameInfoException, 
															InvalidSessionException,
															NullInformationException,
															EmptyInformationException
															
	{
		GameInfo newgameCreated = new GameInfo(UUID.randomUUID(), nameGame,
				   description, players,
				   listDays, Constants.totalTerritories - 1,
				   turnTime, defenseTime, 
				   negotiationTime);
		
		Connection.getConnection().createGame(idSession, newgameCreated);
	}
	
	public static void resignGame(UUID idGame, UUID idSession)
	{
		/*Game game = ListGames.getGame(idGame);
		Connection.getConnection().resignGame(idSession, idGame,game.getPlayers(),
				game.getMap(),EventType.UnknownEvent);*/
	}
	
	public static void quitGame(UUID idGame, UUID idSession)
	{
		/*Game game = ListGames.getGame(idGame);
		Connection.getConnection().quitGame(idSession, idGame, game.getPlayers(),
				game.getMap(),EventType.UnknownEvent);*/
		
	}
	
	public static void endTurn(UUID idGame, UUID idSession)
	{
		/*Game game = ListGames.getGame(idGame);
		Connection.getConnection().endTurn(idSession, idGame, game.getPlayers(),
				game.getMap(), EventType.UnknownEvent);*/
	}
	
	public static void updateGameBuyTerritory(UUID idGame, UUID idSession)
	{
		/*Game game = ListGames.getGame(idGame);
		Connection.getConnection().updateGame(idSession, idGame, game.getPlayers(), 
				game.getMap(), EventType.BuyTerritoryEvent);*/
	}
	
	public static void updateGameBuyArsenal(UUID idGame, UUID idSession)
	{
		/*Game game = ListGames.getGame(idGame);
		Connection.getConnection().updateGame(idSession, idGame, game.getPlayers(), 
				game.getMap(), EventType.BuyArsenalEvent);*/
	}	
}