package ea.domain;

import java.net.ConnectException;
import java.rmi.RemoteException;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Iterator;
import java.util.UUID;

import domain.GameInfo;

import ea.communications.Client;
import ea.exceptions.EmptyInformationException;
import ea.exceptions.NullInformationException;
import exceptions.InvalidSessionException;

@SuppressWarnings("unused")
public class QueryGames 
{
	private ArrayList<GameInfo> allGamesInfo;
	private int separator;
	
	public QueryGames()
	{
		allGamesInfo = null;
		separator = -1;
	}
	
	private ArrayList<GameInfo> getAllGamesInfo()
	{
		if(this.allGamesInfo == null) 
			return new ArrayList<GameInfo>();
		return this.allGamesInfo;
	}
	
	public int getSeparator()
	{
		return separator;
	}
	
	public void queryGamesServer(UUID session) throws RemoteException, InvalidSessionException, 
											  ConnectException, NullInformationException, 
											  EmptyInformationException
	{
		this.allGamesInfo = Connection.listGames(session); 
	}
	
	public void sortGamesByPlayer(String namePlayer)throws NullInformationException
	{
		ArrayList<GameInfo> yourGames = new ArrayList<GameInfo>();
		ArrayList<GameInfo> restGames = new ArrayList<GameInfo>();
		ArrayList<GameInfo> ordenados = new ArrayList<GameInfo>();
		if(this.getAllGamesInfo()!=null){
			Iterator<GameInfo> iter = this.getAllGamesInfo().iterator();
			while(iter.hasNext())
			{
				GameInfo gameInfo = iter.next();
				if(gameInfo.getPlayers().contains(namePlayer))
					yourGames.add(gameInfo);
				else
					restGames.add(gameInfo);
			}
			iter = yourGames.iterator();
			while(iter.hasNext())
				ordenados.add(iter.next());
			
			separator = yourGames.size()-1;
			
			iter = restGames.iterator();
			while(iter.hasNext())
				ordenados.add(iter.next());
		}
		else 
			throw new NullInformationException("La lista de juegos es vaca, por lo que no se puede ordenar.");
		this.allGamesInfo = ordenados;
	}
	
	public ArrayList<UUID> getListGameId()throws NullInformationException
	{
		ArrayList<UUID> result = new ArrayList<UUID>();
		if(this.getAllGamesInfo()!=null){
			Iterator<GameInfo> iter = this.getAllGamesInfo().iterator(); 
			while(iter.hasNext()) {
				GameInfo gameInfo = iter.next(); 
				result.add(gameInfo.getId());
			}
		}
		else
			throw new NullInformationException("La lista de juegos es vaca, por lo que no hay idenditificadores.");
		return result;
	}
	
	public ArrayList<String> getListGamesName()throws NullInformationException
	{
		ArrayList<String> result = new ArrayList<String>();
		if(this.getAllGamesInfo()!=null){
			Iterator<GameInfo> iter = this.getAllGamesInfo().iterator(); 
			while(iter.hasNext()) {
				GameInfo gameInfo = iter.next(); 
				result.add(gameInfo.getName());
			}
		}
		else
			throw new NullInformationException("La lista de juegos es vaca, por lo que no hay nombres.");
		return result;
	}
	
	public ArrayList<String> getListGamesDescription()throws NullInformationException
	{

		ArrayList<String> result = new ArrayList<String>();
		if(this.getAllGamesInfo()!=null){
			Iterator<GameInfo> iter = this.getAllGamesInfo().iterator(); 
			while(iter.hasNext()) {
				GameInfo gameInfo = iter.next(); 
				result.add(gameInfo.getDescription());
			}
		}
		else
			throw new NullInformationException("La lista de juegos es vaca, por lo que no hay descripciones.");
		return result;
	} 
	
	public ArrayList<ArrayList<String>> getListGamesUsers()throws NullInformationException
	{
		ArrayList<ArrayList<String>> result = new ArrayList<ArrayList<String>>();
		if(this.getAllGamesInfo()!=null){
			Iterator<GameInfo> iter = this.getAllGamesInfo().iterator(); 
			while(iter.hasNext()) {
				GameInfo gameInfo = iter.next(); 
				result.add(gameInfo.getPlayers());
			}
		}
		else
			throw new NullInformationException("La lista de juegos es vaca, por lo que no hay lista de players.");
		return result;
	}

	public ArrayList<ArrayList<Calendar>> getGameSessions()throws NullInformationException
	{
		ArrayList<ArrayList<Calendar>> result = new ArrayList<ArrayList<Calendar>>(); 
		if(this.getAllGamesInfo()!=null){
			Iterator<GameInfo> iter = this.getAllGamesInfo().iterator(); 
			while(iter.hasNext()) {
				GameInfo gameInfo = iter.next(); 
				result.add(gameInfo.getGameSessions());
			}
		}
		else
			throw new NullInformationException("La lista de juegos es vaca, por lo que no hay tiempos de juego.");
		return result;
	}
}