package ea.domain;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.Random;
import java.util.UUID;

import domain.Player;
import domain.Spy;
import ea.libs.Constants;
import ea.presentation.Alert;
import ea.presentation.CommonConstants;

public class QuerySpies 
{
	private Player player;
	private ArrayList<InformationSpy> informations;
	private UUID idGame;
	public QuerySpies(UUID idGame,String namePlayer)
	{
		this.idGame = idGame;
		Player aux = new Player(namePlayer,-1);
		player = ListGames.getGame(idGame).getPlayers()
				.get(ListGames.getGame(idGame).getPlayers().indexOf(aux));
		
		createInformations();
	}
	
	private void createInformations()
	{
		ArrayList<Spy> newSpies = new ArrayList<Spy>();
		informations = new ArrayList<InformationSpy>();
		Iterator<Spy> listSpies = player.getSpies().iterator();
		while(listSpies.hasNext())
		{
			Spy aux = listSpies.next();
			if(aux.getUses() != 0)
			{// The next turn, if spy has success, then show information
				informations.add(new InformationSpy(idGame, aux, false));
				newSpies.add(aux);
			}
			//ELSE remove list of spies, because not uses to obtain information
		}
		player.setSpies(newSpies);
	}
	
	public void addNewSpy(int destinyTerritory)
	{
		Spy spy = new Spy(destinyTerritory,Constants.usesSpy);
		player.setMoney(player.getMoney()- Constants.priceSpy);
		player.getSpies().add(spy);
		InformationSpy in = new InformationSpy(idGame,spy,false);
		informations.add(in);
	}
	
	public void recalculateInformation()
	{
		ArrayList<Spy> resultSpies = new ArrayList<Spy>();
		ArrayList<InformationSpy> listSpiesResult = new ArrayList<InformationSpy>();
		Iterator<InformationSpy> iter = informations.iterator();
		while(iter.hasNext())
		{
			InformationSpy spy = iter.next();
			if(!spy.isSuccess())
			{
				Random rnd = new Random();
				int luck = (int)(rnd.nextDouble() * 2.0);
				if(luck == 0)
				{//Success
					spy.setSuccess(true);
					spy.decrementUse();
					spy.InformationObtainSpy();
				
					listSpiesResult.add(spy);
					resultSpies.add(spy.getSpy());
				}
				else
				{//Failure
					if(spy.getUsesSpy()==1)
					{//Finish, the spy has been killed in the attempt
						new Alert ("Aviso", "El espia con localizacin: "+ 
								Names.values()[spy.getLocationSpy()] + " ha muerto en el intento.",
								CommonConstants.ALERT_ICON);
					}
					else
					{
						spy.decrementUse();
						listSpiesResult.add(spy);
						resultSpies.add(spy.getSpy());
					}
				}
			}
			else
			{
				listSpiesResult.add(spy);
				resultSpies.add(spy.getSpy());
			}
		}
		informations = listSpiesResult;
		player.setSpies(resultSpies);
	}
	
	public int lenghtListInformation()
	{
		return informations.size();
	}
	
	public int getLocationSpy(int index)
	{
		return informations.get(index).getLocationSpy();
	}
	
	public int getUsesSpy(int index)
	{
		return informations.get(index).getUsesSpy();
	}
	
	public boolean getSuccessSpy(int index)
	{
		return informations.get(index).isSuccess();
	}
	
	public int getSoldiersSpy(int index)
	{
		if(informations.get(index).isSuccess())
			return informations.get(index).getSoldiers();
		else 
			return -1;
	}
	
	public int getMoneySpy(int index)
	{
		if(informations.get(index).isSuccess())
			return informations.get(index).getMoney();
		else 
			return -1;
	}
	
	public int getCannonsOneSpy(int index)
	{
		if(informations.get(index).isSuccess())
			return informations.get(index).getCannonsOneUse();
		else 
			return -1;
	}
	
	public int getCannonsTwoSpy(int index)
	{
		if(informations.get(index).isSuccess())
			return informations.get(index).getCannonsTwoUses();
		else 
			return -1;
	}
	
	public int getCannonsThreeSpy(int index)
	{
		if(informations.get(index).isSuccess())
			return informations.get(index).getCannonsThreeUses();
		else 
			return -1;
	}
	
	public int getMissilsSpy(int index)
	{
		if(informations.get(index).isSuccess())
			return informations.get(index).getMissils();
		else 
			return -1;
	}
	
	public int getICBMsSpy(int index)
	{
		if(informations.get(index).isSuccess())
			return informations.get(index).getICBMs();
		else 
			return -1;
	}
	
	public int getAntiMissilsSpy(int index)
	{
		if(informations.get(index).isSuccess())
			return informations.get(index).getAntimissils();
		else 
			return -1;
	}
	
	public ArrayList<InformationSpy> getInformations()
	{
		return informations;
	}
	
	public String getOwner(int index)
	{
		if(informations.get(index).isSuccess())
			return informations.get(index).getOwner();
		else 
			return "Error";
	}
}
