package ea.libs.graphs;

import ea.libs.lists.*;

@SuppressWarnings("hiding")
public class GrafoV<X,C> extends Grafo<X,C>{
	class VerticeL<X> extends Vertice<X>{
        private Lista<Arco<X,C>> adyacentes;//lista de arcos. Y es un arco
        public VerticeL(X o){
            super(o);
            adyacentes=new Lista<Arco<X,C>>();
        }
        public Lista<Arco<X,C>> adyacentes(){
            return adyacentes;
        }
        public void insAdyacente(Arco<X,C> a){
//if !adyacentes.pertenece(a)
            adyacentes.insertarFinal(a);
        }
    }//vertice
    class ArcoL<X,C> extends Arco<X,C>{
        public ArcoL(VerticeL<X> or,VerticeL<X> dest,C caracteristicas){
            super(or, dest,caracteristicas);
        }
        public ArcoL(X or,X dest,C p){
            this(new VerticeL<X>(or),new VerticeL<X>(dest),p);
        }
        public ArcoL(VerticeL<X> or,VerticeL<X> dest){
            super(or, dest);
        }
        public ArcoL(X or,X dest){
            this(new VerticeL<X>(or),new VerticeL<X>(dest));
        }
    }//arco
    /** Creates a new instance of grafoV */
    private Lista<Vertice<X>> vertices;//lista de vertices
    private Lista<Arco<X,C>> arcos;//lista de arcos
    public GrafoV(){
        this(false );
    }
    public GrafoV(boolean b){
        super(b);
        vertices=new Lista<Vertice<X>>();
        arcos=new Lista<Arco<X,C>>();
    }
    public boolean esVacio(){
        return vertices.esVacia();
    }
    public Lista<X> Vertices(){
        Lista<X> vert=new Lista<X> ();
        Lista<Vertice<X>> aux=vertices;
        while (!aux.esVacia()){
            vert.insertarFinal(aux.primero().identificador());
            aux=aux.resto();
        }
        return vert;
    }
    public Lista<Vertice<X>> vertices(){
        return vertices;
    }
    public void insertarVertice(Vertice<X> v){
        if (!perteneceVertice(v)) vertices.insertarFinal(new
                VerticeL<X>(v.identificador()));
    }
    public void insertarVertice(X v){
        insertarVertice(new VerticeL<X>(v));
    }
    public boolean perteneceVertice(X v){
        return super .perteneceVertice(new Vertice<X>(v));
    }
    public VerticeL<X> vertice(X v){//PRIVATE?
        return vertice(new VerticeL<X>(v));
    }
    private VerticeL<X> vertice(Vertice<X> v){/*Modificado a publico*/
        VerticeL<X> vale=null ;
        int pos=vertices.posicion(v);
        if (pos>0) vale=(VerticeL<X>)vertices.elementoN(pos);
        return vale;
    }
    public ArcoL<X,C> arco(X or,X des){
        return arco(new ArcoL<X,C>(or,des));
    }
    @SuppressWarnings("unused")
	private ArcoL<X,C> arco(Vertice<X> or,Vertice<X> des){
        return arco(new ArcoL<X,C>((VerticeL<X>)or,(VerticeL<X>)des));
    }
    private ArcoL<X,C> arco(Arco<X,C> a){
        ArcoL<X,C> vale=null ;
        int pos=arcos.posicion(a);
        if (pos>0) vale=(ArcoL<X,C>)arcos.elementoN(pos);
        return vale;
    }
    public Lista<Arco<X,C>> arcos(){
        return arcos;
    }
    public boolean perteneceArco(X or,X dest){
        return super .perteneceArco(new ArcoL<X,C>(or,dest));
    }
    public void insertarArco(X origen, X destino){
        VerticeL<X> orig=vertice(origen);
        VerticeL<X> dest=vertice(destino);
        if (orig!=null && dest!=null ){
            ArcoL<X,C> nuevoArco=new ArcoL<X,C>(orig,dest);
            if (!perteneceArco(nuevoArco)){
                arcos.insertarFinal(nuevoArco);
                orig.insAdyacente(nuevoArco);
                if (!dirigido()) {
                    nuevoArco=new ArcoL<X,C>(dest,orig);
                    arcos.insertarFinal(nuevoArco);
                    dest.insAdyacente(nuevoArco);
                }//if
            }//if
        }//if
    }
    public void insertarArco(X origen, X destino,C p){
        insertarArco(origen,destino);
        ArcoL<X,C> a=arco(origen,destino);
        if (a!=null ){
            a.nuevoPeso(p);
            if (!dirigido()) {
                a=arco(destino, origen);
                a.nuevoPeso(p);
            }//if
        }
    }
    public void insertarArco(Arco<X,C> a){
        if (a.peso()!=null )
            insertarArco(a.origen().identificador(),a.destino().
                    identificador(),a.peso());
        else
            insertarArco(a.origen().identificador(),a.destino().
                    identificador());
    }
    public Lista<X> adyacentes(X v){
        Lista<Vertice<X>> vAdy=adyacentes(new VerticeL<X>(v));
        Lista<X> ady=new Lista<X> ();
        while (!vAdy.esVacia()){
            ady.insertarFinal(vAdy.primero().identificador());
            vAdy=vAdy.resto();
        }
        return ady;
    }
    public Lista<Vertice<X>> adyacentes(Vertice<X> v){
        Lista<Vertice<X>> ady=new Lista<Vertice<X>>();
        VerticeL<X> aux=vertice(v);
        if (aux!=null ){
            Lista<Arco<X,C>> vAdy=aux.adyacentes();
            while (!vAdy.esVacia()){
                ady.insertarFinal(vAdy.primero().destino());
                vAdy=vAdy.resto();
            }
        }
        return ady;
    }
    public Lista<Arco<X,C>> arcosAdyacentes(Vertice<X> v){
        Lista<Arco<X,C>> adyacentes=new Lista<Arco<X,C>>();
        VerticeL<X> aux=vertice(v);
        if (aux!=null ) adyacentes=aux.adyacentes();
        return adyacentes;
    }
    
    @SuppressWarnings("rawtypes")
	public Lista<Vertice<X>> incidentes(Vertice<X> v){
        Lista<Vertice<X>> incidentes=new Lista<Vertice<X>>();
        VerticeL aux=vertice(v);
        if (aux!=null ){
            Lista<Arco<X,C>> arc=arcos();
            while (!arc.esVacia()){
                Arco<X,C> a=arc.primero();
                if (a.destino().equals(v)) incidentes.insertarFinal(a.
                        origen());
                arc=arc.resto();
            }
        }
        return incidentes;
    }//incidentes
    public Lista<X> incidentes(X v){
        Lista<Vertice<X>> aux=incidentes(new VerticeL<X> (v));
        Lista<X> inci=new Lista<X>();
        while (!aux.esVacia()){
            inci.insertarFinal(aux.primero().identificador());
            aux=aux.resto();
        }
        return inci;
    }//incidentes
    public Lista<Arco<X,C>> arcosIncidentes(Vertice<X> v){
        Lista<Arco<X,C>> incidentes=new Lista<Arco<X,C>>();
        VerticeL<X> aux=vertice(v);
        if (aux!=null ){
            Lista<Arco<X,C>> arc=arcos();
            while (!arc.esVacia()){
                Arco<X,C> a=arc.primero();
                if (a.origen().equals(v)) incidentes.insertarFinal(a);
                arc=arc.resto();
            }
        }
        return incidentes;
    }//incidentes
    public void eliminaArco(Arco<X,C> a){
        VerticeL<X> auxO=vertice(a.origen());
        VerticeL<X> auxD=vertice(a.destino());
        if (auxO!=null && auxD!=null ){
            auxO.adyacentes().elimina(a);
            arcos.elimina(a);
            if (!dirigido()){
                ArcoL<X,C> reves=new ArcoL<X,C>((VerticeL<X>)a.
                        destino(),(VerticeL<X>)a.origen());
                auxD.adyacentes().elimina(reves);
                arcos.elimina(reves);
            }
        }
    }
    public void eliminaArco(X origen,X destino){
        eliminaArco(new ArcoL<X,C>(origen,destino));
    }
    public void eliminaVertice(Vertice<X> v){
        if (perteneceVertice(v)){
            Lista<Arco<X,C>> arc=arcos();
            while (!arc.esVacia()){
                if (arc.primero().origen().equals(v) || (arc.primero().
                        destino().equals(v)))
                    eliminaArco(arc.primero());
                else arc=arc.resto();
            }
            vertices.elimina(v);
        }
    }
    @SuppressWarnings({ "rawtypes", "unchecked" })
	public void eliminaVertice(X v){
        eliminaVertice(new VerticeL(v));
    }
    public void nuevoPeso(X or,X dest, C p){
        ArcoL<X,C> n=arco(or,dest);
        if (n!=null ) n.nuevoPeso(p);
        if (!dirigido()){
            n=arco(dest,or);
            if (n!=null ) n.nuevoPeso(p);
        }
    }
    public C peso(X or,X dest){
        C p=null ;
        ArcoL<X,C> n=arco(or,dest);
        if (n!=null ) p=n.peso();
        return p;
    }
}//grafoNV
