package ea.presentation;

import com.trolltech.qt.core.Qt;
import com.trolltech.qt.gui.QDesktopWidget;
import com.trolltech.qt.gui.QHBoxLayout;
import com.trolltech.qt.gui.QIcon;
import com.trolltech.qt.gui.QLabel;
import com.trolltech.qt.gui.QPushButton;
import com.trolltech.qt.gui.QVBoxLayout;
import com.trolltech.qt.gui.QWidget;

public class Alert extends QWidget
{
	private QHBoxLayout buttonBox;
	private String title;
	private String content;
	private String windowsIcon;
	
	public Alert(String title, String content)
	{
		this.setWindowsIcon(CommonConstants.ALERT_ICON);
		this.setTitle(title);
		this.setContent(content);
		this.createMainWindow();
		this.createButtonAccept();
		this.createLabelContent();
		show();
	}
	
	public Alert(String title, String content, String icon)
	{
		this.setWindowsIcon(icon);
		this.setTitle(title);
		this.setContent(content);
		this.createMainWindow();
		this.createButtonAccept();
		this.createLabelContent();
		show();
	}
	
	private void setTitle(String title)
	{
		this.title = title;
	}
	
	private String getTitle()
	{
		return this.title;
	}
	
	private void setContent(String content)
	{
		this.content = content;
	}
	
	private String getContent()
	{
		return this.content;
	}
	
	private QHBoxLayout getLayoutButtonBox()
	{
		return this.buttonBox;
	}
	
	private void createMainWindow()
	{
		setWindowIcon(new QIcon(this.getWindowsIcon()));
		setWindowTitle(this.getTitle());
		this.adjustSize();
		QDesktopWidget qdw = new QDesktopWidget();
        int screen_center_width  = (qdw.width() - 250) / 2,
            screen_center_height = (qdw.height() - 100) / 2;
        setGeometry(screen_center_width, screen_center_height, 
        			250, 100);
        
        setWindowModality(Qt.WindowModality.ApplicationModal);
        setWindowFlags(Qt.WindowType.WindowCloseButtonHint);
        setStyleSheet("background-color:qlinearGradient(x1:0, y1:0, x2:0, y2:1, stop:0 #335577, " +
        		   "stop:1 #6688AA); color: #eeeeff");
	}
	
	private void createLabelContent()
	{
		QLabel lbl = new QLabel(this.getContent());
        lbl.setAlignment(Qt.AlignmentFlag.AlignLeft);
        lbl.setStyleSheet("background-color: #f95353;border-radius:5px;padding:8px");
        QVBoxLayout vbox = new QVBoxLayout(this);
        vbox.addStretch(1);
        vbox.addWidget(lbl, 1, Qt.AlignmentFlag.AlignLeft);
        vbox.addLayout(this.getLayoutButtonBox());
	}
	
	private void createButtonAccept()
	{
		QPushButton ok = new QPushButton("&Aceptar", this);
        ok.setIcon(new QIcon("img/ok.png"));
        ok.clicked.connect(this, "quit()");
        ok.setShortcut("A");
        buttonBox = new QHBoxLayout();
        buttonBox.addStretch(1);
        buttonBox.addWidget(ok, 1, Qt.AlignmentFlag.AlignCenter);
	}
	
	@SuppressWarnings("unused")
	private void quit()
	{
		this.close();
	}

	public void setWindowsIcon(String windowsIcon) {
		this.windowsIcon = windowsIcon;
	}

	public String getWindowsIcon() {
		return windowsIcon;
	}
}