package ea.presentation;

import java.net.ConnectException;
import java.rmi.RemoteException;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Iterator;
import java.util.UUID;

import com.trolltech.qt.core.QDate;
import com.trolltech.qt.core.Qt;
import com.trolltech.qt.gui.QBrush;
import com.trolltech.qt.gui.QCalendarWidget;
import com.trolltech.qt.gui.QColor;
import com.trolltech.qt.gui.QDesktopWidget;
import com.trolltech.qt.gui.QGroupBox;
import com.trolltech.qt.gui.QHBoxLayout;
import com.trolltech.qt.gui.QIcon;
import com.trolltech.qt.gui.QLabel;
import com.trolltech.qt.gui.QLineEdit;
import com.trolltech.qt.gui.QPushButton;
import com.trolltech.qt.gui.QTextCharFormat;
import com.trolltech.qt.gui.QTextEdit;
import com.trolltech.qt.gui.QVBoxLayout;
import com.trolltech.qt.gui.QWidget;

import domain.GameInfo;
import ea.domain.Connection;
import ea.exceptions.EmptyInformationException;
import ea.exceptions.FormatTimeOutException;
import ea.exceptions.NullInformationException;
import ea.libs.Constants;
import exceptions.InvalidGameInfoException;
import exceptions.InvalidSessionException;

public class CreateGame extends QWidget
{
	private String namePlayer; 
	private UUID session; 
	private QCalendarWidget datePicker; 
	private ArrayList<Calendar> listOfDays;
	private QLineEdit nameGame; 
	private QTextEdit descriptionBox; 
	private QLineEdit turnTimeBox; 
	private QLineEdit defenseTimeBox;
	private QLineEdit negociationTimeBox;
	private QLineEdit hourStartGameBox;  
	private QLineEdit minuteStartGameBox; 
	
    public CreateGame(UUID session, String namePlayer)
    {
    	this.namePlayer = namePlayer;
    	this.session = session;
    	this.listOfDays = new ArrayList<Calendar>();
        this.createMainWindow();

        QHBoxLayout name_layout = new QHBoxLayout();
        name_layout.addWidget(new QLabel("Nombre:"));
        nameGame = new QLineEdit();
        name_layout.addWidget(nameGame);
        
        QHBoxLayout buttons_layout = new QHBoxLayout();
        this.createButtonBack(buttons_layout);
        this.createButtonCreate(buttons_layout);
        this.createDatePicker();
        
        QHBoxLayout upLayout = new QHBoxLayout();
        upLayout.addLayout(name_layout);
        
        QLabel descriptionLabel = new QLabel("Descripcin:");
        descriptionBox = new QTextEdit();
        descriptionBox.setMinimumHeight(290);
        
        hourStartGameBox = new QLineEdit("12");
        hourStartGameBox.setFixedWidth(30);
        
        minuteStartGameBox = new QLineEdit("30");
        minuteStartGameBox.setFixedWidth(30);
        
        QHBoxLayout StartGamelayout = new QHBoxLayout();
        StartGamelayout.addWidget(new QLabel("Hora de comienzo:"), 0);
        StartGamelayout.addWidget(hourStartGameBox, 0, Qt.AlignmentFlag.AlignLeft);
        StartGamelayout.addWidget(new QLabel(":"), 50, Qt.AlignmentFlag.AlignLeft);
        StartGamelayout.addWidget(minuteStartGameBox, 950, Qt.AlignmentFlag.AlignLeft);
        
        QLabel turnTimeLabel = new QLabel("Tiempo de turno (segundos):");
        turnTimeBox = new QLineEdit("120");
        turnTimeBox.setFixedWidth(30);
        
        QHBoxLayout turnTimelayout = new QHBoxLayout();
        turnTimelayout.addWidget(turnTimeLabel, 0, Qt.AlignmentFlag.AlignLeft);
        turnTimelayout.addWidget(turnTimeBox, 0, Qt.AlignmentFlag.AlignLeft);
        
        QLabel defenseTimeLabel = new QLabel("Tiempo de defensa (segundos):");
        defenseTimeBox = new QLineEdit("30");
        defenseTimeBox.setFixedWidth(30);
        
        QHBoxLayout defenseTimelayout = new QHBoxLayout();
        defenseTimelayout.addWidget(defenseTimeLabel, 0, Qt.AlignmentFlag.AlignLeft);
        defenseTimelayout.addWidget(defenseTimeBox, 0, Qt.AlignmentFlag.AlignLeft);
        
        QLabel negociationTimeLabel = new QLabel("Tiempo de negociacin (segundos):");
        negociationTimeBox = new QLineEdit("30");
        negociationTimeBox.setFixedWidth(30);
        
        QHBoxLayout negociationTimelayout = new QHBoxLayout();
        negociationTimelayout.addWidget(negociationTimeLabel, 0, Qt.AlignmentFlag.AlignLeft);
        negociationTimelayout.addWidget(negociationTimeBox, 0, Qt.AlignmentFlag.AlignLeft);
        
        QVBoxLayout leftUpLayout = new QVBoxLayout();
        leftUpLayout.addWidget(descriptionLabel, 0, Qt.AlignmentFlag.AlignTop);
        leftUpLayout.addWidget(descriptionBox, 100, Qt.AlignmentFlag.AlignTop);
        leftUpLayout.addLayout(StartGamelayout, 0);
        leftUpLayout.addLayout(turnTimelayout, 1);
        leftUpLayout.addLayout(defenseTimelayout, 1);
        leftUpLayout.addLayout(negociationTimelayout, 1);

        QHBoxLayout leftLayout = new QHBoxLayout();
        leftLayout.setAlignment(new Qt.Alignment(Qt.AlignmentFlag.AlignTop));
        leftLayout.addLayout(leftUpLayout);
        
        QVBoxLayout datePicker_box = new QVBoxLayout();
        datePicker_box.addWidget(new QLabel("Calendario de juego:"));
        datePicker_box.addWidget(getDatePicker());

        QHBoxLayout centrallayout = new QHBoxLayout();
        centrallayout.addLayout(leftLayout, 0);
        centrallayout.addLayout(datePicker_box, 1);
        
        QVBoxLayout mainLayout = new QVBoxLayout();
        mainLayout.addLayout(upLayout, 0);
        mainLayout.addLayout(centrallayout, 1);
        mainLayout.addLayout(buttons_layout, 2);
        
        QGroupBox main_box = new QGroupBox("Crear una partida");
        main_box.setLayout(mainLayout);
        
        QHBoxLayout main_layout = new QHBoxLayout();
        main_layout.addWidget(main_box);
        
        setLayout(main_layout);
        this.show();
    }
 
    private String getDescription()throws EmptyInformationException
    {
    	if(descriptionBox.toPlainText().isEmpty())
    		throw new EmptyInformationException("La descripcin de la partida es vaca.");
    	
    	return this.descriptionBox.toPlainText();
    }
    
    private int getTurnTimeBox()throws FormatTimeOutException, EmptyInformationException
    {
    	if(this.turnTimeBox.text().isEmpty())
    		throw new EmptyInformationException("El tiempo del turno es vaco.");
    	
    	try
    	{
    		if(Integer.parseInt(this.turnTimeBox.text())<= 0)
    	    	throw new FormatTimeOutException("El tiempo de turno tiene que ser " +
    	    			"mayor que 0 segundos.");
    		
    		return Integer.parseInt(this.turnTimeBox.text());
    	}
    	catch(NumberFormatException e)
    	{
    		throw new FormatTimeOutException("El tiempo del turno no es un nmero.");
    	}
    }
    
    private int getDefenseTimeBox()throws FormatTimeOutException, EmptyInformationException
    {
    	if(this.defenseTimeBox.text().isEmpty())
    		throw new EmptyInformationException("El tiempo de defensa es vaco.");
    	
    	try
    	{
    		if(Integer.parseInt(this.defenseTimeBox.text())<= 0)
    	    	throw new FormatTimeOutException("El tiempo de defensa tiene que ser " +
    	    			"mayor que 0 segundos.");
    		
    		return Integer.parseInt(this.defenseTimeBox.text());
    	}
    	catch(NumberFormatException e)
    	{
    		throw new FormatTimeOutException("El tiempo de defensa no es un nmero.");
    	}
    }
    
    private int getNegociationTimeBox()throws FormatTimeOutException, EmptyInformationException
    {
    	if(this.negociationTimeBox.text().isEmpty())
    		throw new EmptyInformationException("El tiempo de negociacin es vaco.");
    	
    	try
    	{
    		if(Integer.parseInt(this.negociationTimeBox.text())<= 0)
    	    	throw new FormatTimeOutException("El tiempo de negociacin tiene que ser " +
    	    			"mayor que 0 segundos.");
    		
    		return Integer.parseInt(this.negociationTimeBox.text());
    	}
    	catch(NumberFormatException e)
    	{
    		throw new FormatTimeOutException("El tiempo de negociacin no es un nmero.");
    	}
    }
    
    private int getHourStartGameBox()throws FormatTimeOutException, EmptyInformationException
    {
    	if(this.hourStartGameBox.text().isEmpty())
    		throw new EmptyInformationException("La hora de inicio de ese da es vaca.");
    	
    	try
    	{	
    		if(!(Integer.parseInt(this.hourStartGameBox.text())>= 0 && 
    				Integer.parseInt(this.hourStartGameBox.text()) <= 23))
    	    	throw new FormatTimeOutException("La hora de inicio es incorrecta, " +
    	    			"debe estar comprendida entre 0 y 23.");
    		
    		return Integer.parseInt(this.hourStartGameBox.text());
    	}
    	catch(NumberFormatException e)
    	{
    		throw new FormatTimeOutException("Las horas de inicio de ese da no son un nmero.");
    	}
    }
    
    private int getMinuteStartGameBox()throws FormatTimeOutException, EmptyInformationException
    {
    	if(this.minuteStartGameBox.text().isEmpty())
    		throw new EmptyInformationException("Los minutos de inicio de ese da estn vacios.");
    	
    	try
    	{	
    		if(!(Integer.parseInt(this.minuteStartGameBox.text())>= 0 && 
    				Integer.parseInt(this.minuteStartGameBox.text()) <= 59))
    	    	throw new FormatTimeOutException("Los minutos de inicio s incorrecta, " +
    	    			"deben de ser comprendidos entre 0 y 59.");
    		
    		return Integer.parseInt(this.minuteStartGameBox.text());
    	}
    	catch(NumberFormatException e)
    	{
    		throw new FormatTimeOutException("Los minutos de inicio de ese da no son un nmero.");
    	}
    }

    private void createDatePicker()
    {
        datePicker = new QCalendarWidget();
        datePicker.setMinimumDate(QDate.currentDate());
        datePicker.setGridVisible(true);
        datePicker.clicked.connect(this, "onClickDatePicker(QDate)");
        datePicker.setDateTextFormat(QDate.currentDate(), getCellStyle("red"));
    }
    
    private QCalendarWidget getDatePicker()
    {
    	return this.datePicker;
    }
    
    private QTextCharFormat getCellStyle(String color)
    {
        QTextCharFormat fmt = new QTextCharFormat();        
        QBrush brBg = new QBrush();
        if(color.equals("green")) brBg.setColor(QColor.green);
        else if(color.equals("red")) brBg.setColor(QColor.red);
        else brBg.setColor(QColor.black);
        fmt.setBackground(brBg);	       
        fmt.setToolTip("Seleccionado");	      
    	return fmt;
    }
    
    private String getNamePlayer() throws NullInformationException, EmptyInformationException
    {
    	if(namePlayer == null)
    		throw new NullInformationException("El nombre del jugador es nulo.");
    	if(namePlayer.isEmpty())
    		throw new EmptyInformationException("El nombre del jugador es vaco.");
    	return namePlayer;
    }
    
    private UUID getSession() throws NullInformationException
    {
    	if(session == null)
    		throw new NullInformationException("El idenfiticador de la sesin es nulo.");
    	return session;
    }
    
    private String getNameGame()throws EmptyInformationException
    {
    	if(this.nameGame.text().isEmpty())
    		throw new EmptyInformationException("El nombre de la partida a crear es vaco.");
    	
    	return this.nameGame.text();
    }
    
    private ArrayList<Calendar> getListOfDays()
    {
    	return this.listOfDays;
    }
    
    private void createMainWindow()
    {
        setWindowTitle(CommonConstants.APP_NAME + " - Crear partida");

        QDesktopWidget qdw = new QDesktopWidget();
        int screen_center_width  = (qdw.width() - CommonConstants.DEFAULT_WIDTH) / 2,
            screen_center_height = (qdw.height() - CommonConstants.DEFAULT_HEIGHT) / 2;
        
        setGeometry(screen_center_width, screen_center_height, 
        		    CommonConstants.DEFAULT_WIDTH, CommonConstants.DEFAULT_HEIGHT);
        setFixedSize(CommonConstants.DEFAULT_WIDTH, CommonConstants.DEFAULT_HEIGHT);

        setWindowIcon(new QIcon(CommonConstants.APP_ICON));
        
        this.setStyleSheet("background-color:qlinearGradient(x1:0, y1:0, x2:0, y2:1, stop:0 #57360f, " +
		   				   "stop:1 #9f7340); color: #eeeeff; QLineEdit{background-color:#fffcca;border-radius:7px;" +
        				   "border:1px solid black;padding:3px;color:#836b23}");			   
    }
    
    private void createButtonBack(QHBoxLayout buttons_layout)
    {
    	QPushButton button = new QPushButton("&Volver", this);
    	button.setShortcut("V");
	    button.setIcon(new QIcon("img/back.png"));
    	button.clicked.connect(this, "onClickBack()"); 
    	button.setMaximumWidth(300);
    	buttons_layout.addWidget(button, 0, Qt.AlignmentFlag.AlignRight);
    }
    
    @SuppressWarnings("unused")
	private void onClickBack()
    {
    	try
    	{
    		MenuGames menu_games = new MenuGames(this.getSession(),this.getNamePlayer());
    		this.close();
    	}
    	catch(NullInformationException e){
    		new Alert("Error","Causa:\n"+ e.getMessage());
    	}
    	catch(EmptyInformationException e){
    		new Alert("Error","Causa:\n"+ e.getMessage());
    	}
    }
    
    private void createButtonCreate(QHBoxLayout buttons_layout)
    {
    	QPushButton button = new QPushButton("&Crear", this);
    	button.setShortcut("C");
    	button.clicked.connect(this, "onClickCreate()"); 
    	button.setIcon(new QIcon(CommonConstants.CREATE_ICON));
    	button.setMaximumWidth(300);
    	buttons_layout.addWidget(button, 0, Qt.AlignmentFlag.AlignLeft);
    }
    
    @SuppressWarnings("unused")
	private void onClickCreate()
    {
    	try{
    		String description = getDescription();
    		int turnTime = getTurnTimeBox(); // On seconds
    		int defenseTime = getDefenseTimeBox(); // On seconds
    		int negotiationTime = getNegociationTimeBox(); // On seconds
    		String nameGametext = getNameGame();
    	
    		if(this.getListOfDays().size() == 0)
    		{
    			new Alert("Error", "Debe elegir al menos un da de juego para jugar la partida.");
    			return;
    		}
    
    		ArrayList<String> players = new ArrayList<String>();
    		players.add(this.getNamePlayer());
    		try
    		{	
    			Connection.createGame(getSession(),nameGametext, description, players,
    					getListOfDays(), turnTime, defenseTime, negotiationTime);
    			
    			new Alert("xito", "La partida se ha creado con xito.", CommonConstants.SUCCESS_ICON);
    			MenuGames list = new MenuGames(this.getSession(), this.getNamePlayer());
    			list.show();
    			this.close();
    		}
    		catch(InvalidGameInfoException  e)
    		{
    			new Alert("Error", "La informacin cuando se ha creado la partida es incorrecta.");
    		}
    		catch(InvalidSessionException  e)
    		{
    			new Alert("Error", "La sesin no es vlida o esta expirada.");
    		}
    		catch(RemoteException e)
    		{
    			new Alert("Error","Causa:\n"+ e.getMessage());
    		} 
    		catch (ConnectException e) 
    		{
    			new Alert("Error",             			
					  "Conexin rechazada.\nException:\n" + e.getLocalizedMessage() +
        			  "\n\nCausa:\n" + e.getCause());
    		}
    	}
    	catch (FormatTimeOutException e)
    	{
    		new Alert("Error","Causa:\n"+ e.getMessage());
    	}
    	catch (EmptyInformationException e)
    	{
    		new Alert("Error","Causa:\n"+ e.getMessage());
    	}
    	catch(NullInformationException e){
    		new Alert("Error","Causa:\n"+ e.getMessage());
    	}
    }
    
    @SuppressWarnings("unused")
	private void onClickDatePicker(QDate date)
    {
    	try{
    		int startHour = getHourStartGameBox();
    		int startMinute = getMinuteStartGameBox();
    		if(CommonConstants.DEBUG) 
    			System.out.println("Hora de creacin de partida: " + startHour);
    	
    		
    		Calendar newDay = Calendar.getInstance();
    												//		QDate	  //  Calendar
    		//QDate month is one unit more than Calendar; January = 1 // January = 0
    		newDay.set(date.year(), date.month()-1, date.day(), startHour, startMinute);
    	
    	
    		boolean ok = true; // [FIXME] This NOT WORK PROPERLY (also contains method). This should remove object from the list if it is clicked again a date
    	
    		Iterator<Calendar> iter = getListOfDays().iterator();
    		while(iter.hasNext())
    		{
    			Calendar cal = iter.next();
    			if(cal.get(Calendar.DATE) == newDay.get(Calendar.DATE) && cal.get(Calendar.YEAR) == newDay.get(Calendar.YEAR)&& cal.get(Calendar.MONTH) == newDay.get(Calendar.MONTH))
    			{
    				iter.remove();
    				getDatePicker().setDateTextFormat(date, getCellStyle("black"));
    				ok = false;
    			}
    		}

    		if(ok)
    		{
    			getDatePicker().setDateTextFormat(date, getCellStyle("green"));
    			this.getListOfDays().add(newDay);
    		}
    	}
    	catch (FormatTimeOutException e)
    	{
    		new Alert("Error","Causa:\n"+ e.getMessage());
    	}
    	catch (EmptyInformationException e)
    	{
    		new Alert("Error","Causa:\n"+ e.getMessage());
    	}
    }
}