	package ea.presentation;

import java.rmi.RemoteException;
import java.util.Iterator;
import java.util.UUID;

import com.trolltech.qt.core.QCoreApplication;
import com.trolltech.qt.core.QSize;
import com.trolltech.qt.core.QTimer;
import com.trolltech.qt.core.Qt;
import com.trolltech.qt.gui.QAction;
import com.trolltech.qt.gui.QApplication;
import com.trolltech.qt.gui.QCloseEvent;
import com.trolltech.qt.gui.QDesktopWidget;
import com.trolltech.qt.gui.QGroupBox;
import com.trolltech.qt.gui.QHBoxLayout;
import com.trolltech.qt.gui.QIcon;
import com.trolltech.qt.gui.QKeyEvent;
import com.trolltech.qt.gui.QLabel;
import com.trolltech.qt.gui.QMenu;
import com.trolltech.qt.gui.QMenuBar;
import com.trolltech.qt.gui.QMouseEvent;
import com.trolltech.qt.gui.QPixmap;
import com.trolltech.qt.gui.QPushButton;
import com.trolltech.qt.gui.QStandardItemModel;
import com.trolltech.qt.gui.QTreeView;
import com.trolltech.qt.gui.QVBoxLayout;
import com.trolltech.qt.gui.QWidget;

import ea.domain.Connection;
import ea.domain.QueryMoney;
import ea.domain.QueryPlayers;
import ea.domain.QuerySpies;
import ea.libs.Constants;
import ea.presentation.CommonConstants;
import exceptions.GameNotFoundException;
import exceptions.InvalidSessionException;
import exceptions.InvalidTimeException;

public class Match extends QWidget implements Runnable
{
	private StateAction pendingAction;
	private UUID idGame;
	private String player;
	private boolean hasTurn;

    private boolean quitSystem = true;
	private UUID session;
	private QLabel updateInfoGame;
	private QMap mapGame;
    private QStandardItemModel modelPlayers;
	private QPushButton buttonCancelPendingAction;
	private QLabel moneyAvailable;
	private QLabel counterLabel;
	private int counter = 1;
	private QTimer timer;
	private QuerySpies manageSpies;

	public Match(UUID session, UUID game, String player) throws RemoteException 
	{
		this.player = player;
		this.idGame = game;
		this.session = session;
		this.updateInfoGame = new QLabel(this);
		this.mapGame = new QMap(this.idGame, this);
		this.manageSpies = new QuerySpies(this.idGame,this.player);
		
		this.createWindow();   
		this.show();
	}
	
	private void obtainInformationSpies()
	{
		//Method invoked when server set Turn
		this.manageSpies.recalculateInformation();
	}
	
	public QuerySpies getInformationSpies()
	{
		return this.manageSpies;
	}
	
	public StateAction getPendingAction()
	{
		return pendingAction;
	}
	
	public void setPendingAction(StateAction pendingAction)
	{
		this.pendingAction = pendingAction;
	}
	
    public void setQuitSystem(boolean quitSystem) 
    {
		this.quitSystem = quitSystem;
	}

	public boolean getQuitSystem() 
	{
		return quitSystem;
	}
	
	public String getPlayerName()
	{
		return this.player;
	}

	public QMap getMapGame()
	{
		return this.mapGame;
	}	

	private void createWindow() 
	{
		setWindowTitle(CommonConstants.APP_NAME + " - Partida");
    	setWindowIcon(new QIcon(CommonConstants.APP_ICON));
        setStyleSheet("background-color:qlinearGradient(x1:0, y1:0, x2:0, y2:1, stop:0 #57360f, " +
				      "stop:1 #9f7340); color: #eeeeff; QLineEdit{background-color:#fffcca;border-radius:7px;" +
				      "border:1px solid black;padding:3px;color:#836b23}");
 		setGeometry(3, 30, 0, 0);
     
        QDesktopWidget qdw = new QDesktopWidget();
        setFixedSize(qdw.availableGeometry().width() - 5, qdw.availableGeometry().height() - 33);
     
        createPanels();
        initializeListPlayers();
    }
	
	private void createPanels()
	{
		QHBoxLayout leftPanel = new QHBoxLayout();
		leftPanel.addWidget(createBoxMap());

		QVBoxLayout rigthPanel = new QVBoxLayout();

		rigthPanel.addWidget(createBoxInfoGame(), 1);

		QHBoxLayout middlePanel = new QHBoxLayout();

		middlePanel.addLayout(leftPanel, 1);
		middlePanel.addLayout(rigthPanel, 2);

		QHBoxLayout upPanel = new QHBoxLayout();
		upPanel.addWidget(createMenuBar());

		QVBoxLayout downPanel = new QVBoxLayout();
		downPanel.addWidget(createActionsBox());

        QVBoxLayout centralwidget = new QVBoxLayout();
        centralwidget.addLayout(upPanel, 0);
        centralwidget.addLayout(middlePanel, 1);
        centralwidget.addLayout(downPanel, 2);  
        
    	setLayout(centralwidget);
	}
		
	private QGroupBox createBoxMap() 
	{
		QHBoxLayout mapLayout = new QHBoxLayout();
		mapLayout.addWidget(this.mapGame);
		mapLayout.addStretch(1);

		QGroupBox groupBoxMap = new QGroupBox();
        groupBoxMap.setTitle("Mapa");
        groupBoxMap.setLayout(mapLayout);
        return groupBoxMap;
	}
	
	private QGroupBox createActionsBox()
	{
        QHBoxLayout actionsLayout = new QHBoxLayout();
        actionsLayout.addWidget(createButtonListSpy(), 0, Qt.AlignmentFlag.AlignLeft);
        actionsLayout.addWidget(createButtonBuySpy(), 0, Qt.AlignmentFlag.AlignLeft);
        actionsLayout.addWidget(createButtonBuyTerritory(), 0, Qt.AlignmentFlag.AlignLeft);
        actionsLayout.addWidget(createButtonResources(), 0, Qt.AlignmentFlag.AlignLeft);
        actionsLayout.addWidget(createButtonAttack(), 0, Qt.AlignmentFlag.AlignLeft);
        actionsLayout.addWidget(createButtonSendMisil(), 0, Qt.AlignmentFlag.AlignLeft);
        actionsLayout.addWidget(createButtonSendICBM(), 0, Qt.AlignmentFlag.AlignLeft);        
        actionsLayout.addWidget(createButtonMove(), 0, Qt.AlignmentFlag.AlignLeft);
        actionsLayout.addWidget(createButtonEndTurn(), 0, Qt.AlignmentFlag.AlignLeft);
        actionsLayout.addWidget(createButtonResign(), 0, Qt.AlignmentFlag.AlignLeft);
        actionsLayout.addWidget(createButtonExit(), 1, Qt.AlignmentFlag.AlignLeft);
        
        createTimer(); 
        
	    QHBoxLayout downLayout = new QHBoxLayout();
	    downLayout.addLayout(actionsLayout, 1);
	    downLayout.addWidget(this.counterLabel, 0);
    
        QGroupBox actionBox = new QGroupBox("Acciones");
        actionBox.setLayout(downLayout);

        return actionBox;
	}
	
	private void createTimer()
	{
	    this.timer = new QTimer();
	    timer.timeout.connect(this, "onClockTick()");
	    timer.start(1000);
	     
	    this.counterLabel = new QLabel();
	    onClockTick();
	}
	
	private void initializeListPlayers()
	{
		QueryPlayers query = new QueryPlayers(idGame);
		int position = query.posTurnPlayer();
		hasTurn = query.ishasMyTurn(this.getPlayerName());
		Iterator<String> i = query.getListPlayersOnline().iterator();
		int index = 0;
		while(i.hasNext())
		{
			//if(position == index)//Has turn, put subray
			// Ponerle un subrrayado	
			//else
				addDataModelPlayers(i.next());
			index++;
		}
	}
	
	private boolean getHasTurn()
	{
		return hasTurn;
	}
	
	private QGroupBox createBoxInfoGame()
	{
		
		 getUpdateInfoGame().setAlignment(Qt.AlignmentFlag.AlignTop);
		 setUpdateInfoGame("Pulsa alguna accin.");
		 
		 this.buttonCancelPendingAction = createButtonCancelPendingAction();
		 this.buttonCancelPendingAction.hide();
		 
		 createModelPlayers();
		 
	     QTreeView treePlayers = new QTreeView();
	     treePlayers.setStyleSheet("background-color:#b8946a;border-radius:2px;" + 
	        			       			"border:1px solid black;color:#836b23");
	     treePlayers.setModel(getModelPlayers());
	     treePlayers.setMaximumHeight(200);
	     
	     this.moneyAvailable = new QLabel(this.getMoneyAvailable());
	     this.moneyAvailable.setAlignment(Qt.AlignmentFlag.AlignRight);
	     this.moneyAvailable.setStyleSheet("font-size:20px;background-color:black;border-radius:10px;" + 
			"border:1px solid green;color:green");
		 
		 QVBoxLayout infoLayout = new QVBoxLayout();
		 infoLayout.addWidget(getUpdateInfoGame(), 1, Qt.AlignmentFlag.AlignTop);
		 infoLayout.addWidget(this.buttonCancelPendingAction, 0, Qt.AlignmentFlag.AlignTop);
		 infoLayout.addWidget(treePlayers, 0);
		 infoLayout.addWidget(this.moneyAvailable, 0);
		 
		 QGroupBox groupBoxInfoGame = new QGroupBox("Informacin de la partida");
		 groupBoxInfoGame.setLayout(infoLayout);
		 return groupBoxInfoGame;
	}
	
	@SuppressWarnings("unused")
	private void onClockTick()
	{
		//getRemainTime() //Negociaton or defense?
		int remainTime = 33; //[TODO] This need precalculate the correct image
		counter = (counter + 1) % remainTime;
		//System.out.println(counter);
		
		//Runtime.getRuntime().gc(); // Invoke the garbage collector for free memory
		
		QPixmap img = new QPixmap("img/counter/" + String.valueOf(this.counter + 1) + ".png");
		if(!img.isNull())
		{
			img = img.scaled(280, 50);
			if(!img.isNull()) counterLabel.setPixmap(img);
		}
		else new Alert("Error fatal", "Memoria insuficiente para cargar la imagen del temporizador.");
		
		counterLabel.setMaximumSize(280, 50);
		counterLabel.show();
		counterLabel.pixmap().dispose();
	}
	
	private void stopTimer()
	{
		timer.stop();
	}
	
	private void startTimer()
	{
		timer.start();
	}
	
	private void setLabelMoneyAvailaible(String data)
	{
		this.moneyAvailable.setText(data);
		this.show();
	}
	
	private String getMoneyAvailable()
	{
		QueryPlayers player = new QueryPlayers(this.getIdGame());
		return String.valueOf(player.getMoney(this.getPlayerName()));
	}
	
	private void createModelPlayers()
    {
        modelPlayers = new QStandardItemModel(0, 1);
        modelPlayers.setHeaderData(0, Qt.Orientation.Horizontal, "Jugadores");
    }
    
    private QStandardItemModel getModelPlayers()
    {
    	return this.modelPlayers;
    }
    
    private void addDataModelPlayers(String player) 
    {
    	QStandardItemModel model = getModelPlayers();
    	model.insertRow(model.rowCount());
    	model.setData(model.rowCount() - 1, 0, player);
    }

    private QPushButton sButton(String shortcut, String icon, String callback)
    {   // Create a simple button(sButton) for actions
        QPushButton button = new QPushButton("");
        button.setFixedSize(40, 40);
        button.setShortcut(shortcut);
        button.setIconSize(new QSize(40, 40));
        button.setIcon(new QIcon(icon));
        button.clicked.connect(this, callback);
    	return button;
    }
    
    private QPushButton createButtonAttack()
    {
    	return sButton("A", CommonConstants.ATTACK_ICON, "onClickAttack()");
    }
    
    private QPushButton createButtonCancelPendingAction()
    {
    	return sButton("C", CommonConstants.CROSS_ICON, "onClickCancelAction()");
    }

    private QPushButton createButtonBuySpy()
    {
    	return sButton("M", CommonConstants.BUY_SPY_ICON, "onClickBuySpy()");
    }
    
    private QPushButton createButtonBuyTerritory()
    {
    	return sButton("T", CommonConstants.BUY_TERRITORY_ICON, "onClickBuyTerritory()");
    }
    
    private QPushButton createButtonListSpy()
    {
    	return sButton("L", CommonConstants.LIST_SPY_ICON, "onClickListSpy()");
    }
    
    private QPushButton createButtonSendMisil()
    {
    	return sButton("I", CommonConstants.SEND_MISSILE, "onClickSendMissile()");
    }
    
    private QPushButton createButtonSendICBM()
    {
    	return sButton("C", CommonConstants.SEND_ICBM, "onClickSendICBM()");
    }    
    
    private QPushButton createButtonResources()
    {
    	return sButton("R", CommonConstants.BUY_ARSENAL, "onClickResources()");
    }
    
    private QPushButton createButtonMove()
    {
    	return sButton("O", CommonConstants.MOVE_ICON, "onClickMove()");
    }
    
    private QPushButton createButtonEndTurn()
    {
    	return sButton("T", CommonConstants.END_TURN_ICON, "actionEndTurnClicked()");
    }
    
    private QPushButton createButtonResign()
    {
    	return sButton("R", CommonConstants.FLAG_WHITE_ICON, "actionResignClicked()");
    }
    
    private QPushButton createButtonExit()
    {
    	return sButton("X", CommonConstants.EXIT_ICON, "actionExitClicked()");
    }
    
    @SuppressWarnings("unused")
	private void onClickAttack()
	{
    	if(this.getHasTurn()){
    		if(pendingAction == null)
    		{ //Not there action pending
    			pendingAction = new StateAction(ea.libs.Constants.action.Attack);
    			setUpdateInfoGame("1 - Selecciona un territorio desde donde va\n a enviar el ataque.");		
    			this.buttonCancelPendingAction.show();
    		}
    		else
    			new Alert("Aviso", "Debe de cancelar la accin en curso antes de comenzar otra.");
    	}
    	else
    		new Alert("Aviso", "Para realizar un ataque debe de esperar su turno.");
    }
    
    @SuppressWarnings("unused")
	private void onClickBuySpy()
    {
    	if(this.getHasTurn()){
    		if(pendingAction == null)
    		{ //Not there action pending
    			QueryMoney condition = new QueryMoney(this.getIdGame(),this.getPlayerName());
    			if(condition.hasMoneyForBuySpy()){
    				pendingAction = new StateAction(ea.libs.Constants.action.BuySpy);
    				setUpdateInfoGame("1 - Selecciona un territorio desde donde va\n a enviar el espa.");
    				this.buttonCancelPendingAction.show();
    			}
    			else
    				new Alert("Aviso", "No tiene suficiente dinero para comprar un espa.");
    		}
    		else
    			new Alert("Aviso", "Debe de cancelar la accin en curso antes de comenzar otra.");
    	}
    	else
    		new Alert("Aviso","Para realizar la compra de un espa debe de esperar su turno.");
    }
    
    @SuppressWarnings("unused")
	private void onClickBuyTerritory()
    {
    	if(this.getHasTurn())
    	{
    		if(pendingAction == null)
    		{ //Not there action pending
    			pendingAction = new StateAction(ea.libs.Constants.action.BuyTerritory);
    			setUpdateInfoGame("1 - Selecciona el territorio propio,\n a partir del cul va a realizar la compra.");

    			this.buttonCancelPendingAction.show();
    		}
    		else
    			new Alert("Aviso", "Debe de cancelar la accin en curso antes de comenzar otra.");
    	}
    	else
    		new Alert("Aviso", "Para realizar la compra de un territorio debe de esperar su turno.");
    }
    
	@SuppressWarnings("unused")
	private void onClickResources()
    {
		if(this.getHasTurn()){
			if(pendingAction == null)
			{ //Not there action pending
				QueryPlayers condition = new QueryPlayers(this.getIdGame());
				if(condition.getMoney(this.getPlayerName())> Constants.priceSoldier)
				{
					pendingAction = new StateAction(ea.libs.Constants.action.BuyArsenal);
					setUpdateInfoGame("1 - Selecciona un territorio para comprarle\n arsenal.");
					this.buttonCancelPendingAction.show();
				}
				else
					new Alert("Aviso", "No tiene dinero suficiente para realizar una compra.");
			}
			else
				new Alert("Aviso", "Debe de cancelar la accin en curso antes de comenzar otra.");
		}
		else
			new Alert("Aviso","Para realizar la compra de arsenal a territorio debe esperar su turno.");
    }
	
	@SuppressWarnings("unused")
	private void onClickListSpy()
    {
    	if(pendingAction == null)
		{ //Not there action pending
    		new MenuListSpy(this.getInformationSpies());
		}
		else
			new Alert("Aviso", "Debe de cancelar la accin en curso antes de comenzar otra.");
    }
	
	@SuppressWarnings("unused")
	private void onClickSendMissile()
	{
    	if(this.getHasTurn()){
    		if(pendingAction == null)
    		{ //Not there action pending
    			pendingAction = new StateAction(ea.libs.Constants.action.SendMissile);
    			setUpdateInfoGame("1 - Selecciona un territorio desde donde va\n a enviar el misil.");		
    			this.buttonCancelPendingAction.show();
    		}
    		else
    			new Alert("Aviso", "Debe de cancelar la accin en curso antes de comenzar otra.");
    	}
    	else
    		new Alert("Aviso", "Para realizar un ataque debe de esperar su turno.");
	}
	
	@SuppressWarnings("unused")
	private void onClickSendICBM()
	{
    	if(this.getHasTurn()){
    		if(pendingAction == null)
    		{ //Not there action pending
    			pendingAction = new StateAction(ea.libs.Constants.action.SendICBM);
    			setUpdateInfoGame("1 - Selecciona un territorio desde donde va\n a enviar el misil ICBM.");		
    			this.buttonCancelPendingAction.show();
    		}
    		else
    			new Alert("Aviso", "Debe de cancelar la accin en curso antes de comenzar otra.");
    	}
    	else
    		new Alert("Aviso", "Para realizar un ataque debe de esperar su turno.");
	}
    
    @SuppressWarnings("unused")
	private void onClickMove()
    {
    	if(this.getHasTurn()){
    		if(pendingAction == null)
    		{ //Not there action pending
    			pendingAction = new StateAction(ea.libs.Constants.action.Move);
    			setUpdateInfoGame("1 - Selecciona un territorio del que desea\n mover tropas.");  
    			this.buttonCancelPendingAction.show();
    		}
    		else
    			new Alert("Aviso", "Debe de cancelar la accin en curso antes de comenzar otra.");
    	}
    	else
    		new Alert("Aviso", "Para realizar el movimiento de tropas debe esperar su turno.");
    }
    
	public void onClickCancelAction()
	{	//This method only invoked when action current
		this.getMapGame().resetMap();
		this.getMapGame().resetCoffers();
		this.getMapGame().initializeTerritories();
		
		pendingAction = null; //Delete action pending
		this.buttonCancelPendingAction.hide();
		setUpdateInfoGame("Pulsa alguna accin.");
	}
	
	@SuppressWarnings("unused")
	private void actionEndTurnClicked() throws RemoteException, Exception
    {
		this.obtainInformationSpies();
		if(this.getHasTurn()){
			if(pendingAction == null)
			{ //Not there action pending
				Connection.endTurn(this.getIdGame(), this.getSession());
				new Alert("xito", "Acaba de pasar su turno.",CommonConstants.SUCCESS_ICON);
			}
			else
				new Alert("Aviso", "Debe de cancelar la accin en curso antes de comenzar otra.");
		}
		else
			new Alert("Aviso", "Para terminar el turno debe de esperar a tenerlo.");
    }

	public UUID getSession() 
	{
		return this.session;
	}

	private QMenuBar createMenuBar() 
	{
    	QMenuBar menubar = new QMenuBar();
    	  
        QMenu menuGame = new QMenu("Men Partida", menubar);

    	QAction actionEndTurn = new QAction(new QIcon(CommonConstants.END_TURN_ICON), "Finalizar Turno", this);
    	actionEndTurn.triggered.connect(this, "actionEndTurnClicked()");
        
    	QAction actionResign = new QAction(new QIcon(CommonConstants.FLAG_WHITE_ICON), "Abandonar partida", this);
        actionResign.triggered.connect(this, "actionResignClicked()");
        
    	QAction actionExit = new QAction(new QIcon(CommonConstants.EXIT_ICON), "&Salir", this);
        actionExit.triggered.connect(this, "actionExitClicked()");
        
        menuGame.addAction(actionEndTurn);
        menuGame.addAction(actionResign);
        menuGame.addAction(actionExit);
        menubar.addAction(menuGame.menuAction());  
        return menubar;
	}
    
    @Override
    protected void mousePressEvent(QMouseEvent e)
    {
    	if(CommonConstants.DEBUG)
    		System.out.println("Evento:" + e.type() + " " + e.button() + " " + e.buttons());
    }
    
    @SuppressWarnings("unused")
	private void actionResignClicked() throws RemoteException, GameNotFoundException, 
											  InvalidSessionException
    {
    	if(this.getHasTurn())
    	{
    		Connection.resignGame(getIdGame(),getSession());
    		this.setQuitSystem(false);
    		this.close();
    		this.setQuitSystem(true);
    	}
    	else
    		new Alert("Aviso", "Para abandonar la partida debe de tener su turno.");
    }

	@SuppressWarnings("unused")
	private void actionExitClicked() throws RemoteException, GameNotFoundException, 
											InvalidSessionException, InvalidTimeException
    {
		if(this.getHasTurn())
		{
			Connection.quitGame(getIdGame(), getSession());
			this.setQuitSystem(false);
			this.close();
		}
		else
			new Alert("Aviso", "Para salir de la partida debe de tener su turno.\n Por motivos de sincronizacin.");
    }
    
    @Override
    public void closeEvent(QCloseEvent event)
    {
    	if(CommonConstants.DEBUG) System.out.println("Closing window Match");
    	if(this.isWindow()) 
    	{	
    		this.close();
    		if(this.getQuitSystem())
    		{
				QCoreApplication.exit();
				QApplication.instance().dispose();
		    	QApplication.quit();
		    	System.exit(0);
    		}
    	}
    	else event.ignore();
    }
    
	protected void keyPressEvent(QKeyEvent e)
	{
	  	if(CommonConstants.DEBUG)
    		System.out.println("Match Evento Key:" + e.type() + " " + e.toString() + e.modifiers() + e.key() + " " + e.text());
	}
    
    public UUID getIdGame()
    {
    	return idGame;
    }

	@Override
	public boolean equals(Object obj)
	{
		if(this == obj) return true;
		if(!super.equals(obj)) return false;
		if (getClass() != obj.getClass()) return false;
		final Match other = (Match) obj;
		if (idGame == null) 
		{
			if (other.idGame != null) return false;
		} 
		else if (!idGame.equals(other.idGame)) return false;
		return true;
	}

	public void setUpdateInfoGame(String string) 
	{
		this.updateInfoGame.setText(string);
	}

	public QLabel getUpdateInfoGame() 
	{
		return updateInfoGame;
	}

	@Override
	public void run() {
		// TODO Auto-generated method stub
		
	}

	private void setSession(UUID session) {
		this.session = session;
	}
	
	public void resolveAttack()
	{
		//Llamar a la funcion del MAP, para resolver el ataque.
	}
	
	public void finishTurn()
	{
		hasTurn = false;
		//Tambin tendra que hacer un update del juego al servidor, y listo
	}
	
	public void changedTurn()
	{
		//Metodo, que hara cerrar todo, hacer un reset de la interfaz de acciones pendientes
		//y actualizar el listado de player, con nuevos usuarios, y el nuevo turno
		
	}
	
	public void negotiationResolve(int idTerritoryDst, int idTerritorySrc,int money, int soldiers)
	{
		NegotiationNotify negotiation = new NegotiationNotify(this,idTerritoryDst,idTerritorySrc,money,soldiers);
	}
	
	public void buySpy()
	{
		this.manageSpies.addNewSpy(this.getPendingAction().getDestinyTerritory());
		this.setLabelMoneyAvailaible(
				String.valueOf(new QueryPlayers(this.idGame).getMoney(this.player)));
	}
	
	public void buyArsenal()
	{
		this.setLabelMoneyAvailaible(
				String.valueOf(new QueryPlayers(this.idGame).getMoney(this.player)));
	}
	
	public void buyTerritory()
	{
		this.setLabelMoneyAvailaible(
				String.valueOf(new QueryPlayers(this.idGame).getMoney(this.player)));
	}
}
