package ea.presentation;

import com.trolltech.qt.core.Qt;
import com.trolltech.qt.gui.QCloseEvent;
import com.trolltech.qt.gui.QDesktopWidget;
import com.trolltech.qt.gui.QGridLayout;
import com.trolltech.qt.gui.QGroupBox;
import com.trolltech.qt.gui.QHBoxLayout;
import com.trolltech.qt.gui.QIcon;
import com.trolltech.qt.gui.QLabel;
import com.trolltech.qt.gui.QPushButton;
import com.trolltech.qt.gui.QSpinBox;
import com.trolltech.qt.gui.QVBoxLayout;
import com.trolltech.qt.gui.QWidget;

import ea.domain.Connection;
import ea.domain.Names;
import ea.domain.QueryMoney;
import ea.domain.QueryPlayers;
import ea.libs.Constants;

public class MenuBuy extends QWidget
{
		private Match father;
		
		private QLabel moneyAvailable;
		private QLabel numSoldiersTerritory; 
		private QLabel numCannonsTerritoryOneUse; 
		private QLabel numCannonsTerritoryTwoUses; 
		private QLabel numCannonsTerritoryThreeUses; 
		private QLabel numMissilesTerritory;
		private QLabel numICBMsTerritory;
		private QLabel numAntiMissilesTerritory;
		
		private QLabel totalPriceSoldiers;
		private QLabel totalPriceCannons;
		private QLabel totalPriceMissiles;
		private QLabel totalPriceMissilesICBM;
		private QLabel totalPriceAntiMissiles;
		private QLabel totalPriceBuy;
		private QLabel totalStore;
		
		private QSpinBox spinBoxSoldiers;
		private QSpinBox spinBoxCannons;
		private QSpinBox spinBoxMissiles;
		private	QSpinBox spinBoxMissilesICBM;
		private QSpinBox spinBoxAntiMissiles;
		
		public MenuBuy(Match father)
		{
			this.father = father;
			this.createMainWindow();
			this.createBoxBuy();
			
			show();
		}
		
		private int getIdTerritory()
		{
			return this.father.getPendingAction().getOriginTerritory();
		}
		
		private String getMoneyAvailable()
		{
			QueryPlayers player = new QueryPlayers(this.father.getIdGame());
			return String.valueOf(player.getMoney(this.father.getPlayerName()));
		}
		
		private String getNumSoldiers(int idTerritory)
		{
			return String.valueOf(this.father.getMapGame().getMapTerritories().getNumSoldiersTerritorySelected(idTerritory));
		}
		
		private int[] getNumCannons(int idTerritory)
		{
			return this.father.getMapGame().getMapTerritories().getNumCannonsTerritorySelected(idTerritory);
		}
		
		private String getNumMissiles(int idTerritory)
		{
			return String.valueOf(this.father.getMapGame().getMapTerritories().getNumMissilesTerritorySelected(idTerritory));
		}
			
		private String getNumICBMs(int idTerritory)
		{
			return String.valueOf(this.father.getMapGame().getMapTerritories().getNumICBMsTerritorySelected(idTerritory));
		}
			
		private String getNumAntiMissiles(int idTerritory)
		{
			return String.valueOf(this.father.getMapGame().getMapTerritories().getNumAntiMissilesTerritorySelected(idTerritory));
		}
		
		private void createMainWindow()
		{
			setWindowIcon(new QIcon(CommonConstants.APP_ICON));
			setWindowTitle(CommonConstants.APP_NAME + " - Comprar recursos");
			QDesktopWidget qdw = new QDesktopWidget();
	        int screen_center_width  = (qdw.width() - 250) / 2,
	            screen_center_height = (qdw.height() - 200) / 2;
	        setGeometry(screen_center_width, screen_center_height, 
	        			250, 100);
	        
	        setWindowModality(Qt.WindowModality.ApplicationModal);
	        setWindowFlags(Qt.WindowType.WindowCloseButtonHint);
	    	setStyleSheet("background-color:qlinearGradient(x1:0, y1:0, x2:0, y2:1, stop:0 #57360f, " +
				      "stop:1 #9f7340); color: #eeeeff; QLineEdit{background-color:#fffcca;border-radius:7px;" +
				      "border:1px solid black;padding:3px;color:#836b23}");
		}
		
		private QLabel sLabel(String data)
		{  // Create a simple label, without background color associated
			QLabel label = new QLabel(data);
			label.setStyleSheet("background-color:none;");
			return label;
		}
		
		private void createBoxBuy()
		{
			QVBoxLayout vbox = new QVBoxLayout();
			vbox.addStretch(1);
			vbox.addWidget(createGroupBoxInfo());
			vbox.addWidget(createGroupBoxBuy());
			vbox.addLayout(createButtonsLayout());
			
			this.setLayout(vbox);
		}
		
		private QGroupBox createGroupBoxInfo()
		{
			int idTerritory = getIdTerritory();
			
			QGridLayout grid = new QGridLayout(this);
			
			// Soldiers row
	        grid.addWidget(sLabel("N soldados"), 0, 0);
	        grid.addWidget(sLabel("="), 
	        			   0, 1, Qt.AlignmentFlag.AlignCenter);
	        
			this.numSoldiersTerritory = sLabel(new Integer(getNumSoldiers(idTerritory)).toString());
	        grid.addWidget(this.numSoldiersTerritory, 0, 2, Qt.AlignmentFlag.AlignRight);

	        // Cannons row
	        grid.addWidget(sLabel("N caones"), 1, 0);
	        grid.addWidget(sLabel("="), 
	        			   1, 1, Qt.AlignmentFlag.AlignCenter);
	        
			int []numCannons = getNumCannons(idTerritory);
			this.numCannonsTerritoryOneUse = sLabel(new Integer(String.valueOf(numCannons[0])).toString());
			this.numCannonsTerritoryTwoUses = sLabel(new Integer(String.valueOf(numCannons[1])).toString());
			this.numCannonsTerritoryThreeUses = sLabel(new Integer(String.valueOf(numCannons[2])).toString());
	        
	        grid.addWidget(this.numCannonsTerritoryOneUse, 1, 2, Qt.AlignmentFlag.AlignRight);
	        grid.addWidget(sLabel("de 1 uso,"), 1, 3, Qt.AlignmentFlag.AlignLeft);
	        grid.addWidget(this.numCannonsTerritoryTwoUses, 1, 4, Qt.AlignmentFlag.AlignLeft);
	        grid.addWidget(sLabel("de 2 usos,"), 1, 5, Qt.AlignmentFlag.AlignLeft);
	        grid.addWidget(this.numCannonsTerritoryThreeUses, 1, 6, Qt.AlignmentFlag.AlignLeft);
	        grid.addWidget(sLabel("de 3 usos"), 1, 7, Qt.AlignmentFlag.AlignLeft);

	        // Missiles row
	        grid.addWidget(sLabel("N misiles"), 2, 0);
	        grid.addWidget(sLabel("="), 
	        			   2, 1, Qt.AlignmentFlag.AlignCenter);
	        
			this.numMissilesTerritory = sLabel(new Integer(getNumMissiles(idTerritory)).toString());
	        grid.addWidget(this.numMissilesTerritory, 2, 2, Qt.AlignmentFlag.AlignRight);

	        // MissilesICBS row
	        grid.addWidget(sLabel("N misilesICBS"), 3, 0);
	        grid.addWidget(sLabel("="), 
	        			   3, 1, Qt.AlignmentFlag.AlignCenter);
	        
			this.numICBMsTerritory = sLabel(new Integer(getNumICBMs(idTerritory)).toString());
	        grid.addWidget(this.numICBMsTerritory, 3, 2, Qt.AlignmentFlag.AlignRight);

	        // Antimissiles row
	        grid.addWidget(sLabel("N antimisiles"), 4, 0);
	        grid.addWidget(sLabel("="), 
	        			   4, 1, Qt.AlignmentFlag.AlignCenter);
	        
	        this.numAntiMissilesTerritory = sLabel(new Integer(getNumAntiMissiles(idTerritory)).toString());
	        grid.addWidget(this.numAntiMissilesTerritory, 4, 2, Qt.AlignmentFlag.AlignRight);
	        
	        QGroupBox groupBoxInfo = new QGroupBox("Informacin del arsenal para " + Names.values()[idTerritory]);
			groupBoxInfo.setLayout(grid);
			return groupBoxInfo;
		}
		
		private QGroupBox createGroupBoxBuy()
		{
			spinBoxSoldiers = new QSpinBox();
			spinBoxSoldiers.setMinimum(0);
			spinBoxSoldiers.valueChanged.connect(this, "onChangeSoldiers(int)");
			
	        spinBoxCannons = new QSpinBox();
	        spinBoxCannons.setMinimum(0);
			spinBoxCannons.valueChanged.connect(this, "onChangeCannons(int)");
			
			spinBoxMissiles = new QSpinBox();
			spinBoxMissiles.setMinimum(0);
			spinBoxMissiles.valueChanged.connect(this, "onChangeMissile(int)");
			
	        spinBoxMissilesICBM = new QSpinBox();
			spinBoxMissilesICBM.setMinimum(0);
			spinBoxMissilesICBM.valueChanged.connect(this, "onChangeMissileICBM(int)");
			
	        spinBoxAntiMissiles = new QSpinBox();
			spinBoxAntiMissiles.setMinimum(0);
			spinBoxAntiMissiles.valueChanged.connect(this, "onChangeAntiMissile(int)");
	       
	        totalPriceSoldiers = sLabel("0");
	        totalPriceCannons = sLabel("0");
	        totalPriceMissiles = sLabel("0");
	        totalPriceMissilesICBM = sLabel("0");
	        totalPriceAntiMissiles = sLabel("0");
	        
	        QGridLayout grid = new QGridLayout(this);
	        
	        //Header
	        grid.addWidget(sLabel("Unidad"), 0, 0);
	        grid.addWidget(sLabel("Precio"), 
	        			   0, 1, Qt.AlignmentFlag.AlignCenter);
	        grid.addWidget(new QWidget(), 0, 2, Qt.AlignmentFlag.AlignCenter);
	        grid.addWidget(sLabel("Cantidad"), 0, 3);
	        grid.addWidget(new QWidget(), 0, 4, Qt.AlignmentFlag.AlignCenter);
	        grid.addWidget(sLabel("Total"), 0, 5, Qt.AlignmentFlag.AlignRight);
	        
	        // Soldiers row
	        grid.addWidget(sLabel("Soldados"), 1, 0);
	        grid.addWidget(sLabel(new Integer(Constants.priceSoldier).toString()), 
	        			   1, 1, Qt.AlignmentFlag.AlignCenter);
	        grid.addWidget(sLabel("x"), 1, 2, Qt.AlignmentFlag.AlignCenter);
	        grid.addWidget(spinBoxSoldiers, 1, 3);
	        grid.addWidget(sLabel("="), 1, 4, Qt.AlignmentFlag.AlignCenter);
	        grid.addWidget(totalPriceSoldiers, 1, 5, Qt.AlignmentFlag.AlignRight);
	        
	        // Cannons row
	        grid.addWidget(sLabel("Caones"), 2, 0);
	        grid.addWidget(sLabel(new Integer(Constants.priceCannon).toString()), 
	        			   2, 1, Qt.AlignmentFlag.AlignCenter);
	        grid.addWidget(sLabel("x"), 2, 2, Qt.AlignmentFlag.AlignCenter);
	        grid.addWidget(spinBoxCannons, 2, 3);
	        grid.addWidget(sLabel("="), 2, 4, Qt.AlignmentFlag.AlignCenter);
	        grid.addWidget(totalPriceCannons, 2, 5, Qt.AlignmentFlag.AlignRight);
	        
	        // Missiles row
	        grid.addWidget(sLabel("Misiles"), 3, 0);
	        grid.addWidget(sLabel(new Integer(Constants.priceMissil).toString()), 
	        			   3, 1, Qt.AlignmentFlag.AlignCenter);
	        grid.addWidget(sLabel("x"), 3, 2, Qt.AlignmentFlag.AlignCenter);
	        grid.addWidget(spinBoxMissiles, 3, 3);
	        grid.addWidget(sLabel("="), 3, 4, Qt.AlignmentFlag.AlignCenter);
	        grid.addWidget(totalPriceMissiles, 3, 5, Qt.AlignmentFlag.AlignRight);
	        
	        // MissilesICBM row
	        grid.addWidget(sLabel("Misiles ICBM"), 4, 0);
	        grid.addWidget(sLabel(new Integer(Constants.priceICBMs).toString()), 
	        			   4, 1, Qt.AlignmentFlag.AlignCenter);
	        grid.addWidget(sLabel("x"), 4, 2, Qt.AlignmentFlag.AlignCenter);
	        grid.addWidget(spinBoxMissilesICBM, 4, 3);
	        grid.addWidget(sLabel("="), 4, 4, Qt.AlignmentFlag.AlignCenter);
	        grid.addWidget(totalPriceMissilesICBM, 4, 5, Qt.AlignmentFlag.AlignRight);
	        
	        // AntiMissiles row
	        grid.addWidget(sLabel("AntiMissiles"), 5, 0);
	        grid.addWidget(sLabel(new Integer(Constants.priceAntimissil).toString()), 
	        			   5, 1, Qt.AlignmentFlag.AlignCenter);
	        grid.addWidget(sLabel("x"), 5, 2, Qt.AlignmentFlag.AlignCenter);
	        grid.addWidget(spinBoxAntiMissiles, 5, 3);
	        grid.addWidget(sLabel("="), 5, 4, Qt.AlignmentFlag.AlignCenter);
	        grid.addWidget(totalPriceAntiMissiles, 5, 5, Qt.AlignmentFlag.AlignRight);
	        
	        // Results row
	        grid.addWidget(sLabel("Dinero actual ="), 6, 0);
	        this.moneyAvailable = sLabel(new Integer(getMoneyAvailable()).toString());
	        grid.addWidget(this.moneyAvailable, 
	        			   6, 1, Qt.AlignmentFlag.AlignCenter);
	        grid.addWidget(sLabel("-"), 6, 2, Qt.AlignmentFlag.AlignCenter);
	        this.totalPriceBuy = sLabel("0");
	        grid.addWidget(this.totalPriceBuy, 6, 3);
	        grid.addWidget(sLabel("="), 6, 4, Qt.AlignmentFlag.AlignCenter);
	        totalStore = sLabel(new Integer(getMoneyAvailable()).toString());
	        grid.addWidget(totalStore, 6, 5, Qt.AlignmentFlag.AlignRight);
	        
	        QGroupBox groupBoxBuy = new QGroupBox("Comprar recursos para el territorio " + Names.values()[getIdTerritory()]);
	        groupBoxBuy.setLayout(grid);
	        return groupBoxBuy;
		}
		
		private QHBoxLayout createButtonsLayout()
		{
	        QPushButton pushButtonBuy = new QPushButton("Comprar");
	        pushButtonBuy.setIcon(new QIcon(CommonConstants.BUY_ICON));
	        pushButtonBuy.setMaximumWidth(80);
	        pushButtonBuy.clicked.connect(this, "onClickBuy()");
	        
	        QPushButton buttonExit = new QPushButton("Cancelar");
	        buttonExit.setIcon(new QIcon(CommonConstants.CROSS_ICON));
	        buttonExit.setMaximumWidth(80);
	        buttonExit.clicked.connect(this, "onClickCancel()");
	        
	        QHBoxLayout buttonsLayout = new QHBoxLayout();
	        buttonsLayout.addWidget(pushButtonBuy);
	        buttonsLayout.addWidget(buttonExit);
	        return buttonsLayout;
		}
		
		@SuppressWarnings("unused")
	    private void onClickBuy()
	    {
			if(Integer.parseInt(this.totalPriceBuy.text())!= 0){
				if(Integer.parseInt(totalStore.text()) >= 0)
				{	
					this.father.getMapGame().getMapTerritories()
					.buyArsenal(getIdTerritory(),spinBoxSoldiers.value(),
							spinBoxCannons.value(), spinBoxMissiles.value(),
							spinBoxMissilesICBM.value(), spinBoxAntiMissiles.value());
				
					QueryMoney charge = new QueryMoney(this.father.getIdGame(), this.father.getPlayerName());
					charge.chargeBuyArsenal(Integer.parseInt(totalStore.text()));
					
					Connection.updateGameBuyArsenal(this.father.getIdGame(), this.father.getSession());
					
					new Alert(
							"xito",
							"La compra se ha realizado con xito.",
							CommonConstants.SUCCESS_ICON);
					this.father.buyArsenal();
					this.close();
				}
				else
				{
					new Alert("Aviso", "No tiene dinero suficiente para realizar la compra.");
				}
			}
			else
				new Alert("Aviso", "Debe de elegir un armamento para comprar.");
	    }
		
		@SuppressWarnings("unused")
		private void onClickCancel()
		{
			this.close();
		}
		
	    @Override
	    public void closeEvent(QCloseEvent event)
	    {
	    	if(CommonConstants.DEBUG) System.out.println("Closing window MenuBuy");

	    	this.father.getMapGame().cancelBuyArsenal();
	    	this.close();
	    }
		
		@SuppressWarnings("unused")
		private void onChangeSoldiers(int value)
		{
			totalPriceSoldiers.setText(new Integer(value * Constants.priceSoldier).toString());
			updateFields();
		}
		
		@SuppressWarnings("unused")
		private void onChangeCannons(int value)
		{
			totalPriceCannons.setText(new Integer(value * Constants.priceCannon).toString());
			updateFields();
		}
		
		@SuppressWarnings("unused")
		private void onChangeMissile(int value)
		{
			totalPriceMissiles.setText(new Integer(value * Constants.priceMissil).toString());
			updateFields();
		}
		
		@SuppressWarnings("unused")
		private void onChangeMissileICBM(int value)
		{
			totalPriceMissilesICBM.setText(new Integer(value * Constants.priceICBMs).toString());
			updateFields();
		}
		
		@SuppressWarnings("unused")
		private void onChangeAntiMissile(int value)
		{
			totalPriceAntiMissiles.setText(new Integer(value * Constants.priceAntimissil).toString());
			updateFields();
		}
		
		private void updateFields()
		{
			int price1 = Integer.parseInt(totalPriceSoldiers.text());
			int price2 = Integer.parseInt(totalPriceCannons.text());
			int price3 = Integer.parseInt(totalPriceMissiles.text());
			int price4 = Integer.parseInt(totalPriceMissilesICBM.text());
			int price5 = Integer.parseInt(totalPriceAntiMissiles.text());

			int totalprice = price1 + price2 + price3 + price4 + price5;
			totalPriceBuy.setText(new Integer(totalprice).toString());
			
			int result = Integer.parseInt(getMoneyAvailable()) - totalprice;
			totalStore.setText(new Integer(result).toString()); 
		}
}
