package ea.presentation;

import java.net.ConnectException;
import java.rmi.RemoteException;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Enumeration;
import java.util.Iterator;
import java.util.List;
import java.util.UUID;
import java.util.Vector;

import com.trolltech.qt.QThread;
import com.trolltech.qt.core.*;
import com.trolltech.qt.core.Qt.ItemFlag;
import com.trolltech.qt.core.Qt.ItemFlags;
import com.trolltech.qt.gui.*;

import domain.Arsenal;
import domain.Game;
import domain.GameListener;
import domain.ObserverGame;
import domain.Territory;
import ea.domain.Connection;
import ea.domain.ListGames;
import ea.domain.Listener;
import ea.domain.QueryGames;
import ea.exceptions.EmptyInformationException;
import ea.exceptions.NullInformationException;
import exceptions.AlreadyInGameException;
import exceptions.FullGameException;
import exceptions.GameNotFoundException;
import exceptions.InvalidSessionException;
import exceptions.InvalidTerritoryException;
import exceptions.InvalidTimeException;
import exceptions.NotCurrentPlayerGameException;
import exceptions.WrongLoginException;

public class MenuGames extends QWidget {
	private UUID session;
	private String namePlayer;
	private QStandardItemModel modelGames; 
	private QStandardItemModel modelGameDays;
	private QStandardItemModel modelPlayers;
	private QLabel descriptionGame;
	private QTreeView gameView; 
	private QueryGames queryGames;
	private boolean quitSystem = true;

	public MenuGames(UUID session, String namePlayer){
		this.session = session;
		this.namePlayer = namePlayer;
		this.createMainWindow();

		this.createModelGames();
		this.createTreeViewGameView();

		this.queryGames = new QueryGames();
		this.queryAndFillInformation();

		QVBoxLayout description_layout = new QVBoxLayout();
		description_layout.setGeometry(new QRect(5, 5, 100, 100));

		this.createLabelDescriptionGame();

		this.createModelPlayers();

		QTreeView treePlayers = new QTreeView();
		treePlayers.setStyleSheet("background-color:#b8946a;border-radius:2px;"
				+ "border:1px solid black;color:#836b23");
		treePlayers.setModel(getModelPlayers());

		this.createModelGameDays();

		QTreeView treeGameDays = new QTreeView();
		treeGameDays
				.setStyleSheet("background-color:#b8946a;border-radius:2px;"
						+ "border:1px solid black;color:#836b23");
		treeGameDays.setModel(getModelGameDays());

		QHBoxLayout modelsLayout = new QHBoxLayout();
		modelsLayout.addWidget(treePlayers, 0);
		modelsLayout.addWidget(treeGameDays, 0);

		description_layout.addWidget(this.getLabelDescriptionGame());
		description_layout.addLayout(modelsLayout);

		QGroupBox description_box = new QGroupBox("Descripcin de la partida");
		description_box.setLayout(description_layout);
		description_box.setFixedSize(400, 400);
		description_box.setAttribute(Qt.WidgetAttribute.WA_ContentsPropagated);
		description_box
				.setStyleSheet("QLabel{background-color:#fffcca;border-radius:7px;"
						+ "border:3px dotted black;padding:5px;color:#836b23}");

		QHBoxLayout game_view_layout = new QHBoxLayout();
		game_view_layout.addWidget(this.getTreeViewGameView());
		game_view_layout.addWidget(description_box);

		QGroupBox game_group_box = new QGroupBox("Listado de partidas");
		game_group_box.setLayout(game_view_layout);

		QVBoxLayout mainLayout = new QVBoxLayout(this);
		mainLayout.addWidget(game_group_box);

		QHBoxLayout buttonsBox = new QHBoxLayout();
		this.createButtonCreateGame(buttonsBox);
		this.createButtonRefreshGame(buttonsBox);
		this.createButtonJoinGame(buttonsBox);
		this.createButtonBack(buttonsBox);

		QGroupBox buttons_group_box = new QGroupBox("Acciones");
		buttons_group_box.setLayout(buttonsBox);

		mainLayout.addWidget(buttons_group_box);

		setLayout(mainLayout);
		show();
	}

	private String getNamePlayer()
	{
		return this.namePlayer;
	}
	
	private UUID getSession()
	{
		return this.session;
	}
		
	private void setQuitSystem(boolean quitSystem) {
		this.quitSystem = quitSystem;
	}

	private boolean getQuitSystem() {
		return quitSystem;
	}

	private void queryAndFillInformation(){
		try {
			this.getQueryGames().queryGamesServer(this.getSession());
			this.getQueryGames().sortGamesByPlayer(this.getNamePlayer());
			this.completeInformation();
		} catch (InvalidSessionException e) {
			new Alert("Error", "Invalida sesin.");
		} catch (RemoteException e) {
			new Alert("Error", "Causa:\n" + e.getMessage());
		} catch (ConnectException e) {
			new Alert("Error", "Conexin rechazada.\nException:\n"
					+ e.getLocalizedMessage() + "\n\nCausa:\n" + e.getCause());
		} catch (NullInformationException e) {
			new Alert("Error fatal", "Causa:\n" + e.getLocalizedMessage());
		}catch(EmptyInformationException e) {
			new Alert("Error fatal", "Causa:\n" + e.getLocalizedMessage()); 
        }
	}

	private void completeInformation()throws NullInformationException
	{
		ArrayList<String> dataNames = this.getQueryGames().getListGamesName();
		ArrayList<ArrayList<String>> dataUsers = this.getQueryGames()
				.getListGamesUsers();

		int separator = this.getQueryGames().getSeparator();

		for (int i = 0; i <= separator; i++) {// Your games
			System.out.println("Partida: " + dataNames.get(i) + "Jugadores: "
					+ dataUsers.get(i));
			this.addDataModelGames(dataNames.get(i), ((Integer) dataUsers
					.get(i).size()).toString());
		}

		for (int i = separator + 1; i < dataNames.size(); i++) {// Rest Games
			this.addDataModelGames(dataNames.get(i), ((Integer) dataUsers
					.get(i).size()).toString());
		}
	}

	private QueryGames getQueryGames() {
		return this.queryGames;
	}

	private void createMainWindow() {
		setWindowTitle(CommonConstants.APP_NAME + " - Crear/Unirse a partida");

		QDesktopWidget qdw = new QDesktopWidget();
		int screen_center_width = (qdw.width() - CommonConstants.DEFAULT_WIDTH) / 2, screen_center_height = (qdw
				.height() - CommonConstants.DEFAULT_HEIGHT) / 2;

		setGeometry(screen_center_width, screen_center_height,
				CommonConstants.DEFAULT_WIDTH, CommonConstants.DEFAULT_HEIGHT);
		setFixedSize(CommonConstants.DEFAULT_WIDTH,
				CommonConstants.DEFAULT_HEIGHT);

		setWindowIcon(new QIcon(CommonConstants.APP_ICON));
		setStyleSheet("background-color:qlinearGradient(x1:0, y1:0, x2:0, y2:1, stop:0 #57360f, "
				+ "stop:1 #9f7340); color: #eeeeff");
	}

	@Override
	public void closeEvent(QCloseEvent event) {
		if (CommonConstants.DEBUG)
			System.out.println("Closing window MenuGames");
		if (this.isWindow()) {
			this.close();
			if (this.getQuitSystem()) {
				QCoreApplication.exit();
				QApplication.instance().dispose();
		    	QApplication.quit();
		    	System.exit(0);
    		}
    	}
    	else event.ignore();
    }
    
    private void createModelGameDays()
    {
        modelGameDays = new QStandardItemModel(0, 1);
        modelGameDays.setHeaderData(0, Qt.Orientation.Horizontal, "Das de juego");
    }
    
    private QStandardItemModel getModelGameDays()
    {
    	return this.modelGameDays;
    }
    
    private void addDataModelGameDays(String day) 
    {
    	QStandardItemModel model = getModelGameDays();
    	model.insertRow(model.rowCount());
    	model.setData(model.rowCount() - 1, 0, day);
    	
    	ItemFlags itemFlags = new ItemFlags(new ItemFlag[]{ItemFlag.ItemIsSelectable, ItemFlag.ItemIsEnabled});
        model.itemFromIndex(model.index(model.rowCount()-1, 0)).setFlags(itemFlags);
    }
    
    private void createModelPlayers()
    {
        modelPlayers = new QStandardItemModel(0, 1);
        modelPlayers.setHeaderData(0, Qt.Orientation.Horizontal, "Jugadores");
    }
    
    private QStandardItemModel getModelPlayers()
    {
    	return this.modelPlayers;
    }
    
    private void addDataModelPlayers(String player) 
    {
    	QStandardItemModel model = getModelPlayers();
    	model.insertRow(model.rowCount());
    	model.setData(model.rowCount() - 1, 0, player);
    	
    	ItemFlags itemFlags = new ItemFlags(new ItemFlag[]{ItemFlag.ItemIsSelectable, ItemFlag.ItemIsEnabled});
        model.itemFromIndex(model.index(model.rowCount()-1, 0)).setFlags(itemFlags);
    }
    
    private void createModelGames()
    {
    	this.modelGames = new QStandardItemModel(0, 3, this);
        this.modelGames.setHeaderData(0, Qt.Orientation.Horizontal, "Partida");
        this.modelGames.setHeaderData(1, Qt.Orientation.Horizontal, "Jugadores");
        this.modelGames.setHeaderData(2, Qt.Orientation.Horizontal, "Territorios Libres");
    }
    
    private void addDataModelGames(String game, String players) 
    {
    	QStandardItemModel model = this.getModelGames();
    	
    	model.insertRow(model.rowCount());
		model.setData(model.rowCount()-1,0, game);
		model.setData(model.rowCount()-1, 1, players);
		model.setData(model.rowCount()-1, 2, String.valueOf(ea.libs.Constants.totalTerritories - Integer.parseInt(players)));
		
		ItemFlags itemFlags = new ItemFlags(new ItemFlag[]{ItemFlag.ItemIsSelectable, ItemFlag.ItemIsEnabled});
        model.itemFromIndex(model.index(model.rowCount()-1, 0)).setFlags(itemFlags);
        model.itemFromIndex(model.index(model.rowCount()-1, 1)).setFlags(itemFlags);
        model.itemFromIndex(model.index(model.rowCount()-1, 2)).setFlags(itemFlags);
	}
    
    private QStandardItemModel getModelGames()
    {
    	return this.modelGames;
    }
    
    private void createLabelDescriptionGame()
    {
        this.descriptionGame = new QLabel();
        this.descriptionGame.setText("Pincha sobre una partida para ver su descripcin");
        this.descriptionGame.setAlignment(new Qt.Alignment(Qt.AlignmentFlag.AlignTop));
        this.descriptionGame.setLineWidth(80);
        this.descriptionGame.setMaximumWidth(400);
    }
    
    private QLabel getLabelDescriptionGame()
    {
    	return this.descriptionGame;
    }
    
    // Taken from http://progcookbook.blogspot.com/2006/02/text-wrapping-function-for-java.html
    @SuppressWarnings("unchecked")
	private String [] wrapText (String text, int len)
    {
      // return empty array for null text
      if (text == null)
      return new String [] {};

      // return text if len is zero or less
      if (len <= 0)
      return new String [] {text};

      // return text if less than length
      if (text.length() <= len)
      return new String [] {text};

      char [] chars = text.toCharArray();
      @SuppressWarnings("rawtypes")
	  Vector lines = new Vector();
      StringBuffer line = new StringBuffer();
      StringBuffer word = new StringBuffer();

      for (int i = 0; i < chars.length; i++) {
        word.append(chars[i]);

        if (chars[i] == ' ') {
          if ((line.length() + word.length()) > len) {
            lines.add(line.toString());
            line.delete(0, line.length());
          }

          line.append(word);
          word.delete(0, word.length());
        }
      }

      // handle any extra chars in current word
      if (word.length() > 0) {
        if ((line.length() + word.length()) > len) {
          lines.add(line.toString());
          line.delete(0, line.length());
        }
        line.append(word);
      }

      // handle extra line
      if (line.length() > 0) {
        lines.add(line.toString());
      }

      String [] ret = new String[lines.size()];
      int c = 0; // counter
      for (@SuppressWarnings("rawtypes")
    		 Enumeration e = lines.elements(); e.hasMoreElements(); c++) {
        ret[c] = (String) e.nextElement();
      }

      return ret;
    }
    
    private void setLabelDescriptionGame(String data)
    {
    	String [] aux = wrapText(data, 70);
    	data = "";
    	for(int i = 0; i < aux.length; i++) 
    	    data += aux[i] + "\r\n";
    	
    	this.getLabelDescriptionGame().setText(data);
    }
    
    private void createTreeViewGameView()
    {
        this.gameView = new QTreeView();
        this.gameView.setStyleSheet("background-color:#b8946a;border-radius:2px;" + 
        			       			"border:1px solid black;color:#836b23");
        this.gameView.setModel(this.getModelGames());
        this.gameView.setAlternatingRowColors(true);
        this.gameView.clicked.connect(this, "onClickTreeViewGameView(com.trolltech.qt.core.QModelIndex)");
        this.gameView.doubleClicked.connect(this, "onDoubleClickTreeViewGameView(com.trolltech.qt.core.QModelIndex)");
    } 
    
    @SuppressWarnings("unused")
	private void onClickTreeViewGameView(QModelIndex index)
    {  
    	// First remove data for players and days if exist
    	for(int i = 0; i <= this.getModelGameDays().rowCount() + 1; i++)
    		getModelGameDays().removeRow(i);
    	
       	for(int i = 0; i <= this.getModelPlayers().rowCount() + 1; i++)
    		getModelPlayers().removeRow(i);
       	
       	// Double delete...yes, necessary
    	for(int i = 0; i <= this.getModelGameDays().rowCount() + 1; i++)
    		getModelGameDays().removeRow(i);
    	
       	for(int i = 0; i <= this.getModelPlayers().rowCount() + 1; i++)
    		getModelPlayers().removeRow(i);
    	
       	try{
       		ArrayList<String> dataDescriptions = this.getQueryGames().getListGamesDescription();
       		ArrayList<String> dataGamesName = this.getQueryGames().getListGamesName();
       		ArrayList<ArrayList<String>> dataGamesUsers = this.getQueryGames().getListGamesUsers();
       		ArrayList<ArrayList<Calendar>> dataGamesSessions = this.getQueryGames().getGameSessions();
  
       		/*
    	 	System.out.println("Columna:" + index.column() + "\n" +
    			"Data:" + index.data() + "\n" +
    			"Model:" + index.model() + "\n" +
    			"Row:" + index.row() + "\n");
       		 */
       		if(CommonConstants.DEBUG)
       			System.out.println("Nombres juegos: " + dataGamesName.toString());
    	
       		String data = "Nombre:" + dataGamesName.get(index.row()) + "\n" +
    				  "Descripcion:\n" + dataDescriptions.get(index.row());
    	
       		setLabelDescriptionGame(data);
    	
       		Iterator<Calendar> iter = dataGamesSessions.get(index.row()).iterator();
       		while(iter.hasNext())
       		{
       			Calendar cal = iter.next();
       			addDataModelGameDays(cal.get(Calendar.HOUR_OF_DAY) + ":" + 
					             cal.get(Calendar.MINUTE) + " " +
					             cal.get(Calendar.DAY_OF_MONTH) + "-" +
					             cal.get(Calendar.MONTH +1) + "-" +
					             cal.get(Calendar.YEAR));
       		}

       		Iterator<String> iterb = dataGamesUsers.get(index.row()).iterator();
       		while (iterb.hasNext()) {
       			addDataModelPlayers(iterb.next());
       		}
       	}
       	catch(NullInformationException e){
       		new Alert("Error fatal", "Causa:\n" + e.getLocalizedMessage());
       	}
    }
    
    @SuppressWarnings("unused")
	private void onDoubleClickTreeViewGameView(QModelIndex index)
    {  
    	try{
    		ArrayList<UUID> dataGameID = this.getQueryGames().getListGameId();
    		UUID idgame = dataGameID.get(index.row());
    		int separator = this.getQueryGames().getSeparator();
    		if(index.row()<= separator){//This player is associated to game
    			try
    			{
    				Game game = Connection.playGame(this.getSession(), idgame);
    				ListGames.addGame(game);
    				Match matchRunnable = new Match(this.getSession(), idgame, this.getNamePlayer());
    				QThread runner = new QThread(matchRunnable);
				
    				Listener.addMatchObserver(matchRunnable);
    				runner.start();
				
    			} catch (GameNotFoundException e) {
    				new Alert("Error", "La partida no ha sido encontrada.");
    			} catch (InvalidSessionException e) {
    				new Alert("Error", "La sesin no es vlida o esta expirada.");
    			} catch (InvalidTimeException e) {
    				new Alert("Error", "El tiempo de juego de la partida no ha llegado.");
    			} catch (RemoteException e) {
    				new Alert("Error", "Causa:\n" + e.getMessage());
    			} catch (NullInformationException e) {
    				new Alert("Error fatal", "Causa:\n" + e.getLocalizedMessage());
    			} catch (NotCurrentPlayerGameException e) {
    				new Alert("Error", "No es permitido un jugador concurrente en una partida");
    			} catch(AlreadyInGameException e){
    				new Alert("Error", "El jugador ya est jugando la partida.");
    			} catch(EmptyInformationException e){
    				new Alert("Error fatal", "Causa:\n" + e.getLocalizedMessage()); 
    			}
    		} else
    			new Alert("Error",
					"No puede jugar una partida a la que no est inscrito.");
    	}catch(NullInformationException e){
    		new Alert("Error fatal", "Causa:\n" + e.getLocalizedMessage());
    	}
	}
    
    private QTreeView getTreeViewGameView()
    {
    	return this.gameView;
    }
    
    
    private void createButtonCreateGame(QHBoxLayout buttonsBox)
    {
    	QPushButton create_button = new QPushButton("&Crear", this);
    	create_button.setShortcut("C");
    	create_button.clicked.connect(this, "onClickCreateGame()"); 
    	create_button.setIcon(new QIcon(CommonConstants.CREATE_ICON));
    	buttonsBox.addWidget(create_button);
    }
    
    
    @SuppressWarnings("unused")
	private void onClickCreateGame()
    {
    	if(this.getSession() == null)
    	{
    		new Alert("Error","Causa:\n El identificador de sesin es nulo.");
    		return;
    	}
    	if(this.getNamePlayer()== null)
    	{
    		new Alert("Error","Causa:\n El nombre del jugador es nulo.");
    		return;
    	}
    	if(this.getNamePlayer().isEmpty())
    	{
    		new Alert("Error","Causa:\n El nombre del jugador es vaco.");
    		return;
    	}
    	
    	this.setQuitSystem(false);
    	this.close();
    	this.setQuitSystem(true);
    	CreateGame create_game = new CreateGame(this.getSession(), this.getNamePlayer());
    }
    
    private void createButtonJoinGame(QHBoxLayout buttonsBox)
    {
    	QPushButton join_button = new QPushButton("&Unirse", this);
    	join_button.setShortcut("U");
    	join_button.clicked.connect(this, "onClickJoinGame()");
    	join_button.setIcon(new QIcon(CommonConstants.JOIN_ICON));
    	buttonsBox.addWidget(join_button);
    }
    
    
    @SuppressWarnings("unused")
	private void onClickJoinGame()
    {
    	List<QModelIndex> selection = this.getTreeViewGameView().selectionModel().selectedRows();
    	
    	if(selection.isEmpty())	
    		new Alert("Aviso", "Debe seleccionarse alguna partida.");
    	else
    	{	
	    	try{
	    		ArrayList<UUID> dataGameID = this.getQueryGames().getListGameId();
	    		UUID game = dataGameID.get(selection.get(0).row());
	    
	    		if(CommonConstants.DEBUG) 
	    			System.out.println("Game UUID al hacer join:" + game.toString());
	    	
	    		try 
	    		{
	    			Connection.joinGame(this.getSession(), game);
	    			this.onClickRefreshGame();
	    			new Alert("xito", "Te has unido correctamente a la partida seleccionada.", CommonConstants.SUCCESS_ICON);
	    		} 
	    		catch(FullGameException  e)
	    		{
	    			new Alert("Error", "Esta partida no permite ya ms jugadores.");
	    		}
	    		catch(GameNotFoundException  e)
	    		{
	    			new Alert("Error", "La partida seleccionada no ha sido encontrada.");
	    		}
	    		catch(InvalidSessionException  e)
	    		{
	    			new Alert("Error", "La sesin no es vlida o esta expirada.");
	    		}
	    		catch(AlreadyInGameException  e)
	    		{
	    			new Alert("Error", "El jugador ya estaba unido a esta partida.");
	    		}
	    		catch(RemoteException e)
	    		{
	    			new Alert("Error","Causa:\n"+ e.getMessage());
	    		} 
	    		catch(EmptyInformationException e)
	    		{
	    			new Alert("Error fatal", 
            			"Causa:\n" + e.getLocalizedMessage()); 
	    		}
	    		catch (NullInformationException e) 
	    		{
	    			new Alert("Error fatal", "Causa:\n" + e.getLocalizedMessage());
	    		}
	    	}catch(NullInformationException e){
	    		new Alert("Error fatal", "Causa:\n" + e.getLocalizedMessage());
	    	}
    	}
    }
    
    private void createButtonRefreshGame(QHBoxLayout buttonsBox)
    {
    	QPushButton refresh_button = new QPushButton("&Actualizar", this);
    	refresh_button.clicked.connect(this, "onClickRefreshGame()");
    	refresh_button.setShortcut("A");
    	refresh_button.setIcon(new QIcon(CommonConstants.REFRESH_ICON));
    	buttonsBox.addWidget(refresh_button);
    }
    
    
	private void onClickRefreshGame()
    {
    	for(int i = 0; i <= this.getModelGames().rowCount() + 1; i++)
    		this.getTreeViewGameView().model().removeRow(i);
    	// [FIXME] Crazy stuff, for some reason one game always remains, so delete again
    	for(int i = 0; i <= this.getModelGames().rowCount() + 1; i++)
    		this.getTreeViewGameView().model().removeRow(i);
    	this.setLabelDescriptionGame("Pincha sobre una partida para ver su descripcin");
    	
    	for(int i = 0; i <= this.getModelGameDays().rowCount() + 1; i++)
    		getModelGameDays().removeRow(i);
    	
       	for(int i = 0; i <= this.getModelPlayers().rowCount() + 1; i++)
    		getModelPlayers().removeRow(i);
       	
    	this.queryAndFillInformation();
    }
    
    
    private void createButtonBack(QHBoxLayout buttonsBox)
    {
    	QPushButton button = new QPushButton("&Volver", this);
    	button.setShortcut("V");
	    button.setIcon(new QIcon("img/back.png"));
    	button.clicked.connect(this, "onClickBack()"); 
    	buttonsBox.addWidget(button);
    }
    
    
    @SuppressWarnings("unused")
	private void onClickBack()
    {
    	if(ListGames.isEmpty())
    	{
    		try
    		{
    			Connection.logoutPlayer(this.getSession());
    			this.setQuitSystem(false);
        		this.close();
        		this.setQuitSystem(true);
        		MenuLogin menu_login = new MenuLogin();
        		menu_login.show();
    		}
        	catch(RemoteException e)
            {
            	new Alert("Error","Causa:\n"+ e.getMessage());
            } 
        	catch (InvalidSessionException e) 
            {
            	new Alert("Error",             			
						"Conexin rechazada.\nException:\n" + e.getLocalizedMessage() +
            			"\n\nCausa:\n" + e.getCause());
			} 
        	catch (NullInformationException e) 
        	{
	            new Alert("Error fatal", 
            			"Causa:\n" + e.getLocalizedMessage()); 
			}
			catch(EmptyInformationException e)
            {
				new Alert("Error fatal", 
	            			"Causa:\n" + e.getLocalizedMessage()); 
            }
    	}
    	else
    		new Alert("Aviso","Para cerrar sesin, debe de cerrar toda partida que est jugando.");
    }
}