package ea.presentation;

import com.trolltech.qt.core.Qt;
import com.trolltech.qt.gui.QCloseEvent;
import com.trolltech.qt.gui.QDesktopWidget;
import com.trolltech.qt.gui.QGridLayout;
import com.trolltech.qt.gui.QGroupBox;
import com.trolltech.qt.gui.QHBoxLayout;
import com.trolltech.qt.gui.QIcon;
import com.trolltech.qt.gui.QLabel;
import com.trolltech.qt.gui.QPixmap;
import com.trolltech.qt.gui.QPushButton;
import com.trolltech.qt.gui.QVBoxLayout;
import com.trolltech.qt.gui.QWidget;

import ea.domain.Names;
import ea.domain.QuerySpies;

public class MenuInformationSpy extends QWidget 
{
	private QuerySpies informationSpied;
	private int index;
	
	public MenuInformationSpy(QuerySpies information, int index)
	{
		this.informationSpied = information;
		this.index = index;
		this.createMainWindow();
		this.createBoxBuy();
		
		show();
	}
	
	private void createMainWindow()
	{
		setWindowIcon(new QIcon(CommonConstants.APP_ICON));
		setWindowTitle(CommonConstants.APP_NAME + " - Informacin del territorio");
		QDesktopWidget qdw = new QDesktopWidget();
        int screen_center_width  = (qdw.width() - 250) / 2,
            screen_center_height = (qdw.height() - 300) / 2;
        setGeometry(screen_center_width, screen_center_height, 
        			250, 100);
        
        setWindowModality(Qt.WindowModality.ApplicationModal);
        setWindowFlags(Qt.WindowType.WindowCloseButtonHint);
    	setStyleSheet("background-color:qlinearGradient(x1:0, y1:0, x2:0, y2:1, stop:0 #57360f, " +
			      "stop:1 #9f7340); color: #eeeeff; QLineEdit{background-color:#fffcca;border-radius:7px;" +
			      "border:1px solid black;padding:3px;color:#836b23}");
	}

	private QLabel sLabel(String data)
	{  // Create a simple label, without background color associated
		QLabel label = new QLabel(data);
		label.setStyleSheet("background-color:none;");
		return label;
	}
	
	private QGroupBox createGroupBoxInfo()
	{	
		QGridLayout grid = new QGridLayout(this);
		
		// Soldiers row
        grid.addWidget(sLabel("N soldados"), 0, 0);
        grid.addWidget(sLabel("="), 
        			   0, 1, Qt.AlignmentFlag.AlignCenter);
        
        grid.addWidget(sLabel(new Integer(
        		informationSpied.getSoldiersSpy(index)).toString()), 
        		0, 2, Qt.AlignmentFlag.AlignRight);

        // Cannons row
        grid.addWidget(sLabel("N caones"), 1, 0);
        grid.addWidget(sLabel("="), 
        			   1, 1, Qt.AlignmentFlag.AlignCenter);
		
        grid.addWidget(sLabel(new Integer(String.valueOf(
        		informationSpied.getCannonsOneSpy(index))).toString()), 
        		1, 2, Qt.AlignmentFlag.AlignRight);
        grid.addWidget(sLabel("de 1 uso,"), 1, 3, Qt.AlignmentFlag.AlignLeft);
        grid.addWidget(sLabel(new Integer(String.valueOf(
        		informationSpied.getCannonsTwoSpy(index))).toString()),
        		1, 4, Qt.AlignmentFlag.AlignLeft);
        grid.addWidget(sLabel("de 2 usos,"), 1, 5, Qt.AlignmentFlag.AlignLeft);
        grid.addWidget(sLabel(new Integer(String.valueOf(
        		informationSpied.getCannonsThreeSpy(index))).toString()),
        		1, 6, Qt.AlignmentFlag.AlignLeft);
        grid.addWidget(sLabel("de 3 usos"), 1, 7, Qt.AlignmentFlag.AlignLeft);

        // Missiles row
        grid.addWidget(sLabel("N misiles"), 2, 0);
        grid.addWidget(sLabel("="), 
        			   2, 1, Qt.AlignmentFlag.AlignCenter);
        
        grid.addWidget(sLabel(new Integer(informationSpied.getMissilsSpy(index)).toString())
        		, 2, 2, Qt.AlignmentFlag.AlignRight);

        // MissilesICBS row
        grid.addWidget(sLabel("N misilesICBS"), 3, 0);
        grid.addWidget(sLabel("="), 
        			   3, 1, Qt.AlignmentFlag.AlignCenter);

        grid.addWidget(sLabel(new Integer(informationSpied.getICBMsSpy(index)).toString()),
        		3, 2, Qt.AlignmentFlag.AlignRight);

        // Antimissils row
        grid.addWidget(sLabel("N antimisiles"), 4, 0);
        grid.addWidget(sLabel("="), 
        			   4, 1, Qt.AlignmentFlag.AlignCenter);
        
        grid.addWidget(sLabel(new Integer(informationSpied.getAntiMissilsSpy(index)).toString()),
        		4, 2, Qt.AlignmentFlag.AlignRight);
        
        
        //Money row
        grid.addWidget(sLabel("Dinero"),5,0);
        grid.addWidget(sLabel("="), 
 			   5, 1, Qt.AlignmentFlag.AlignCenter);
        grid.addWidget(sLabel(new Integer(informationSpied.getMoneySpy(index)).toString()),
        		5, 2, Qt.AlignmentFlag.AlignRight);
        
        
        QGroupBox groupBoxInfo = new QGroupBox("Informacin del territorio " + 
        		Names.values()[informationSpied.getLocationSpy(index)] + " cuyo propietario: " + 
        		informationSpied.getOwner(index));
		groupBoxInfo.setLayout(grid);
		return groupBoxInfo;
	}
	
	private void createBoxBuy()
	{
		QHBoxLayout hb = new QHBoxLayout();
		hb.addWidget(createImage());
		hb.addWidget(createGroupBoxInfo());
		
		QVBoxLayout vbox = new QVBoxLayout();
		vbox.addLayout(hb);
		vbox.addLayout(createButtonsLayout());
		this.setLayout(vbox);
	}

	private QLabel createImage()
	{
		QLabel label = new QLabel();
		QPixmap img = new QPixmap("img/zone/" + 
				String.valueOf(informationSpied.getLocationSpy(index)+1) + ".png");
		label.setPixmap(img.scaled(200, 200));
		label.setMinimumWidth(200);
		label.setMinimumHeight(200);
		return label;
	}
	
	private QHBoxLayout createButtonsLayout()
	{
        QPushButton pushButton = new QPushButton("Aceptar");
        pushButton.setIcon(new QIcon(CommonConstants.SUCCESS_ICON));
        pushButton.setMaximumWidth(80);
        pushButton.clicked.connect(this, "onClickAcept()");
        
        QHBoxLayout buttonsLayout = new QHBoxLayout();
        buttonsLayout.addWidget(pushButton);
        return buttonsLayout;
	}
	
	@SuppressWarnings("unused")
	private void onClickAcept()
	{
		this.close();
	}
	
    @Override
    public void closeEvent(QCloseEvent event)
    {
    	if(CommonConstants.DEBUG) System.out.println("Closing window MenuShowInformation");
    	this.close();
    }
}