package ea.presentation;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.UUID;

import com.trolltech.qt.core.QModelIndex;
import com.trolltech.qt.core.Qt;
import com.trolltech.qt.core.Qt.ItemFlag;
import com.trolltech.qt.core.Qt.ItemFlags;
import com.trolltech.qt.gui.QCloseEvent;
import com.trolltech.qt.gui.QDesktopWidget;
import com.trolltech.qt.gui.QGridLayout;
import com.trolltech.qt.gui.QGroupBox;
import com.trolltech.qt.gui.QHBoxLayout;
import com.trolltech.qt.gui.QIcon;
import com.trolltech.qt.gui.QLabel;
import com.trolltech.qt.gui.QPixmap;
import com.trolltech.qt.gui.QPushButton;
import com.trolltech.qt.gui.QStandardItemModel;
import com.trolltech.qt.gui.QTreeView;
import com.trolltech.qt.gui.QVBoxLayout;
import com.trolltech.qt.gui.QWidget;

import ea.domain.InformationSpy;
import ea.domain.Names;
import ea.domain.QuerySpies;
import exceptions.AlreadyInGameException;

public class MenuListSpy extends QWidget 
{
			private QuerySpies information;
		    private QStandardItemModel model;
			
			public MenuListSpy(QuerySpies information)
			{
				this.information = information;
				this.createMainWindow();
				this.createBox();
				
				show();
			}
			
			private void createMainWindow()
			{
				setWindowIcon(new QIcon(CommonConstants.APP_ICON));
				setWindowTitle(CommonConstants.APP_NAME + " - Listar espas");
				QDesktopWidget qdw = new QDesktopWidget();
		        int screen_center_width  = (qdw.width() - 250) / 2,
		            screen_center_height = (qdw.height() - 300) / 2;
		        setGeometry(screen_center_width, screen_center_height, 
		        			350, 300);
		        
		        setWindowModality(Qt.WindowModality.ApplicationModal);
		        setWindowFlags(Qt.WindowType.WindowCloseButtonHint);
		    	setStyleSheet("background-color:qlinearGradient(x1:0, y1:0, x2:0, y2:1, stop:0 #57360f, " +
					      "stop:1 #9f7340); color: #eeeeff; QLineEdit{background-color:#fffcca;border-radius:7px;" +
					      "border:1px solid black;padding:3px;color:#836b23}");
			}
			
			private QGroupBox createGroupBoxInfo()
			{
				createModel();
				 
			    QTreeView tree = new QTreeView();
			    tree.setStyleSheet("background-color:#b8946a;border-radius:2px;" + 
			        			       			"border:1px solid black;color:#836b23");
			    tree.setModel(getModel());
			    tree.setMaximumHeight(200);
			    tree.doubleClicked.connect(this, 
			    		"onDoubleClickTreeViewGameView(com.trolltech.qt.core.QModelIndex)");
				
			    addInformation();
			     
			    QHBoxLayout hb = new QHBoxLayout();
			    hb.addWidget(tree);
			     
		        QGroupBox groupBoxInfo = new QGroupBox("Listado de espas");
				groupBoxInfo.setLayout(hb);
				return groupBoxInfo;
			}
			
			private void addInformation()
			{
				Iterator<InformationSpy> iter = information.getInformations().iterator();
				while(iter.hasNext())
				{
					InformationSpy index = iter.next();
					if(index.isSuccess())
						addDataModel(Names.values()[index.getLocationSpy()].name(), 
							String.valueOf(index.getUsesSpy()), "Exito");
					else
						addDataModel(Names.values()[index.getLocationSpy()].name(), 
								String.valueOf(index.getUsesSpy()), "Fallo");
				}
			}
			
			private void createModel()
		    {
		        model = new QStandardItemModel(0, 3);
		        model.setHeaderData(0, Qt.Orientation.Horizontal, "Territorio");
		        model.setHeaderData(1, Qt.Orientation.Horizontal, "Numero de usos");
		        model.setHeaderData(2, Qt.Orientation.Horizontal, "Estado");
		    }
			
		    private QStandardItemModel getModel()
		    {
		    	return this.model;
		    }
		    
		    private void addDataModel(String territory, String uses, String state) 
		    {
		    	QStandardItemModel model = getModel();
		    	model.insertRow(model.rowCount());
		    	model.setData(model.rowCount() - 1, 0, territory);
		    	model.setData(model.rowCount() - 1, 1, uses);
		    	model.setData(model.rowCount() - 1, 2, state);
		    	
		    	ItemFlags itemFlags = new ItemFlags(new ItemFlag[]{ItemFlag.ItemIsSelectable, ItemFlag.ItemIsEnabled});
		        model.itemFromIndex(model.index(model.rowCount()-1, 0)).setFlags(itemFlags);
		        model.itemFromIndex(model.index(model.rowCount()-1, 1)).setFlags(itemFlags);
		        model.itemFromIndex(model.index(model.rowCount()-1, 2)).setFlags(itemFlags);
		    }
			
			private void createBox()
			{
				QVBoxLayout vbox = new QVBoxLayout();
				vbox.addWidget(createGroupBoxInfo());
				vbox.addLayout(createButtonsLayout());
				this.setLayout(vbox);
			}
			
			private QHBoxLayout createButtonsLayout()
			{
		        QPushButton pushButton = new QPushButton("Aceptar");
		        pushButton.setIcon(new QIcon(CommonConstants.SUCCESS_ICON));
		        pushButton.setMaximumWidth(80);
		        pushButton.clicked.connect(this, "onClickAcept()");
		        
		        QHBoxLayout buttonsLayout = new QHBoxLayout();
		        buttonsLayout.addWidget(pushButton);
		        return buttonsLayout;
			}
			
			private void onDoubleClickTreeViewGameView(QModelIndex index) throws AlreadyInGameException
		    {  
		    	if(information.getSuccessSpy(index.row()))
		    		new MenuInformationSpy(information,index.row());
		    }
			
			@SuppressWarnings("unused")
			private void onClickAcept()
			{
				this.close();
			}
			
		    @Override
		    public void closeEvent(QCloseEvent event)
		    {
		    	if(CommonConstants.DEBUG) System.out.println("Closing window MenuListSpy");
		    	this.close();
		    }
	}
