package ea.presentation;
import java.net.ConnectException;
import java.rmi.RemoteException;
import java.util.UUID;


import ea.domain.Connection;

import ea.exceptions.EmptyInformationException;
import ea.exceptions.NullInformationException;
import ea.presentation.MenuGames;
import exceptions.*;

import com.trolltech.qt.core.*;
import com.trolltech.qt.gui.*;

public class MenuLogin extends QWidget
{
    private QLineEdit user;
    private QLineEdit password;

    public MenuLogin()
    {
        this.createMainWindow();

        this.createLabelUser();
        this.createInputUser();

        this.createLabelPassword();
        this.createInputPassword();

        this.createButtonLogin();
        this.createButtonRegister();

        this.show();
    }

    private void createMainWindow()
    {
        setWindowTitle(CommonConstants.APP_NAME);

        QDesktopWidget qdw = new QDesktopWidget();
        int screen_center_width  = (qdw.width() - CommonConstants.DEFAULT_WIDTH) / 2,
            screen_center_height = (qdw.height() - CommonConstants.DEFAULT_HEIGHT) / 2;
        
        setGeometry(screen_center_width, screen_center_height, 
        		    CommonConstants.DEFAULT_WIDTH, CommonConstants.DEFAULT_HEIGHT);
        setFixedSize(CommonConstants.DEFAULT_WIDTH, CommonConstants.DEFAULT_HEIGHT);

        setWindowIcon(new QIcon(CommonConstants.APP_ICON));

        QPalette palette = new QPalette();
        palette.setBrush(backgroundRole(), new QBrush(new QImage("img/risk_background.png")));

        setPalette(palette);
        
        this.setStyleSheet("QLineEdit{background-color:#fffcca;border-radius:7px;" + 
        			       "border:1px solid black;padding:3px;color:#836b23}");
    }

    private void createLabelUser()
    {
        QLabel label_user = new QLabel(this);
        label_user.setStyleSheet("background-image:url();background-repeat:repeat;");
        label_user.setGeometry(75, 400, 110, 20);
        label_user.setText("Usuario: ");
        label_user.setFont(new QFont("Tahoma", 10, QFont.Weight.Bold.value()));
        label_user.setAlignment(Qt.AlignmentFlag.AlignBottom, Qt.AlignmentFlag.AlignRight);
        label_user.show();
    }

    private void createInputUser()
    {
        this.user = new QLineEdit(this);
        this.user.setStyleSheet("background-image:url();background-repeat:repeat;");
        this.user.setGeometry(200, 400, 110, 20);
    }
    
    private QLineEdit getUser()
    {
    	return  this.user;
    }

    private void createLabelPassword()
    {
        QLabel label_pass = new QLabel(this);
        label_pass.setStyleSheet("background-image:url();background-repeat:repeat;");
        label_pass.setGeometry(75, 425, 110, 20);
        label_pass.setText("Contrase\u00F1a: ");
        label_pass.setFont(new QFont("Tahoma", 10, QFont.Weight.Bold.value()));
        label_pass.setAlignment(Qt.AlignmentFlag.AlignBottom, Qt.AlignmentFlag.AlignRight);
        label_pass.show();
    }

    private void createInputPassword()
    {
        this.password = new QLineEdit(this);
        this.password.setEchoMode(QLineEdit.EchoMode.Password);
        this.password.setStyleSheet("background-image:url();background-repeat:repeat;");
        this.password.setGeometry(200, 425, 110, 20);
    }

    private QLineEdit getPassword()
    {
    	return this.password;
    }
    
    private void createButtonLogin()
    {
        QPushButton login_button = new QPushButton("&Login", this);
        login_button.setGeometry(155, 450, 75, 25);
        login_button.setShortcut("Enter");
        login_button.setIcon(new QIcon("img/ok.png"));
        login_button.clicked.connect(this, "onClickLogin()");
    }
    
    @SuppressWarnings({ "unused" })
	private void onClickLogin()
    {
    	boolean ok = true;
    	String user = this.getUser().text();
    	String pass = this.getPassword().text();
    	
    	if(pass.isEmpty()) 
    	{
    		new Alert("Aviso", "Ha introducido una contrase\u00F1a vacia.");
    		ok = false;
    	}
    	
    	if(user.isEmpty()) 
    	{
    		new Alert("Aviso", "Ha introducido un usuario vacio.");
    		ok = false;
    	}

    	if(ok)
    	{
    		if(CommonConstants.DEBUG) 
        		System.out.println("Usuario:" + user + " Password:" + pass);	
    		
    		try
    		{
    			if(CommonConstants.FAKESERVER)
    				Connection.setHost("localhost");
    			else
    				Connection.setHost("161.67.106.74");
        	}
        	catch(NullInformationException ex)
        	{
                System.err.println(ex.getMessage());
                return;
            }
        	catch(EmptyInformationException ex)
            {
            	System.err.println(ex.getMessage());
                return;
            }
        	
        	try
        	{
	    		UUID result = Connection.loginPlayer(this.getUser().text(), 
	    										 			   this.getPassword().text());
				this.close();
	    		MenuGames menu_games = new MenuGames(result,this.getUser().text());
        	}
        	catch(WrongLoginException e)
        	{
        		new Alert("Error", "Los datos de acceso son incorrectos.");
        	}
        	catch(RemoteException e)
            {
            	new Alert("Error","Causa:\n"+ e.getMessage());
            } 
        	catch (ConnectException e) 
            {
            	new Alert("Error",             			
						"Conexin rechazada.\nException:\n" + e.getLocalizedMessage() +
            			"\n\nCausa:\n" + e.getCause());
			} catch (NullInformationException e) {
	            new Alert("Error fatal", 
            			"Causa:\n" + e.getLocalizedMessage()); 
			}
			catch(EmptyInformationException e)
            {
				new Alert("Error fatal", 
	            			"Causa:\n" + e.getLocalizedMessage()); 
            }
    	}
    }
    
    private void createButtonRegister()
    {
        QPushButton button = new QPushButton("&Registro", this);
        button.setGeometry(237, 450, 75, 25);
        button.setShortcut("R");
        button.setIcon(new QIcon(CommonConstants.KEY_ICON));
        button.clicked.connect(this, "onClickRegister()");
    }
    
	@SuppressWarnings("unused")
	private void onClickRegister()
    {
		MenuRegister menu_register = new MenuRegister();
		this.close();
    }
}