package ea.presentation;

import java.rmi.RemoteException;

import com.trolltech.qt.core.Qt;
import com.trolltech.qt.gui.QCloseEvent;
import com.trolltech.qt.gui.QDesktopWidget;
import com.trolltech.qt.gui.QGridLayout;
import com.trolltech.qt.gui.QGroupBox;
import com.trolltech.qt.gui.QHBoxLayout;
import com.trolltech.qt.gui.QIcon;
import com.trolltech.qt.gui.QLabel;
import com.trolltech.qt.gui.QPushButton;
import com.trolltech.qt.gui.QSpinBox;
import com.trolltech.qt.gui.QVBoxLayout;
import com.trolltech.qt.gui.QWidget;

import ea.domain.Names;
import ea.domain.QueryAttack;
import exceptions.GameNotFoundException;
import exceptions.InvalidSessionException;
import exceptions.InvalidTerritoryException;

public class MenuMissile extends QWidget {
	private Match father;
	private QLabel numAntiMissilesTerritory;
	private QWidget numICBMsTerritory;
	private QWidget numMissilesTerritory;
	private QLabel numCannonsTerritoryThreeUses;
	private QLabel numCannonsTerritoryTwoUses;
	private QWidget numCannonsTerritoryOneUse;
	private QWidget numSoldiersTerritory;

	private QSpinBox spinBoxMissile;

	public MenuMissile(Match father) {
		this.father = father;
		this.createMainWindow();
		this.createBoxAttack();
		show();
	}

	private void createMainWindow() {
		setWindowIcon(new QIcon(CommonConstants.APP_ICON));
		setWindowTitle(CommonConstants.APP_NAME + " - Enviar ICBM");
		QDesktopWidget qdw = new QDesktopWidget();
		int screen_center_width = (qdw.width() - 250) / 2, screen_center_height = (qdw
				.height() - 100) / 2;
		setGeometry(screen_center_width, screen_center_height, 250, 100);

		setWindowModality(Qt.WindowModality.ApplicationModal);
		setWindowFlags(Qt.WindowType.WindowCloseButtonHint);
		setStyleSheet("background-color:qlinearGradient(x1:0, y1:0, x2:0, y2:1, stop:0 #57360f, "
				+ "stop:1 #9f7340); color: #eeeeff; QLineEdit{background-color:#fffcca;border-radius:7px;"
				+ "border:1px solid black;padding:3px;color:#836b23}");
	}

	private int getIdTerritory() {
		return this.father.getPendingAction().getOriginTerritory();
	}

	private String getNumSoldiers(int idTerritory) {
		return String.valueOf(this.father.getMapGame().getMapTerritories()
				.getNumSoldiersTerritorySelected(idTerritory));
	}

	private int[] getNumCannons(int idTerritory) {
		return this.father.getMapGame().getMapTerritories()
				.getNumCannonsTerritorySelected(idTerritory);
	}

	private String getNumMissiles(int idTerritory) {
		return String.valueOf(this.father.getMapGame().getMapTerritories()
				.getNumMissilesTerritorySelected(idTerritory));
	}

	private String getNumICBMs(int idTerritory) {
		return String.valueOf(this.father.getMapGame().getMapTerritories()
				.getNumICBMsTerritorySelected(idTerritory));
	}

	private QLabel sLabel(String data) { // Create a simple label, without
											// background color associated
		QLabel label = new QLabel(data);
		label.setStyleSheet("background-color:none;");
		return label;
	}

	private void createBoxAttack() {
		QVBoxLayout vbox = new QVBoxLayout();
		vbox.addStretch(1);
		vbox.addWidget(createGroupBoxUp());
		vbox.addWidget(createGroupBoxAttack());
		vbox.addLayout(createButtonsLayout());

		this.setLayout(vbox);
	}

	private QGroupBox createGroupBoxUp() {
		QGridLayout grid = new QGridLayout(this);

		// Soldiers row
		grid.addWidget(sLabel("N soldados"), 0, 0);

		this.numSoldiersTerritory = sLabel(new Integer(
				getNumSoldiers(this.father.getPendingAction()
						.getOriginTerritory())).toString());
		grid.addWidget(this.numSoldiersTerritory, 0, 1,
				Qt.AlignmentFlag.AlignRight);

		// Cannons 1 use row
		grid.addWidget(sLabel("N caones de 1 uso"), 1, 0);

		this.numCannonsTerritoryOneUse = sLabel(String
				.valueOf(getNumCannons(this.father.getPendingAction()
						.getOriginTerritory())[0]));
		grid.addWidget(this.numCannonsTerritoryOneUse, 1, 1,
				Qt.AlignmentFlag.AlignRight);

		// Cannons 2 uses row
		grid.addWidget(sLabel("N caones de 2 usos"), 2, 0);

		this.numCannonsTerritoryTwoUses = sLabel(String
				.valueOf(getNumCannons(this.father.getPendingAction()
						.getOriginTerritory())[1]));
		grid.addWidget(this.numCannonsTerritoryTwoUses, 2, 1,
				Qt.AlignmentFlag.AlignRight);

		// Cannons 3 uses row
		grid.addWidget(sLabel("N caones de 3 usos"), 3, 0);

		this.numCannonsTerritoryThreeUses = sLabel(String
				.valueOf(getNumCannons(this.father.getPendingAction()
						.getOriginTerritory())[2]));
		grid.addWidget(numCannonsTerritoryThreeUses, 3, 1,
				Qt.AlignmentFlag.AlignRight);

		// Missiles row
		grid.addWidget(sLabel("N misiles"), 4, 0);

		this.numMissilesTerritory = sLabel(new Integer(
				getNumMissiles(this.father.getPendingAction()
						.getOriginTerritory())).toString());
		grid.addWidget(this.numMissilesTerritory, 4, 1,
				Qt.AlignmentFlag.AlignRight);

		// Missiles ICBM row
		grid.addWidget(sLabel("N misiles ICBM"), 5, 0);

		this.numICBMsTerritory = sLabel(new Integer(
				getNumICBMs(this.father.getPendingAction()
						.getOriginTerritory())).toString());
		grid.addWidget(this.numICBMsTerritory, 5, 1,
				Qt.AlignmentFlag.AlignRight);

		// Anti Missiles row
		grid.addWidget(sLabel("N anti misiles"), 6, 0);

		this.numAntiMissilesTerritory = sLabel(new Integer(
				getNumMissiles(this.father.getPendingAction()
						.getOriginTerritory())).toString());
		grid.addWidget(this.numAntiMissilesTerritory, 6, 1,
				Qt.AlignmentFlag.AlignRight);

		QGroupBox groupBox = new QGroupBox(
				"Informacin del arsenal personal de "
						+ Names.values()[this.father.getPendingAction()
								.getOriginTerritory()]);
		groupBox.setLayout(grid);
		return groupBox;
	}

	private QGroupBox createGroupBoxAttack() {
		QGridLayout grid = new QGridLayout(this);

		spinBoxMissile = new QSpinBox();
		spinBoxMissile.setMinimum(1);
		spinBoxMissile.setMaximum(Integer.parseInt(getNumMissiles(this.getIdTerritory())));

		// ICBM row
		grid.addWidget(sLabel("Misiles"), 1, 0);
		grid.addWidget(spinBoxMissile, 1, 3);

		QGroupBox groupBox = new QGroupBox("Atacar a "
				+ Names.values()[this.father.getPendingAction().getDestinyTerritory()]);
		groupBox.setLayout(grid);
		return groupBox;
	}

	private QHBoxLayout createButtonsLayout() {
		QPushButton pushButton = new QPushButton("Atacar");
		pushButton.setIcon(new QIcon(CommonConstants.ATTACK_ICON));
		pushButton.setMaximumWidth(80);
		pushButton.clicked.connect(this, "onClickAccept()");

		QPushButton buttonExit = new QPushButton("Cancelar");
		buttonExit.setIcon(new QIcon(CommonConstants.CROSS_ICON));
		buttonExit.setMaximumWidth(80);
		buttonExit.clicked.connect(this, "onClickCancel()");

		QHBoxLayout buttonsLayout = new QHBoxLayout();
		buttonsLayout.addWidget(pushButton);
		buttonsLayout.addWidget(buttonExit);
		return buttonsLayout;
	}

	@SuppressWarnings("unused")
	private void onClickAccept() throws RemoteException, GameNotFoundException,
			InvalidSessionException, InvalidTerritoryException {

		if (spinBoxMissile.value() > 0) {
			this.father.getPendingAction().setNumMissils(spinBoxMissile.value());
			QueryAttack attack = new QueryAttack(this.father.getSession(),
					this.father.getIdGame(), this.father.getPendingAction()
							.getOriginTerritory(), this.father
							.getPendingAction().getDestinyTerritory(), 0, 0, 0,
					0, spinBoxMissile.value(), 0, this.father.getMapGame()
							.getMapTerritories());

			attack.sendAttack();

			new Alert("xito", "El envo se realiz correctamente.",
					CommonConstants.SUCCESS_ICON);
			this.close();
		} else {
			new Alert("Aviso", "Debe de elegir al menos un misil para lanzar.");
		}
	}

	@Override
	public void closeEvent(QCloseEvent event) {
		if (CommonConstants.DEBUG)
			System.out.println("Closing window MenuShowInformation");
		this.father.getMapGame().cancelAttack();
		this.close();
	}

	@SuppressWarnings("unused")
	private void onClickCancel() {
		this.close();
	}
}
