package ea.presentation;

import com.trolltech.qt.core.Qt;
import com.trolltech.qt.gui.QCloseEvent;
import com.trolltech.qt.gui.QDesktopWidget;
import com.trolltech.qt.gui.QGridLayout;
import com.trolltech.qt.gui.QGroupBox;
import com.trolltech.qt.gui.QHBoxLayout;
import com.trolltech.qt.gui.QIcon;
import com.trolltech.qt.gui.QLabel;
import com.trolltech.qt.gui.QPushButton;
import com.trolltech.qt.gui.QSpinBox;
import com.trolltech.qt.gui.QVBoxLayout;
import com.trolltech.qt.gui.QWidget;

import domain.Negotiation;
import domain.Player;

import ea.domain.Names;
import ea.domain.QueryPlayers;

public class MenuNegotiate extends QWidget {
	private Match father;
	private QLabel numSoldiersTerritoryDestine;
	private QLabel numMoney;
	private int soldier;
	private int money;
	private int idTerritoryDst;
	QSpinBox spinBoxSoldiers = new QSpinBox();
	QSpinBox spinBoxMoney = new QSpinBox();

	public MenuNegotiate(Match father, int idTerritoryDst) {
		this.father = father;
		this.idTerritoryDst = idTerritoryDst;
		this.createMainWindow();
		this.createBoxNegotiate();
		this.show();
	}

	private void createMainWindow() {
		setWindowIcon(new QIcon(CommonConstants.APP_ICON));
		setWindowTitle("Negociar");
		QDesktopWidget qdw = new QDesktopWidget();
		int screen_center_width = (qdw.width() - 250) / 2, screen_center_height = (qdw
				.height() - 100) / 2;
		setGeometry(screen_center_width, screen_center_height, 250, 100);

		setWindowModality(Qt.WindowModality.ApplicationModal);
		setWindowFlags(Qt.WindowType.WindowCloseButtonHint);
		setStyleSheet("background-color:qlinearGradient(x1:0, y1:0, x2:0, y2:1, stop:0 #57360f, "
				+ "stop:1 #9f7340); color: #eeeeff; QLineEdit{background-color:#fffcca;border-radius:7px;"
				+ "border:1px solid black;padding:3px;color:#836b23}");
	}

	private int getIdTerritory() {
		return idTerritoryDst;
	}

	private String getNumSoldiers(int idTerritory) {
		return String.valueOf(this.father.getMapGame().getMapTerritories()
				.getNumSoldiersTerritorySelected(idTerritory));
	}

	private String getNumMoney(int idTerritory) {
		QueryPlayers query = new QueryPlayers(this.father.getIdGame());
		return String.valueOf(query.getMoney(this.father.getPlayerName()));
	}

	private QLabel sLabel(String data) { // Create a simple label, without
											// background color associated
		QLabel label = new QLabel(data);
		label.setStyleSheet("background-color:none;");
		return label;
	}

	private void createBoxNegotiate() {
		QVBoxLayout vbox = new QVBoxLayout();
		vbox.addStretch(1);
		vbox.addWidget(createGroupBoxUp());
		vbox.addWidget(createGroupBoxAttack());
		vbox.addLayout(createButtonsLayout());

		this.setLayout(vbox);
	}

	private QGroupBox createGroupBoxUp() {
		QGridLayout grid = new QGridLayout(this);

		// Soldiers row
		grid.addWidget(sLabel("N soldados"), 0, 0);

		this.numSoldiersTerritoryDestine = sLabel(getNumSoldiers(this.idTerritoryDst));
		grid.addWidget(this.numSoldiersTerritoryDestine, 0, 1,
				Qt.AlignmentFlag.AlignRight);

		grid.addWidget(sLabel("Dinero acumulado"), 1, 0);
		this.numMoney = sLabel(String.valueOf(getNumMoney(this.idTerritoryDst)));
		grid.addWidget(this.numMoney, 1, 1, Qt.AlignmentFlag.AlignRight);

		QGroupBox groupBox = new QGroupBox(
				"Informacin del arsenal personal de "
						+ Names.values()[getIdTerritory()]);
		groupBox.setLayout(grid);
		return groupBox;
	}

	private QGroupBox createGroupBoxAttack() {
		int idTerritory = this.getIdTerritory();
		QGridLayout grid = new QGridLayout(this);

		spinBoxSoldiers.setMinimum(0);
		spinBoxSoldiers.setMaximum(Integer
				.parseInt(getNumSoldiers(idTerritory)));

		spinBoxMoney.setMinimum(0);
		spinBoxMoney.setMaximum(Integer.parseInt(getNumMoney(idTerritory)));

		// Soldiers row
		grid.addWidget(sLabel("Soldados"), 1, 0);
		grid.addWidget(spinBoxSoldiers, 1, 3);

		// Cannons row
		grid.addWidget(sLabel("Dinero"), 2, 0);
		grid.addWidget(spinBoxMoney, 2, 3);

		QGroupBox groupBox = new QGroupBox("Negociar con: "
				+ Names.values()[getIdTerritory()]);
		groupBox.setLayout(grid);
		return groupBox;
	}

	private QHBoxLayout createButtonsLayout() {
		QPushButton pushButton = new QPushButton("Negociar");
		pushButton.setIcon(new QIcon(CommonConstants.NEGOTIATE_ICON));
		pushButton.setMaximumWidth(80);
		pushButton.clicked.connect(this, "onClickNegotiate()");

		QPushButton buttonExit = new QPushButton("Cancelar");
		buttonExit.setIcon(new QIcon(CommonConstants.CROSS_ICON));
		buttonExit.setMaximumWidth(80);
		buttonExit.clicked.connect(this, "onClickCancel()");

		QHBoxLayout buttonsLayout = new QHBoxLayout();
		buttonsLayout.addWidget(pushButton);
		buttonsLayout.addWidget(buttonExit);
		return buttonsLayout;
	}

	private void onClickNegotiate()
		{
			this.soldier = (int)this.spinBoxSoldiers.value();
			this.money = (int)this.spinBoxMoney.value();
			boolean correct = true;
			if(soldier < 1 && money < 1){ new Alert("Aviso"," Debes negociar con al menos un soldado o una moneda"); correct = false;}
			if(soldier == Integer.parseInt(getNumSoldiers(getIdTerritory())) ){ new Alert("Aviso","No puedes seleccionar todo los soldados"); correct = false;}
			if(money == Integer.parseInt(getNumMoney(getIdTerritory()))){ new Alert("Aviso","No puedes seleccionar todo el dinero"); correct = false; }
			
			if(correct){
				Negotiation negotiate = new Negotiation(this.father.getSession(),this.father.getIdGame(),this.money,this.soldier);
				negotiate.sendNegotiation();
				this.close();
			}
			
			
		}

	@SuppressWarnings("unused")
	private void onClickCancel() {
		this.close();
	}

	@Override
	public void closeEvent(QCloseEvent event) {
		if (CommonConstants.DEBUG)
			System.out.println("Closing window MenuNegotiate");

		// this.father.getMapGame().cancelAttack();
		this.close();
	}
}
