package ea.presentation;

import java.net.ConnectException;
import java.rmi.RemoteException;

import com.trolltech.qt.core.QRect;
import com.trolltech.qt.gui.*;

import ea.domain.Connection;
import ea.exceptions.EmptyInformationException;
import ea.exceptions.NullInformationException;
import ea.presentation.CommonConstants;
import exceptions.UserAlreadyExistsException;

public class MenuRegister extends QWidget 
{
	private QLineEdit password;
	private QLineEdit email;
	private QLineEdit user;
	
	public MenuRegister() 
	{
		createMainWindow();
		        
		QGroupBox groupBox = new QGroupBox(this);
		groupBox.setGeometry(new QRect(25, 250, 461, 221));
		groupBox.setTitle("Formulario de registro");
		        
		QWidget gridLayoutWidget = new QWidget(groupBox);
		gridLayoutWidget.setGeometry(new QRect(20, 20, 421, 181));
		
		QGridLayout gridLayout = new QGridLayout(gridLayoutWidget);

		QLabel userLabel = new QLabel("Usuario *", gridLayoutWidget);
		user = new QLineEdit(gridLayoutWidget);

		QHBoxLayout userLayout = new QHBoxLayout();
		userLayout.addWidget(userLabel);
		userLayout.addWidget(user);

		QLabel emailLabel = new QLabel("E-mail *", gridLayoutWidget);
		email = new QLineEdit(gridLayoutWidget);

		QHBoxLayout emailLayout = new QHBoxLayout();
		emailLayout.addWidget(emailLabel);
		emailLayout.addWidget(email);

		QLabel passLabel = new QLabel("Contrase\u00f1a *", gridLayoutWidget);
		passLabel.setMaximumSize(66, 40);

		password = new QLineEdit(gridLayoutWidget);
		password.setEchoMode(com.trolltech.qt.gui.QLineEdit.EchoMode.Password);
		        
		QHBoxLayout passLayout = new QHBoxLayout();
		passLayout.addWidget(passLabel);
		passLayout.addWidget(password);

		QHBoxLayout buttonBox = new QHBoxLayout();
		
		gridLayout.addLayout(userLayout, 0, 0, 1, 1);
		gridLayout.addLayout(passLayout, 1, 0, 1, 1);
		gridLayout.addLayout(emailLayout, 2, 0, 1, 1);
		gridLayout.addLayout(buttonBox, 3, 0, 1, 1);

		createButtonRegister(buttonBox);
		createButtonBack(buttonBox);
		this.show();
	}
	
	private String getPassword()
	{
		return this.password.text();
	}
	
	private String getEmail()
	{
		return this.email.text();
	}
	
	private String getUser()
	{
		return this.user.text();
	}
	
	private void createMainWindow()
    {
		setWindowTitle(CommonConstants.APP_NAME + " - Registro");

        QDesktopWidget qdw = new QDesktopWidget();
        int screen_center_width  = (qdw.width() - CommonConstants.DEFAULT_WIDTH) / 2,
            screen_center_height = (qdw.height() - CommonConstants.DEFAULT_HEIGHT) / 2;
        
        setGeometry(screen_center_width, screen_center_height, 
        		    CommonConstants.DEFAULT_WIDTH, CommonConstants.DEFAULT_HEIGHT);
        setFixedSize(CommonConstants.DEFAULT_WIDTH, CommonConstants.DEFAULT_HEIGHT);

        setWindowIcon(new QIcon(CommonConstants.APP_ICON));

        QPalette palette = new QPalette();
        palette.setBrush(backgroundRole(), new QBrush(new QImage("img/risk_background.png")));

        setPalette(palette);
        
        this.setStyleSheet("QLineEdit{background-color:#fffcca;border-radius:7px;" + 
        			       "border:1px solid black;padding:3px;color:#836b23}");
    }

	private void createButtonRegister(QHBoxLayout buttonBox) 
	{
	    QPushButton button = new QPushButton("&Registrarse", this);
	    button.setShortcut("R");
	    button.setIcon(new QIcon(CommonConstants.KEY_ICON));
	    button.clicked.connect(this, "onClickRegister()");
	    buttonBox.addWidget(button);
	}
	
	private void createButtonBack(QHBoxLayout buttonBox) 
	{
	    QPushButton button = new QPushButton("&Volver", this);
	    button.setShortcut("V");
	    button.setIcon(new QIcon(CommonConstants.BACK_ICON));
	    button.clicked.connect(this, "onClickBack()");
	    buttonBox.addWidget(button);
	}

	@SuppressWarnings("unused")
	private void onClickRegister()
	{
		boolean ok = true;
		String user = getUser();
		String email = getEmail();
		String pass = getPassword();

		if(pass.isEmpty()) 
		{
			new Alert("Aviso", "Ha introducido una contrase\u00F1a vacia.");
			ok = false;
		}

		if(user.isEmpty()) 
		{
			new Alert("Aviso", "Ha introducido un usuario vacio.");
			ok = false;
		}

		if(email.isEmpty()) 
		{
			new Alert("Aviso", "Ha introducido un email vacio.");
			ok = false;
		}

		if(ok) 
		{
			try
			{
				Connection.getConnection();
			}
			catch(NullInformationException ex)
        	{
				//Not create connection yet.
				try
	    		{
	    			if(CommonConstants.FAKESERVER)
	    				Connection.setHost("localhost");
	    			else
	    				Connection.setHost("161.67.106.74");
	        	}
	        	catch(NullInformationException e)
	        	{
	                System.err.println(e.getMessage());
	                return;
	            }
	        	catch(EmptyInformationException e)
	            {
	        		System.err.println(e.getMessage());
	                return;
	            }
            }
        	catch(EmptyInformationException ex)
            {
        		//Not create connection yet.
				try
	    		{
	    			if(CommonConstants.FAKESERVER)
	    				Connection.setHost("localhost");
	    			else
	    				Connection.setHost("161.67.106.74");
	        	}
	        	catch(NullInformationException e)
	        	{
	                System.err.println(e.getMessage());
	                return;
	            }
	        	catch(EmptyInformationException e)
	            {
	        		System.err.println(e.getMessage());
	                return;
	            }
            }
			
        	//Already created connection
			try
			{
				Connection.registerPlayer(user, pass, email);
				new Alert("xito", "El registro del usuario se ha realizado con xito.", CommonConstants.SUCCESS_ICON);
				MenuLogin menu_login = new MenuLogin();
				this.close();
			} 
			catch(UserAlreadyExistsException e) 
			{
				new Alert("Error", "El usuario ya existe.");
			}
			catch(RemoteException e)
            {
            	new Alert("Error","Causa:\n"+ e.getMessage());
            } 
        	catch (ConnectException e) 
            {
            	new Alert("Error",             			
						"Conexin rechazada.\nException:\n" + e.getLocalizedMessage() +
            			"\n\nCausa:\n" + e.getCause());
			} 
        	catch (NullInformationException e) {
        		new Alert("Error fatal", 
        			"Causa:\n" + e.getLocalizedMessage()); 
        	}
        	catch(EmptyInformationException e)
        	{
        		new Alert("Error fatal", 
            		"Causa:\n" + e.getLocalizedMessage()); 
        	}
		}
	}

	@SuppressWarnings("unused")
	private void onClickBack() 
	{
		new MenuLogin();
		this.close();
	}
}