package ea.presentation;

import com.trolltech.qt.core.Qt;
import com.trolltech.qt.gui.QCloseEvent;
import com.trolltech.qt.gui.QDesktopWidget;
import com.trolltech.qt.gui.QGridLayout;
import com.trolltech.qt.gui.QGroupBox;
import com.trolltech.qt.gui.QHBoxLayout;
import com.trolltech.qt.gui.QIcon;
import com.trolltech.qt.gui.QLabel;
import com.trolltech.qt.gui.QPixmap;
import com.trolltech.qt.gui.QPushButton;
import com.trolltech.qt.gui.QVBoxLayout;
import com.trolltech.qt.gui.QWidget;

import ea.domain.Names;

public class MenuShowInformation extends QWidget 
{
			private Match father;
			private int idTerritory;
			
			public MenuShowInformation(Match father, int idTerritory)
			{
				this.idTerritory = idTerritory;
				this.father = father;
				this.createMainWindow();
				this.createBoxBuy();
				
				show();
			}
			
			private int getIdTerritory()
			{
				return idTerritory;
			}
			
			private String getNumSoldiers(int idTerritory)
			{
				return String.valueOf(this.father.getMapGame().getMapTerritories().getNumSoldiersTerritorySelected(idTerritory));
			}
			
			private int[] getNumCannons(int idTerritory)
			{
				return this.father.getMapGame().getMapTerritories().getNumCannonsTerritorySelected(idTerritory);
			}
			
			private String getNumMissiles(int idTerritory)
			{
				return String.valueOf(this.father.getMapGame().getMapTerritories().getNumMissilesTerritorySelected(idTerritory));
			}
				
			private String getNumICBMs(int idTerritory)
			{
				return String.valueOf(this.father.getMapGame().getMapTerritories().getNumICBMsTerritorySelected(idTerritory));
			}
				
			private String getNumAntiMissiles(int idTerritory)
			{
				return String.valueOf(this.father.getMapGame().getMapTerritories().getNumAntiMissilesTerritorySelected(idTerritory));
			}
			
			private void createMainWindow()
			{
				setWindowIcon(new QIcon(CommonConstants.APP_ICON));
				setWindowTitle(CommonConstants.APP_NAME + " - Informacin del territorio");
				QDesktopWidget qdw = new QDesktopWidget();
		        int screen_center_width  = (qdw.width() - 250) / 2,
		            screen_center_height = (qdw.height() - 300) / 2;
		        setGeometry(screen_center_width, screen_center_height, 
		        			250, 100);
		        
		        setWindowModality(Qt.WindowModality.ApplicationModal);
		        setWindowFlags(Qt.WindowType.WindowCloseButtonHint);
		    	setStyleSheet("background-color:qlinearGradient(x1:0, y1:0, x2:0, y2:1, stop:0 #57360f, " +
					      "stop:1 #9f7340); color: #eeeeff; QLineEdit{background-color:#fffcca;border-radius:7px;" +
					      "border:1px solid black;padding:3px;color:#836b23}");
			}

			private QLabel sLabel(String data)
			{  // Create a simple label, without background color associated
				QLabel label = new QLabel(data);
				label.setStyleSheet("background-color:none;");
				return label;
			}
			
			private QGroupBox createGroupBoxInfo()
			{
				int idTerritory = getIdTerritory();
				
				QGridLayout grid = new QGridLayout(this);
				
				// Soldiers row
		        grid.addWidget(sLabel("N soldados"), 0, 0);
		        grid.addWidget(sLabel("="), 
		        			   0, 1, Qt.AlignmentFlag.AlignCenter);
		        
		        grid.addWidget(sLabel(new Integer(getNumSoldiers(idTerritory)).toString()), 0, 2, Qt.AlignmentFlag.AlignRight);

		        // Cannons row
		        grid.addWidget(sLabel("N caones"), 1, 0);
		        grid.addWidget(sLabel("="), 
		        			   1, 1, Qt.AlignmentFlag.AlignCenter);
		        
				int []numCannons = getNumCannons(idTerritory);
				
		        grid.addWidget(sLabel(new Integer(String.valueOf(numCannons[0])).toString()), 1, 2, Qt.AlignmentFlag.AlignRight);
		        grid.addWidget(sLabel("de 1 uso,"), 1, 3, Qt.AlignmentFlag.AlignLeft);
		        grid.addWidget(sLabel(new Integer(String.valueOf(numCannons[1])).toString()), 1, 4, Qt.AlignmentFlag.AlignLeft);
		        grid.addWidget(sLabel("de 2 usos,"), 1, 5, Qt.AlignmentFlag.AlignLeft);
		        grid.addWidget(sLabel(new Integer(String.valueOf(numCannons[2])).toString()), 1, 6, Qt.AlignmentFlag.AlignLeft);
		        grid.addWidget(sLabel("de 3 usos"), 1, 7, Qt.AlignmentFlag.AlignLeft);

		        // Missiles row
		        grid.addWidget(sLabel("N misiles"), 2, 0);
		        grid.addWidget(sLabel("="), 
		        			   2, 1, Qt.AlignmentFlag.AlignCenter);
		        
		        grid.addWidget(sLabel(new Integer(getNumMissiles(idTerritory)).toString()), 2, 2, Qt.AlignmentFlag.AlignRight);

		        // MissilesICBS row
		        grid.addWidget(sLabel("N misilesICBS"), 3, 0);
		        grid.addWidget(sLabel("="), 
		        			   3, 1, Qt.AlignmentFlag.AlignCenter);

		        grid.addWidget(sLabel(new Integer(getNumICBMs(idTerritory)).toString()), 3, 2, Qt.AlignmentFlag.AlignRight);

		        // Antimissiles row
		        grid.addWidget(sLabel("N antimisiles"), 4, 0);
		        grid.addWidget(sLabel("="), 
		        			   4, 1, Qt.AlignmentFlag.AlignCenter);
		        
		        grid.addWidget(sLabel(new Integer(getNumAntiMissiles(idTerritory)).toString()), 4, 2, Qt.AlignmentFlag.AlignRight);
		        
		        QGroupBox groupBoxInfo = new QGroupBox("Informacin del arsenal para " + Names.values()[idTerritory]);
				groupBoxInfo.setLayout(grid);
				return groupBoxInfo;
			}
			
			private void createBoxBuy()
			{
				QHBoxLayout hb = new QHBoxLayout();
				hb.addWidget(createImage());
				hb.addWidget(createGroupBoxInfo());
				
				QVBoxLayout vbox = new QVBoxLayout();
				vbox.addLayout(hb);
				vbox.addLayout(createButtonsLayout());
				this.setLayout(vbox);
			}

			private QLabel createImage()
			{
				QLabel label = new QLabel();
				QPixmap img = new QPixmap("img/zone/" + String.valueOf(getIdTerritory()+1) + ".png");
				label.setPixmap(img.scaled(200, 200));
				label.setMinimumWidth(200);
				label.setMinimumHeight(200);
				return label;
			}
			
			private QHBoxLayout createButtonsLayout()
			{
		        QPushButton pushButton = new QPushButton("Aceptar");
		        pushButton.setIcon(new QIcon(CommonConstants.SUCCESS_ICON));
		        pushButton.setMaximumWidth(80);
		        pushButton.clicked.connect(this, "onClickAcept()");
		        
		        QHBoxLayout buttonsLayout = new QHBoxLayout();
		        buttonsLayout.addWidget(pushButton);
		        return buttonsLayout;
			}
			
			@SuppressWarnings("unused")
			private void onClickAcept()
			{
				this.close();
			}
			
		    @Override
		    public void closeEvent(QCloseEvent event)
		    {
		    	if(CommonConstants.DEBUG) System.out.println("Closing window MenuShowInformation");
		    	this.close();
		    }
	}
