package ea.presentation;

import com.trolltech.qt.core.Qt;
import com.trolltech.qt.gui.QCloseEvent;
import com.trolltech.qt.gui.QDesktopWidget;
import com.trolltech.qt.gui.QGridLayout;
import com.trolltech.qt.gui.QGroupBox;
import com.trolltech.qt.gui.QHBoxLayout;
import com.trolltech.qt.gui.QIcon;
import com.trolltech.qt.gui.QLabel;
import com.trolltech.qt.gui.QPushButton;
import com.trolltech.qt.gui.QSpinBox;
import com.trolltech.qt.gui.QVBoxLayout;
import com.trolltech.qt.gui.QWidget;

import domain.Negotiation;
import domain.Player;

import ea.domain.Names;
import ea.domain.QueryMoney;
import ea.domain.QueryPlayers;

public class NegotiationNotify extends QWidget {
	private Match father;
	private QLabel numSoldiersTerritory;
	private QLabel numMoney;
	private int soldiers;
	private int money;
	private int idTerritoryDst;
	private int idTerritorySrc;

	public NegotiationNotify(Match father, int idTerritoryDst, int idTerritorySrc, int money, int soldiers) {
		this.father = father;
		this.idTerritoryDst = idTerritoryDst;
		this.idTerritorySrc = idTerritorySrc;
		this.money = money;
		this.soldiers = soldiers;
		this.createMainWindow();
		this.createBoxNegotiate();
		this.show();
	}

	private void createMainWindow() {
		setWindowIcon(new QIcon(CommonConstants.APP_ICON));
		setWindowTitle("Peticin Negociacin");
		QDesktopWidget qdw = new QDesktopWidget();
		int screen_center_width = (qdw.width() - 250) / 2, screen_center_height = (qdw
				.height() - 100) / 2;
		setGeometry(screen_center_width, screen_center_height, 250, 100);

		setWindowModality(Qt.WindowModality.ApplicationModal);
		setWindowFlags(Qt.WindowType.WindowCloseButtonHint);
		setStyleSheet("background-color:qlinearGradient(x1:0, y1:0, x2:0, y2:1, stop:0 #57360f, "
				+ "stop:1 #9f7340); color: #eeeeff; QLineEdit{background-color:#fffcca;border-radius:7px;"
				+ "border:1px solid black;padding:3px;color:#836b23}");
	}

	private int getIdTerritory() {
		return idTerritoryDst;
	}

	private String getNumSoldiers(int idTerritory) {
		return String.valueOf(this.father.getMapGame().getMapTerritories()
				.getNumSoldiersTerritorySelected(idTerritory));
	}

	private String getNumMoney(int idTerritory) {
		QueryPlayers query = new QueryPlayers(this.father.getIdGame());
		return String.valueOf(query.getMoney(this.father.getPlayerName()));
	}

	private QLabel sLabel(String data) { // Create a simple label, without
											// background color associated
		QLabel label = new QLabel(data);
		label.setStyleSheet("background-color:none;");
		return label;
	}

	private void createBoxNegotiate() {
		QVBoxLayout vbox = new QVBoxLayout();
		vbox.addStretch(1);
		vbox.addWidget(createGroupBoxUp());
		vbox.addLayout(createButtonsLayout());

		this.setLayout(vbox);
	}

	private QGroupBox createGroupBoxUp() {
		QGridLayout grid = new QGridLayout(this);

		// Soldiers row
		grid.addWidget(sLabel("N soldados ofrecidos"), 0, 0);

		this.numSoldiersTerritory = sLabel(String.valueOf(this.soldiers));
		grid.addWidget(this.numSoldiersTerritory, 0, 1,
				Qt.AlignmentFlag.AlignRight);

		grid.addWidget(sLabel("Dinero ofrecido: "), 1, 0);
		this.numMoney = sLabel(String.valueOf(this.money));
		grid.addWidget(this.numMoney, 1, 1, Qt.AlignmentFlag.AlignRight);

		QGroupBox groupBox = new QGroupBox(
				"Informacin del arsenal personal de "
						+ Names.values()[getIdTerritory()]);
		groupBox.setLayout(grid);
		return groupBox;
	}

	

	private QHBoxLayout createButtonsLayout() {
		QPushButton pushButton = new QPushButton("Si");
		pushButton.setIcon(new QIcon(CommonConstants.NEGOTIATE_ICON));
		pushButton.setMaximumWidth(80);
		pushButton.clicked.connect(this, "onClickNegotiate()");

		QPushButton buttonExit = new QPushButton("No");
		buttonExit.setIcon(new QIcon(CommonConstants.CROSS_ICON));
		buttonExit.setMaximumWidth(80);
		buttonExit.clicked.connect(this, "onClickCancel()");

		QHBoxLayout buttonsLayout = new QHBoxLayout();
		buttonsLayout.addWidget(pushButton);
		buttonsLayout.addWidget(buttonExit);
		return buttonsLayout;
	}

	private void onClickNegotiate()
		{
			//Acepto la negociacin
			//Incremento con lo que me ofrecen.
			QueryMoney queryM = new QueryMoney(this.father.getIdGame(),this.father.getPlayerName());
			QueryPlayers queryP = new QueryPlayers(this.father.getIdGame());
			int total = queryP.getMoney(this.father.getPlayerName());
			queryM.sumNegotiationMoney(total,this.money);
			//decremento el del usuario que ataca
			String name = this.father.getMapGame().getMapTerritories().getNamebyIdTerritory(this.idTerritoryDst);
			QueryMoney queryMdst = new QueryMoney(this.father.getIdGame(),name);
			total = queryP.getMoney(name);
			queryMdst.lessNegotiationMoney(total,this.money);
			
			//Incremento los soldados
			this.father.getMapGame().getMapTerritories().lessSoldier(this.idTerritoryDst, this.soldiers);
			this.father.getMapGame().getMapTerritories().sumSoldier(this.idTerritorySrc, this.soldiers);
			
		}

	@SuppressWarnings("unused")
	private void onClickCancel() {
		//Resolver el ataque funcin hecha por Rober.
		this.close();
	}

	@Override
	public void closeEvent(QCloseEvent event) {
		if (CommonConstants.DEBUG)
			System.out.println("Closing window MenuNegotiate");

		// this.father.getMapGame().cancelAttack();
		this.close();
	}
}