package ea.presentation;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.UUID;

import com.trolltech.qt.core.QSignalMapper;
import com.trolltech.qt.core.Qt;
import com.trolltech.qt.gui.QColor;
import com.trolltech.qt.gui.QHBoxLayout;
import com.trolltech.qt.gui.QImage;
import com.trolltech.qt.gui.QLabel;
import com.trolltech.qt.gui.QMouseEvent;
import com.trolltech.qt.gui.QPixmap;
import com.trolltech.qt.gui.QWidget;

import ea.domain.Map;
import ea.domain.Names;
import ea.domain.QueryPlayers;
import ea.libs.Constants;
import ea.presentation.CommonConstants;

public class QMap extends QWidget {
	public Signal1<QMouseEvent> clicked = new Signal1<QMouseEvent>();

	private Match father;
	@SuppressWarnings("unused")
	private QSignalMapper signalMapper;

	private QImage mapImage;
	private Map mapTerritories;

	private QLabel updateInfoLabel;

	private QLabel labelMap;

	private ArrayList<QLabel> cross;
	private ArrayList<QLabel> coffers;
	private QLabel territoryBuyArsenal;

	public QMap(UUID game, Match presentation) {
		this.father = presentation;
		updateInfoLabel = presentation.getUpdateInfoGame();
		this.cross = new ArrayList<QLabel>();
		this.coffers = new ArrayList<QLabel>();
		this.signalMapper = new QSignalMapper(this);

		this.mapTerritories = new Map(game);

		QPixmap map = new QPixmap("img/risk_map.png");
		mapImage = map.toImage();

		// signalMapper.mappedString.connect(this, "clicked(String)");
		this.clicked.connect(this, "mousePressEvent(QMouseEvent)");

		labelMap = new QLabel();
		labelMap.setPixmap(map);
		labelMap.setFixedHeight(542);
		labelMap.setFixedWidth(1058);

		QHBoxLayout hb = new QHBoxLayout();
		hb.addWidget(labelMap);
		setLayout(hb);
		initializeTerritories(this.mapTerritories.myTerritories(this.father
				.getPlayerName()));
	}

	public Map getMapTerritories() {
		return mapTerritories;
	}

	public void cancelBuyArsenal() {
		territoryBuyArsenal.close();
		this.father.onClickCancelAction();
	}

	public void cancelAttack() {
		this.father.onClickCancelAction();
	}

	public void cancelMove() {
		resetMap();
		this.father.onClickCancelAction();
	}

	public void aceptInformation() {
		this.father.onClickCancelAction();
	}

	public void initializeTerritories() {
		this.initializeTerritories(this.mapTerritories
				.myTerritories(this.father.getPlayerName()));
	}

	private void initializeTerritories(ArrayList<Integer> myIds) {

		Iterator<Integer> iter = myIds.iterator();
		while (iter.hasNext()) {
			int idTerritory = iter.next();
			coffers.add(setIcon(CommonConstants.OWNER_ICON,
					Constants.coordinates[idTerritory].getX(),
					Constants.coordinates[idTerritory].getY()));
		}
	}

	public void clicked(String data) {
		System.out.println("Clicked! " + data);
	}

	private void setBackgroundMap(String name) {
		QPixmap map = new QPixmap(name);
		mapImage = map.toImage();
		labelMap.setPixmap(map);
		labelMap.show();
	}

	@Override
	protected void mousePressEvent(QMouseEvent e) {
		if (CommonConstants.DEBUG)
			System.out.println("QMap Evento:" + e.type() + " " + e.button()
					+ " " + e.buttons());

		if (e.button() == Qt.MouseButton.LeftButton) {
			if (father.getPendingAction() != null) { // There action pending
				QColor color = new QColor(mapImage.pixel(e.x() - 5, e.y() - 10));

				if (CommonConstants.DEBUG)
					System.out.println("Color(A, R, G, B): (" + color.alpha()
							+ "," + color.red() + "," + color.green() + ","
							+ color.blue() + ")");

				Names territoryName = mapTerritories
						.searchTerritoryName(color.alpha(), color.red(),
								color.green(), color.blue());
				int territorySelected = mapTerritories.getSelectedTerritory();

				if (territoryName != Names.Ninguno) {
					switch (father.getPendingAction().getAction()) {
					case BuySpy:
						switch (father.getPendingAction().getStep()) {
						case 1:
							// The first analyse if territorySelected is owner
							if (mapTerritories.isOwnsTerritory(
									territorySelected, father.getPlayerName())) {
								// Selection of origin territory
								updateInfoLabel
									.setText("2 - Selecciona el territorio que quiere espiar.");
								resetCoffers();
								ArrayList<Integer> allMyTerritories = this.mapTerritories
											.myTerritories(this.father
													.getPlayerName());
								int pos = allMyTerritories
											.indexOf(territorySelected);
								allMyTerritories.remove(pos);

								initializeTerritories(allMyTerritories);

								cross.add(setIcon(
										CommonConstants.FLAG_ICON,
										Constants.coordinates[territorySelected]
										.getX(),
										Constants.coordinates[territorySelected]
										.getY()));

								Iterator<Integer> iter = mapTerritories
								.freeTerritories().iterator();

								while (iter.hasNext()) {
									int idTerritory = iter.next();
									cross.add(setIcon(
											CommonConstants.CROSS_ICON,
											Constants.coordinates[idTerritory]
													.getX(),
											Constants.coordinates[idTerritory]
													.getY()));
								}

								this.father.getPendingAction()
										.setOriginTerritory(
												territorySelected);
								this.father.getPendingAction().setStep(2);
							}
							break;
						case 2:
							ArrayList<Integer> free = mapTerritories
							.freeTerritories();

							ArrayList<Integer> owners = mapTerritories
							.ownersTerritories(father.getPlayerName());

							if (!free.contains(territorySelected)
									&& !owners.contains(territorySelected)) {

								resetMap();

								this.father.getPendingAction()
										.setDestinyTerritory(territorySelected);
								this.father.getPendingAction().setStep(3);
								this.father.buySpy();

								new Alert(
										"xito",
										"Acabas de mandar un espa desde: "
												+ Names.values()[this.father
														.getPendingAction()
														.getOriginTerritory()]
												+ " a: " + territoryName + "\n",
										CommonConstants.SUCCESS_ICON);

								this.father.onClickCancelAction();
							}
							break;
						}
						break;
					case BuyArsenal:
						switch (father.getPendingAction().getStep()) {
						case 1:
							if (mapTerritories.isOwnsTerritory(
									territorySelected, father.getPlayerName())) {
								// Selection of origin territory
								updateInfoLabel
										.setText("2 - Selecciona el arsenal que quiere com-\nprarle.");
								resetCoffers();
								ArrayList<Integer> allMyTerritories = this.mapTerritories
										.myTerritories(this.father
												.getPlayerName());
								int pos = allMyTerritories
										.indexOf(territorySelected);
								allMyTerritories.remove(pos);

								initializeTerritories(allMyTerritories);

								territoryBuyArsenal = setIcon(
										CommonConstants.BUY_ICON,
										Constants.coordinates[territorySelected]
												.getX(),
										Constants.coordinates[territorySelected]
												.getY());

								this.father.getPendingAction()
										.setOriginTerritory(territorySelected);
								this.father.getPendingAction().setStep(2);
								new MenuBuy(this.father);
							}
							break;
						}
						break;
					case Move:
						switch (father.getPendingAction().getStep()) {
						case 1:
							// The first analyse if territorySelected is owner
							if (mapTerritories.isOwnsTerritory(
									territorySelected, father.getPlayerName())) {
								// Selection of origin territory
								boolean restriction = true;
								ArrayList<Integer> own = mapTerritories
										.ownersAdyacentTerritories(
												color.alpha(), color.red(),
												color.green(), color.blue(),
												father.getPlayerName());

								if (own.size() == 0) {
									restriction = false;
									new Alert(
											"Aviso",
											"El territorio seleccionado no tiene territorios contiguos propietarios para mover tropas.");
								}

								if (restriction) {
									updateInfoLabel
											.setText("2 - Selecciona el territorio destino\n del movimiento de tropas.");
									resetCoffers();
									ArrayList<Integer> allMyTerritories = this.mapTerritories
											.myTerritories(this.father
													.getPlayerName());
									int pos = allMyTerritories
											.indexOf(territorySelected);
									allMyTerritories.remove(pos);

									initializeTerritories(allMyTerritories);

									setBackgroundMap("adyacentes/"
											+ territoryName + ".png");

									Iterator<Integer> iter = (mapTerritories
											.freeAdyacentTerritories(
													color.alpha(), color.red(),
													color.green(), color.blue()))
											.iterator();

									while (iter.hasNext()) {
										int idTerritory = iter.next();
										cross.add(setIcon(
												CommonConstants.CROSS_ICON,
												Constants.coordinates[idTerritory]
														.getX(),
												Constants.coordinates[idTerritory]
														.getY()));
									}

									iter = (mapTerritories
											.occupiedAdyacentTerritories(
													color.alpha(), color.red(),
													color.green(), color.blue()))
											.iterator();

									while (iter.hasNext()) {
										int idTerritory = iter.next();
										if (!own.contains(idTerritory))
											cross.add(setIcon(
													CommonConstants.CROSS_ICON,
													Constants.coordinates[idTerritory]
															.getX(),
													Constants.coordinates[idTerritory]
															.getY()));
									}

									this.father.getPendingAction()
											.setOriginTerritory(
													territorySelected);
									this.father.getPendingAction().setStep(2);
								}
							}
							break;
						case 2:
							int origin = this.father.getPendingAction()
									.getOriginTerritory();
							ArrayList<Integer> owners = mapTerritories
									.ownersAdyacentTerritories(
											Constants.colors[origin].getA(),
											Constants.colors[origin].getR(),
											Constants.colors[origin].getG(),
											Constants.colors[origin].getB(),
											father.getPlayerName());

							if (owners.contains(territorySelected)) {
								this.father.getPendingAction()
										.setDestinyTerritory(territorySelected);
								this.father.getPendingAction().setStep(3);
								this.father
										.setUpdateInfoGame("3 - Elija las unidades que desee mover.");

								new MenuMove(this.father);
							}
							break;
						}
						break;
					case Attack:
						switch (father.getPendingAction().getStep()) {
						case 1:
							// The first analyse if territorySelected is owner
							if (mapTerritories.isOwnsTerritory(
									territorySelected, father.getPlayerName())) {
								// Selection of origin territory
								boolean restriction = true;
								Iterator<Integer> adyFake = mapTerritories
										.occupiedAdyacentTerritories(
												color.alpha(), color.red(),
												color.green(), color.blue())
										.iterator();

								ArrayList<Integer> own = mapTerritories
										.ownersAdyacentTerritories(
												color.alpha(), color.red(),
												color.green(), color.blue(),
												father.getPlayerName());
								ArrayList<Integer> adyTrue = new ArrayList<Integer>();
								while (adyFake.hasNext()) {
									int delete = adyFake.next();
									if (!own.contains(delete))
										adyTrue.add(delete);
								}

								if (adyTrue.size() == 0) {
									restriction = false;
									new Alert("Aviso",
											"El territorio seleccionado no tiene territorios contiguos ocupados.");
								}
								
								if(restriction)
								{
									restriction = mapTerritories
										.correctTerritoryOverlandAttack(territorySelected);
									if(restriction == false)
										new Alert("Aviso", "El territorio seleccionado no tiene unidades suficientes" +
												" para realizar un ataque terrestre.");
								}
							
								if (restriction) {
									updateInfoLabel
											.setText("2 - Selecciona el territorio que quiere atacar.");
									resetCoffers();
									ArrayList<Integer> allMyTerritories = this.mapTerritories
											.myTerritories(this.father
													.getPlayerName());
									int pos = allMyTerritories
											.indexOf(territorySelected);
									allMyTerritories.remove(pos);

									initializeTerritories(allMyTerritories);

									setBackgroundMap("adyacentes/"
											+ territoryName + ".png");

									Iterator<Integer> iter = (mapTerritories
											.freeAdyacentTerritories(
													color.alpha(), color.red(),
													color.green(), color.blue()))
											.iterator();

									while (iter.hasNext()) {
										int idTerritory = iter.next();
										cross.add(setIcon(
												CommonConstants.CROSS_ICON,
												Constants.coordinates[idTerritory]
														.getX(),
												Constants.coordinates[idTerritory]
														.getY()));
									}

									this.father.getPendingAction()
											.setOriginTerritory(
													territorySelected);
									this.father.getPendingAction().setStep(2);
								}
							}
							break;
						case 2:
							int origin = this.father.getPendingAction()
									.getOriginTerritory();
							ArrayList<Integer> adyacents = mapTerritories
									.occupiedAdyacentTerritories(
											Constants.colors[origin].getA(),
											Constants.colors[origin].getR(),
											Constants.colors[origin].getG(),
											Constants.colors[origin].getB());

							ArrayList<Integer> owners = mapTerritories
									.ownersAdyacentTerritories(
											Constants.colors[origin].getA(),
											Constants.colors[origin].getR(),
											Constants.colors[origin].getG(),
											Constants.colors[origin].getB(),
											father.getPlayerName());

							if (adyacents.contains(territorySelected)
									&& !owners.contains(territorySelected)) {

								resetMap();
								updateInfoLabel
										.setText("3 - Seleccione el arsenal para atacar.");
								this.father.getPendingAction()
										.setDestinyTerritory(territorySelected);
								this.father.getPendingAction().setStep(3);
								new MenuAttack(this.father);
							}
							break;

						}
						break;
					case BuyTerritory:
						switch (father.getPendingAction().getStep()) {
						case 1:
							// The first analyse if territorySelected is owner
							if (mapTerritories.isOwnsTerritory(
									territorySelected, father.getPlayerName())) {
								// Selection of origin territory
								boolean restriction = true;
								ArrayList<Integer> ad = mapTerritories
										.freeAdyacentTerritories(color.alpha(),
												color.red(), color.green(),
												color.blue());

								if (ad.size() == 0) {
									restriction = false;
									new Alert("Aviso",
											"El territorio seleccionado no tiene territorios contiguos libres.");
								}

								if (restriction) {

									QueryPlayers aux2 = new QueryPlayers(
											father.getIdGame());
									ArrayList<Integer> withMoney = (mapTerritories
											.freeAdyacentTerritoriesToBuy(color
													.alpha(), color.red(),
													color.green(),
													color.blue(),
													aux2.getMoney(father
															.getPlayerName())));
									if (withMoney.size() == 0) {
										restriction = false;
										new Alert(
												"Aviso",
												"El territorio seleccionado tiene territorios contiguos libres,\n pero no tiene "
														+ "dinero suficiente para comprarlos.");
									}
								}

								if (restriction) {
									resetCoffers();
									ArrayList<Integer> allMyTerritories = this.mapTerritories
											.myTerritories(this.father
													.getPlayerName());
									int pos = allMyTerritories
											.indexOf(territorySelected);
									allMyTerritories.remove(pos);

									initializeTerritories(allMyTerritories);

									setBackgroundMap("adyacentes/"
											+ territoryName + ".png");

									Iterator<Integer> iter = (mapTerritories
											.occupiedAdyacentTerritories(
													color.alpha(), color.red(),
													color.green(), color.blue()))
											.iterator();

									ArrayList<Integer> own = (mapTerritories
											.ownersAdyacentTerritories(
													color.alpha(), color.red(),
													color.green(),
													color.blue(),
													this.father.getPlayerName()));
									while (iter.hasNext()) {
										int idTerritory = iter.next();
										if (!own.contains(idTerritory)) {
											cross.add(setIcon(
													CommonConstants.CROSS_ICON,
													Constants.coordinates[idTerritory]
															.getX(),
													Constants.coordinates[idTerritory]
															.getY()));
										}
									}

									QueryPlayers aux2 = new QueryPlayers(
											father.getIdGame());
									Iterator<Integer> notBuy = (mapTerritories
											.freeAdyacentTerritoriesNotToBuy(
													color.alpha(), color.red(),
													color.green(),
													color.blue(),
													aux2.getMoney(father
															.getPlayerName())))
											.iterator();
									while (notBuy.hasNext()) {
										int idTerritory = notBuy.next();
										cross.add(setIcon(
												CommonConstants.CROSS_ICON,
												Constants.coordinates[idTerritory]
														.getX(),
												Constants.coordinates[idTerritory]
														.getY()));
									}

									updateInfoLabel
											.setText("2 - Selecciona el territorio que quiere com-\n prar.");
									this.father.getPendingAction()
											.setOriginTerritory(
													territorySelected);
									this.father.getPendingAction().setStep(2);
								}
							}
							break;
						case 2:
							int origin = this.father.getPendingAction()
									.getOriginTerritory();
							QueryPlayers aux2 = new QueryPlayers(
									father.getIdGame());
							ArrayList<Integer> buy = (mapTerritories
									.freeAdyacentTerritoriesToBuy(
											Constants.colors[origin].getA(),
											Constants.colors[origin].getR(),
											Constants.colors[origin].getG(),
											Constants.colors[origin].getB(),
											aux2.getMoney(father
													.getPlayerName())));

							if (buy.contains(territorySelected)) {
								this.father.getPendingAction()
										.setDestinyTerritory(territorySelected);
								this.father.getPendingAction().setStep(3);
								this.father
										.setUpdateInfoGame("3 - Elija las unidades que desee mover para\n ocupar el territorio.");

								new MenuBuyTerritory(this.father);
							}
							break;
						}
						break;
					case SendICBM:
						switch (father.getPendingAction().getStep()) {
						case 1:
							// The first analyse if territorySelected is owner
							if (mapTerritories.isOwnsTerritory(
									territorySelected, father.getPlayerName())) {
								// Selection of origin territory
								boolean restriction = mapTerritories
										.correctTerritoryICBM(territorySelected);

								if (restriction) {
									updateInfoLabel
											.setText("2 - Selecciona el territorio victima del lanza-\n miento.");
									resetCoffers();
									ArrayList<Integer> allMyTerritories = this.mapTerritories
											.myTerritories(this.father
													.getPlayerName());
									int pos = allMyTerritories
											.indexOf(territorySelected);
									allMyTerritories.remove(pos);

									initializeTerritories(allMyTerritories);

									cross.add(setIcon(
											CommonConstants.FLAG_ICON,
											Constants.coordinates[territorySelected]
													.getX(),
											Constants.coordinates[territorySelected]
													.getY()));

									Iterator<Integer> iter = mapTerritories
											.freeTerritories().iterator();

									while (iter.hasNext()) {
										int idTerritory = iter.next();
										cross.add(setIcon(
												CommonConstants.CROSS_ICON,
												Constants.coordinates[idTerritory]
														.getX(),
												Constants.coordinates[idTerritory]
														.getY()));
									}

									this.father.getPendingAction()
											.setOriginTerritory(
													territorySelected);
									this.father.getPendingAction().setStep(2);
								} else {
									new Alert("Aviso",
											"El territorio seleccionado no tiene ICBM disponibles.");
								}
							}
							break;
						case 2:
							ArrayList<Integer> free = mapTerritories
									.freeTerritories();

							ArrayList<Integer> owners = mapTerritories
									.ownersTerritories(father.getPlayerName());

							if (!free.contains(territorySelected)
									&& !owners.contains(territorySelected)) {

								resetMap();
								updateInfoLabel
										.setText("3 - Seleccione el nmero de ICBM.");
								this.father.getPendingAction()
										.setDestinyTerritory(territorySelected);
								this.father.getPendingAction().setStep(3);
								new MenuICBM(this.father);
							}
							break;
						}
					case SendMissile:
						switch (father.getPendingAction().getStep()) {
						case 1:
							// The first analyse if territorySelected is owner
							if (mapTerritories.isOwnsTerritory(
									territorySelected, father.getPlayerName())) {
								// Selection of origin territory
								boolean restriction = mapTerritories
										.correctTerritoryMissile(territorySelected);

								if (restriction) {
									updateInfoLabel
											.setText("2 - Selecciona el territorio donde lanzar el misil.");
									resetCoffers();

									ArrayList<Integer> allMyTerritories = this.mapTerritories
											.myTerritories(this.father
													.getPlayerName());
									int pos = allMyTerritories
											.indexOf(territorySelected);
									allMyTerritories.remove(pos);

									initializeTerritories(allMyTerritories);

									cross.add(setIcon(
											CommonConstants.FLAG_ICON,
											Constants.coordinates[territorySelected]
													.getX(),
											Constants.coordinates[territorySelected]
													.getY()));

									setBackgroundMap("img/Continents/"
											+ mapTerritories
													.getContinent(territorySelected)
											+ ".png");

									Iterator<Integer> iter = mapTerritories
											.freeTerritoriesContinent(
													mapTerritories
															.getContinent(territorySelected))
											.iterator();

									while (iter.hasNext()) {
										int idTerritory = iter.next();
										cross.add(setIcon(
												CommonConstants.CROSS_ICON,
												Constants.coordinates[idTerritory]
														.getX(),
												Constants.coordinates[idTerritory]
														.getY()));
									}

									this.father.getPendingAction()
											.setOriginTerritory(
													territorySelected);
									this.father.getPendingAction().setStep(2);
								} else {
									new Alert("Aviso",
											"El territorio seleccionado no tiene misiles disponibles.");
								}
							}
							break;
						case 2:

							int origin = this.father.getPendingAction()
									.getOriginTerritory();

							ArrayList<Integer> occupied = mapTerritories
									.occupiedTerritoriesContinent(mapTerritories
											.getContinent(origin));

							ArrayList<Integer> owners = mapTerritories
									.ownersTerritories(father.getPlayerName());

							if (occupied.contains(territorySelected)
									&& !owners.contains(territorySelected)) {

								resetMap();
								updateInfoLabel
										.setText("3 - Seleccione el nmero de misiles.");
								this.father.getPendingAction()
										.setDestinyTerritory(territorySelected);
								this.father.getPendingAction().setStep(3);
								new MenuMissile(this.father);
							}
							break;
						}
						break;
					}
				}
			} else {
				// Show information territory
				QColor color = new QColor(mapImage.pixel(e.x() - 5, e.y() - 10));

				if (CommonConstants.DEBUG)
					System.out.println("Color(A, R, G, B): (" + color.alpha()
							+ "," + color.red() + "," + color.green() + ","
							+ color.blue() + ")");

				Names territoryName = mapTerritories
						.searchTerritoryName(color.alpha(), color.red(),
								color.green(), color.blue());
				int territorySelected = mapTerritories.getSelectedTerritory();

				if (territoryName != Names.Ninguno
						&& mapTerritories.isOwnsTerritory(territorySelected,
								father.getPlayerName())) {
					// Selection of origin territory
					new MenuShowInformation(this.father, territorySelected);
				}
			}
		}
	}

	@SuppressWarnings("rawtypes")
	public void resetCoffers() {
		@SuppressWarnings("rawtypes")
		Iterator i = coffers.iterator();
		while (i.hasNext()) {
			((QLabel) i.next()).close();
		}
		coffers = new ArrayList<QLabel>();
	}

	@SuppressWarnings("rawtypes")
	public void resetMap() {
		setBackgroundMap("img/risk_map.png");

		@SuppressWarnings("rawtypes")
		Iterator i = cross.iterator();
		while (i.hasNext()) {
			((QLabel) i.next()).close();
		}
		cross = new ArrayList<QLabel>();
	}

	public QLabel setIcon(String name, int x, int y) {
		QLabel label = new QLabel(this);
		label.setStyleSheet("QLabel{background-color:none;}");

		label.setPixmap(new QPixmap(name));
		label.setGeometry(x, y, label.geometry().width(), label.geometry()
				.height());
		label.show();
		label.pixmap().dispose();
		return label;
	}
}
