package ea.server;

import java.rmi.Naming;
import java.rmi.Remote;
import java.rmi.RemoteException;
import java.rmi.registry.LocateRegistry;
import java.rmi.server.UnicastRemoteObject;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.UUID;

import com.trolltech.qt.QThread;
import com.trolltech.qt.core.QObject;
import com.trolltech.qt.core.QTimer;
import com.trolltech.qt.gui.QApplication;

import communications.IClient;
import communications.IServer;

import domain.*;
import domain.Territory.Continent;

import ea.domain.Listener;
import ea.libs.Constants;
import ea.presentation.Match;
import exceptions.*;


public class Server extends UnicastRemoteObject implements IServer 
{
	private static final long serialVersionUID = -7880864684661813222L;
	private static Remote callback;

	private static UUID id;
	private static Server server;
	private boolean sendAttack = false;
	
	private UUID game;
	private Territory src;
	private Territory dst;
	private Arsenal arsenal;
	
	
	public Server() throws RemoteException 
	{
	}

	@Override
	public UUID loginUser(String name, String password, Remote callback)
						  throws RemoteException, WrongLoginException 
	{
		if(name.equals("as"))
			throw new WrongLoginException("Usuario incorrecto.");
		UUID id = UUID.randomUUID();
		System.out.println("Hello, world! User: " + name + "Password: "
				+ password + "UUID: " + id);
		
		this.callback=callback;
		return id;
	}

	@Override
	public void registerUser(String name, String password, String email)
			                 throws RemoteException, UserAlreadyExistsException 
	{
		System.out.println("Player " + name + "registered");
	}

	@Override
	public void logoutUser(UUID session) throws RemoteException,
											    InvalidSessionException 
	{
		System.out.println("User logout");
	}

	@Override
	public ArrayList<GameInfo> listGames(UUID Session) throws RemoteException, InvalidSessionException
	{
		System.out.println("Requesting the list of all availables games");
		
		ArrayList<String> users = new ArrayList<String>();
		users.add("Macario");
		users.add("Ismael");
		users.add("Alberto");
		
		ArrayList<Calendar> gameSessions = new ArrayList<Calendar>();
		gameSessions.add(Calendar.getInstance());
		gameSessions.add(Calendar.getInstance());
		gameSessions.add(Calendar.getInstance());

		id= UUID.randomUUID();
		GameInfo g1 = new GameInfo(id, "Partida 1", 
				"La descripcin de la partida 1\nEs una partida ambientada en la guerra colonial.", users, gameSessions, 10, 5, 3, 4);
		GameInfo g2 = new GameInfo(id, "Partida 2", 
				"La descripcin de la partida 2\nEs una partida ambientada en la primera guerra mundial.", users, gameSessions, 15, 5, 3, 4);
		GameInfo g3 = new GameInfo(id, "Partida 3", 
				"La descripcin de la partida 3 la cadena la cadena la cadena la cadena la cadena la cadena la cadena la cadena la cadena la cadena la cadena la cadena la cadena la cadena la cadena la cadena la cadena la cadena la cadena la cadena la cadena la cadena\nEs una partida ambientada en la segunda guerra mundial.", users, gameSessions, 20, 5, 3, 4);
		
		ArrayList<GameInfo> allGameInfo = new ArrayList<GameInfo>();
		allGameInfo.add(g1);
		allGameInfo.add(g2);
		allGameInfo.add(g3);
		
		return allGameInfo;
	}

	@Override
	public UUID createGame(UUID session, GameInfo info) throws RemoteException,
												 InvalidGameInfoException,
												 InvalidSessionException
	{
		UUID id = UUID.randomUUID();
		System.out.println("Create game");
		return id;
	}

	@Override
	public void joinGame(UUID session, UUID game) throws RemoteException,
														 FullGameException, 
														 GameNotFoundException, 
														 InvalidSessionException,
														 AlreadyInGameException
	{
		System.out.println("User join game");
	}

	@Override
	public void resignGame(UUID session, UUID game) throws RemoteException,
														   GameNotFoundException, 
														   InvalidSessionException 
	{
		System.out.println("User resign game");
	}

	@Override
	public Game playGame(UUID session, UUID game) throws RemoteException,
														 GameNotFoundException, 
														 InvalidSessionException,
														 InvalidTimeException 
	{
		ArrayList<String> users = new ArrayList<String>();
		users.add("Macario");
		users.add("Ismael");
		users.add("Alberto");
		
		ArrayList<Calendar> gameSessions = new ArrayList<Calendar>();
		gameSessions.add(Calendar.getInstance());
		gameSessions.add(Calendar.getInstance());
		gameSessions.add(Calendar.getInstance());

		GameInfo g1 = new GameInfo(game, "Partida 1", 
				"Es una partida ambientada en la guerra colonial.", users, gameSessions, 30, 120, 30, 30); 
		
		ArrayList<Player> players = new ArrayList<Player>();
		ArrayList<Spy> spies = new ArrayList<Spy>();
		
		players.add(new Player("Macario",9000,true,true,spies));
		players.add(new Player("Ismael",7000,true,false,spies));
		players.add(new Player("Alberto",50,false,false,spies));
		
		ArrayList<Territory> territories = new ArrayList<Territory>();
		int [] caones = {20,30,40};
		int [] iniciales = {0,0,0};
		int soldados = 2000;
		int missiles = 400;
		int icbms = 100;
		int antimissiles = 300;
		Territory aux = new Territory(0,Continent.Europe,"Macario",soldados,caones,missiles, icbms, antimissiles);
		territories.add(aux);
		
		Territory auxx = new Territory(1,Continent.Europe,"Alberto",soldados,caones,missiles, icbms, antimissiles);
		territories.add(auxx);
		
		Territory aux1 = new Territory(2,Continent.Europe,"Ismael",soldados,caones,missiles, icbms, antimissiles);
		territories.add(aux1);
		
		Territory aux2 = new Territory(3,Continent.Europe,"",0,iniciales,0, 0,0);
		territories.add(aux2);
		
		Territory aux3 = new Territory(4,Continent.Europe,"Macario",soldados,caones,missiles, icbms, antimissiles);
		territories.add(aux3);
		
		Territory aux4 = new Territory(5,Continent.Europe,"",0,iniciales,0, 0,0);
		territories.add(aux4);
		
		Territory aux5 = new Territory(6,Continent.Europe,"Macario",soldados,caones,missiles, icbms, antimissiles);
		territories.add(aux5);
		
		Territory aux6 = new Territory(7,Continent.Asia,"Macario",soldados,caones,missiles, icbms, antimissiles);
		territories.add(aux6);
		
		Territory aux7 = new Territory(8,Continent.Asia,"",0,iniciales,0, 0,0);
		territories.add(aux7);
		
		Territory aux8 = new Territory(9,Continent.Asia,"",0,iniciales,0, 0,0);
		territories.add(aux8);
		
		Territory aux9 = new Territory(10,Continent.Asia,"Ismael",soldados,caones,missiles, icbms, antimissiles);
		territories.add(aux9);
		
		Territory aux10 = new Territory(11,Continent.Asia,"",0,iniciales,0, 0,0);
		territories.add(aux10);
		
		Territory aux11 = new Territory(12,Continent.Asia,"",0,iniciales,0, 0,0);
		territories.add(aux11);
		
		Territory aux12 = new Territory(13,Continent.Asia,"",0,iniciales,0, 0,0);
		territories.add(aux12);
		
		Territory aux13 = new Territory(14,Continent.Asia,"Macario",soldados,caones,missiles, icbms, antimissiles);
		territories.add(aux13);
		
		Territory aux14 = new Territory(15,Continent.Asia,"Alberto",soldados,caones,missiles, icbms, antimissiles);
		territories.add(aux14);
		
		Territory aux15 = new Territory(16,Continent.Asia,"Ismael",soldados,caones,missiles, icbms, antimissiles);
		territories.add(aux15);
		
		Territory aux16 = new Territory(17,Continent.Asia,"",0,iniciales,0, 0,0);
		territories.add(aux16);
		
		Territory aux17 = new Territory(18,Continent.Asia,"",0,iniciales,0, 0,0);
		territories.add(aux17);
		
		Territory aux18 = new Territory(19,Continent.Africa,"",0,iniciales,0, 0,0);
		territories.add(aux18);
		
		Territory aux19 = new Territory(20,Continent.Africa,"",0,iniciales,0, 0,0);
		territories.add(aux19);
		
		Territory aux20 = new Territory(21,Continent.Africa,"",0,iniciales,0, 0,0);
		territories.add(aux20);
		
		Territory aux21 = new Territory(22,Continent.Africa,"",0,iniciales,0, 0,0);
		territories.add(aux21);
		
		Territory aux22 = new Territory(23,Continent.Africa,"Alberto",soldados,caones,missiles, icbms, antimissiles);
		territories.add(aux22);
		
		Territory aux23 = new Territory(24,Continent.Africa,"",0,iniciales,0, 0,0);
		territories.add(aux23);
		
		Territory aux24 = new Territory(25,Continent.NorthAmerica,"",0,iniciales,0, 0,0);
		territories.add(aux24);
		
		Territory aux25 = new Territory(26,Continent.NorthAmerica,"",0,iniciales,0, 0,0);
		territories.add(aux25);
		
		Territory aux26 = new Territory(27,Continent.NorthAmerica,"",0,iniciales,0, 0,0);
		territories.add(aux26);
		
		Territory aux27 = new Territory(28,Continent.NorthAmerica,"",0,iniciales,0, 0,0);
		territories.add(aux27);
		
		Territory aux28 = new Territory(29,Continent.NorthAmerica,"Alberto",soldados,caones,missiles, icbms, antimissiles);
		territories.add(aux28);
		
		Territory aux29 = new Territory(30,Continent.NorthAmerica,"",0,iniciales,0, 0,0);
		territories.add(aux29);
		
		Territory aux30 = new Territory(31,Continent.NorthAmerica,"",0,iniciales,0, 0,0);
		territories.add(aux30);
		
		Territory aux31 = new Territory(32,Continent.NorthAmerica,"",0,iniciales,0, 0,0);
		territories.add(aux31);
		
		Territory aux32 = new Territory(33,Continent.NorthAmerica,"",0,iniciales,0, 0,0);
		territories.add(aux32);
		
		Territory aux33 = new Territory(34,Continent.SouthAmerica,"",0,iniciales,0, 0,0);
		territories.add(aux33);
		
		Territory aux34 = new Territory(35,Continent.SouthAmerica,"",0,iniciales,0, 0,0);
		territories.add(aux34);
		
		Territory aux35 = new Territory(36,Continent.SouthAmerica,"",0,iniciales,0, 0,0);
		territories.add(aux35);
		
		Territory aux36 = new Territory(37,Continent.SouthAmerica,"",0,iniciales,0, 0,0);
		territories.add(aux36);
		
		Territory aux37 = new Territory(38,Continent.Oceania,"",0,iniciales,0, 0,0);
		territories.add(aux37);
		
		Territory aux38 = new Territory(39,Continent.Oceania,"Ismael",soldados,caones,missiles, icbms, antimissiles);
		territories.add(aux38);
		
		Territory aux39 = new Territory(40,Continent.Oceania,"",0,iniciales,0, 0,0);
		territories.add(aux39);
		
		Territory aux40 = new Territory(41,Continent.Oceania,"",0,iniciales,0, 0,0);
		territories.add(aux40);
		
		Game result = null;
		result = new Game(g1, territories, players);
		System.out.println("User play game");
		return result;
	}

	void sendAttack()
	{
		//Hago que en lugar de tu atacar a fulano, te ataque fulano
		if(sendAttack){
			IClient res = (IClient) callback;
			try{
				this.sendAttack = false;
				res.territoryUnderAttack(game,dst,src,arsenal);
			}
			catch(java.lang.NullPointerException e){}
			catch(RemoteException e){}
			catch(GameNotFoundException e){}
			catch(InvalidTerritoryException e){}
		}
	}
	
	
	@Override
	public void quitGame(UUID session, UUID game) throws RemoteException,
														 GameNotFoundException, 
														 InvalidSessionException,
														 InvalidTimeException 
	{
		System.out.println("User quit game");
	}

	@Override
	public void updateGame(UUID session, UUID game, 
						   ArrayList<Player> playerUpdate,
						   ArrayList<Territory> territoryUpdate, EventType event)
						   throws RemoteException, GameNotFoundException,
							      InvalidSessionException, NotCurrentPlayerGameException
	{
		System.out.println("Update game");
	}

	@Override
	public void attackTerritory(UUID session, UUID game, Territory src,
							    Territory dst, Arsenal arsenal) 
								throws RemoteException,
									   GameNotFoundException, 
									   InvalidSessionException,
									   InvalidTerritoryException 
    {
		System.out.println("Attack Territory");
		System.out.println("Origin: "+src.getIdTerritory());
		System.out.println("Destiny: "+dst.getIdTerritory());
		System.out.println("Soldiers: "+arsenal.getSoldiers());
		System.out.println("Cannons: "+arsenal.getCannons());
		System.out.println("Missiles: "+arsenal.getMissiles());
		System.out.println("ICMBs: "+arsenal.getICBMs());
		
		this.game =game;
		this.src = src;
		this.dst = dst;
		this.arsenal = arsenal;
		this.sendAttack = true;
	}
	
	public UUID getGame()
	{
		return game;
	}

	public Territory getSRC()
	{
		return src;
	}
	
	public Territory getDST()
	{
		return dst;
	}
	
	public Arsenal getArsenal()
	{
		return arsenal;
	}
	
	@Override
	public void acceptAttack(UUID session, UUID game) throws RemoteException,
														     GameNotFoundException, 
														     InvalidSessionException 
	{
		System.out.println("Attack acepted");
	}

	@Override
	public void requestedNegotiation(UUID session, UUID game, int money, int soldiers) 
									 throws RemoteException, 
									 		GameNotFoundException,
									 		InvalidSessionException 
	{
		System.out.println("Requested Negotiation");
	}

	@Override
	public void endTurn(UUID session, UUID game) throws RemoteException, 
														InvalidTimeException, 
														InvalidSessionException
	{
		System.out.println("End turn");
	}
	
	public static void main(String args[]) 
	{
		try 
		{
	        QApplication.initialize(args);
	        
			server = new Server();
			LocateRegistry.createRegistry(1000);
			Naming.bind("rmi://localhost:1000/fakeserver", server);

			System.err.println("Server ready");
			
			AdapterQTimer ada = new AdapterQTimer(server);
			QThread runner = new QThread(ada);
			runner.start();
			
			
			
	        QApplication.exec();
		} 
		catch(java.rmi.ServerException e) 
		{
			System.err.println("Server ServerException: " + e.toString());
			e.printStackTrace();
		} 
		catch(java.rmi.server.ExportException e) 
		{
			System.err.println("Server ExportException: " + e.toString());
			e.printStackTrace();
		} 
		catch(Exception e) 
		{
			System.err.println("Server exception: " + e.toString());
			e.printStackTrace();
		}
	}
}