package ea.test;


import java.rmi.RemoteException;
import java.util.UUID;

import junit.framework.TestCase;
import exceptions.UserAlreadyExistsException;
import exceptions.WrongLoginException;
import ea.client.Client;
import ea.libs.domain.Connection;
import ea.libs.exceptions.NullInformationException;
import ea.server.Server;

public class TestCaseAuthentication extends TestCase
{
    // fixtures
    String user, password;
    Server server;
    Connection conn;
    Client cliente;
    UUID session;

    public void setUp()
    {
    	try {
    		cliente = Connection.getConnection(); //Conexion con servidor
    											  //Se devuelve un cliente
    		cliente.registerPlayer("edu", "12345", "edseal20@hotmail.com");
		} catch (UserAlreadyExistsException e) {
			e.printStackTrace();
		} catch (Exception e){
			e.printStackTrace();
		}
    }

    public void testTc5() //Usuario y contrasea vlidas
    {
        try
        {
            user = "edu";
            password = "12345";
            session = null;
            
            session = cliente.loginPlayer(user, password);            
            assertNotNull(session);
            
        } catch (RemoteException re){
        	fail(re.toString());
        } catch (WrongLoginException wle){
        	fail(wle.toString());
        }catch (java.net.ConnectException e) {
			fail(e.toString());
		} catch (NullInformationException e) {
			fail(e.toString());
		}catch(Exception e){
			fail(e.toString());
		}
        
    }
    
    public void testTc1() //Usuario y contrasea no vlidas
    {
        try
        {
            user = "eduseal";
            password = "123456";
            session = null;
            
            session = cliente.loginPlayer(user, password);            
            fail("Se espera WrongLoginException");
            
        } catch (RemoteException re){
        	fail(re.toString());
        } catch (WrongLoginException wle){
        	assertNull(session);
        }catch (java.net.ConnectException e) {
			fail(e.toString());
		} catch (NullInformationException e) {
			fail(e.toString());
		}catch(Exception e){
			fail(e.toString());
		}
        
    }
    
    public void testTc2() //Usuario invlido y contrasea vlida
    {
        try
        {
            user = "eduseal";
            password = "12345";
            session = null;
            
            session = cliente.loginPlayer(user, password);            
            fail("Se espera WrongLoginException");
            
	    } catch (RemoteException re){
	    	fail(re.toString());
	    } catch (WrongLoginException wle){
	    	assertNull(session);
	    }catch (java.net.ConnectException e) {
			fail(e.toString());
		} catch (NullInformationException e) {
			fail(e.toString());
		}catch(Exception e){
			fail(e.toString());
		}
        
    }
    
    public void testTc3() //Usuario no vlido y contrasea vaca
    {
        try
        {
            user = "eduseal";
            password = "";
            session = null;
            
            session = cliente.loginPlayer(user, password);            
            fail("Se espera WrongLoginException y/o NullInformationException");
        } catch (RemoteException re){
        	fail(re.toString());
        } catch (WrongLoginException wle){
        	assertNull(session);
        }catch (java.net.ConnectException e) {
			fail(e.toString());
		} catch (NullInformationException e) {
			assertNull(session);
		}catch(Exception e){
			fail(e.toString());
		}
        
    }
    
    public void testTc4() //Usuario vlido y contrasea no vlida
    {
        try
        {
            user = "edu";
            password = "123456";
            session = null;
            
            session = cliente.loginPlayer(user, password);            
            fail("Se espera WrongLoginException");
        } catch (RemoteException re){
        	fail(re.toString());
        } catch (WrongLoginException wle){
        	assertNull(session);
        }catch (java.net.ConnectException e) {
			fail(e.toString());
		} catch (NullInformationException e) {
			fail(e.toString());
		}catch(Exception e){
			fail(e.toString());
		}
        
    }
    
    public void testTc6() //Usuario vlido y contrasea vaca
    {
        try
        {
            user = "edu";
            password = "";
            session = null;
            
            session = cliente.loginPlayer(user, password);            
            fail("Se espera WrongLoginException y/o NullInformationException.");
        } catch (RemoteException re){
        	fail(re.toString());
        } catch (WrongLoginException wle){
        	assertNull(session);
        }catch (java.net.ConnectException e) {
			fail(e.toString());
		} catch (NullInformationException e) {
			assertNull(session);
		}catch(Exception e){
			fail(e.toString());
		}
        
    }
    
    public void testTc7() //Usuario vaco y contrasea no vlida
    {
        try
        {
            user = "";
            password = "123456";
            session = null;
            
            session = cliente.loginPlayer(user, password);            
            fail("Se espera WrongLoginException y/o NullInformationException.");
        } catch (RemoteException re){
        	fail(re.toString());
        } catch (WrongLoginException wle){
        	assertNull(session);
        }catch (java.net.ConnectException e) {
			fail(e.toString());
		} catch (NullInformationException e) {
			assertNull(session);
		}catch(Exception e){
			fail(e.toString());
		}
        
    }
    
    public void testTc8() //Usuario vaco y contrasea no vlida
    {
        try
        {
            user = "";
            password = "12345";
            session = null;
            
            session = cliente.loginPlayer(user, password);            
            fail("Se espera WrongLoginException y/o NullInformationException.");
        } catch (RemoteException re){
        	fail(re.toString());
        } catch (WrongLoginException wle){
        	assertNull(session);
        }catch (java.net.ConnectException e) {
			fail(e.toString());
		} catch (NullInformationException e) {
			assertNull(session);
		}catch(Exception e){
			fail(e.toString());
		}
        
    }
    
    public void testTc9() //Usuario vaco y contrasea vaca
    {
        try
        {
            user = "";
            password = "";
            session = null;
            
            session = cliente.loginPlayer(user, password);            
            fail("Se espera WrongLoginException y/o NullInformationException.");
        } catch (RemoteException re){
        	fail(re.toString());
        } catch (WrongLoginException wle){
        	assertNull(session);
        }catch (java.net.ConnectException e) {
			fail(e.toString());
		} catch (NullInformationException e) {
			assertNull(session);
		}catch(Exception e){
			fail(e.toString());
		}
        
    }

    public void tearDown()
    {
        cliente = null;
        System.gc();	//Liberar memoria
    }
}
