package ea.test;


import java.net.ConnectException;
import java.rmi.RemoteException;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.UUID;

import junit.framework.TestCase;
import domain.GameInfo;
import ea.client.Client;
import exceptions.InvalidGameInfoException;
import exceptions.InvalidSessionException;
import exceptions.UserAlreadyExistsException;
import exceptions.WrongLoginException;
import ea.libs.domain.Connection;
import ea.libs.exceptions.NullInformationException;
import ea.server.Server;

public class TestCaseCreateGame extends TestCase{

	Server server;
    Connection conn;
    GameInfo info;
    UUID session, game;
    UUID sessionnovalida = null;
    Client cliente;
    GameInfo g1;
    ArrayList<GameInfo> usuario;
    ArrayList<String> users;
    ArrayList<Calendar> gameSessions;
    

    public void setUp()
    {
    	try {
    		cliente = Connection.getConnection();
    		cliente.registerPlayer("edu", "12345", "edseal20@hotmail.com");
    		session = cliente.loginPlayer("edu", "12345");	
    		users = new ArrayList<String>(); //Creamos lista de usuarios
    	    gameSessions = new ArrayList<Calendar>();	//Creamos sesiones de juego
    	    
		} catch (RemoteException e) {
			e.printStackTrace();
		} catch (UserAlreadyExistsException e) {
			e.printStackTrace();
		} catch (ConnectException e) {
			e.printStackTrace();
		} catch (WrongLoginException e) {
			e.printStackTrace();
		} catch (NullInformationException e) {
			e.printStackTrace();
		} catch (Exception e){
			e.printStackTrace();
		}
    }
    
    private boolean buscarUUID(UUID valor, GameInfo[] ajuegos){
    	for(GameInfo g:ajuegos){
    		if(g.getId().equals(valor)) return true;
    	}
    	return false;
    }
    
    private boolean buscarUUID(UUID valor, ArrayList<GameInfo> juegos){
    	GameInfo[] ajuegos = (GameInfo[])juegos.toArray();
    	for(GameInfo g:ajuegos){
    		if(g.getId().equals(valor)) return true;
    	}
    	return false;
    }
    
    private UUID uuidInexistente(ArrayList<GameInfo> juegos){
    	GameInfo[] ajuegos = (GameInfo[])juegos.toArray();
    	UUID valor = UUID.randomUUID();
    	while(buscarUUID(valor,ajuegos)){
    		valor = UUID.randomUUID();
    	}
    	return valor;
    }
    
    public void testTc1(){ //GameInfo vlido y sessin vlida	
    	try{
    		users.add("edu");	//Aadimos un usuario a la lista de usuarios
        	gameSessions.add(Calendar.getInstance());	//Aadimos una sesion de juego
        	ArrayList<GameInfo> juegos = cliente.listGames(session);
        	//Creamos el GameInfo con un uuid no repetido
    		g1 = new GameInfo(uuidInexistente(juegos), "Partida 1", 
    				"La descripcin de la partida 1\nEs una partida ambientada en la guerra colonial.", users, gameSessions, 10, 5, 3, 4);
    		cliente.createGame(session,g1);
    		juegos = cliente.listGames(session);
    		assertTrue(buscarUUID(g1.getId(),juegos)); //Se espera que nuestro uuid de juego
    												   //este aadido en la lista de juegos
    		
    	} catch (RemoteException re) {
			fail(re.toString());
		} catch (InvalidGameInfoException iie) {
			fail(iie.toString());
		}catch (ConnectException e) {
			fail(e.toString());
		} catch (InvalidSessionException e) {
			fail(e.toString());
		} catch (Exception e){
			fail(e.toString());
		}
    }
    
    public void testTc2(){ //GameInfo no vlido y sessin vlida
    	ArrayList<GameInfo> juegos = null;
    	int size = 0;
    	try{
    		users.add("edu");
        	gameSessions.add(Calendar.getInstance());
        	juegos = cliente.listGames(session);
        	juegos.trimToSize();
        	size = juegos.size();
    		g1 = new GameInfo(juegos.get(1).getId(), "Partida 1", 
    				"La descripcin de la partida 1\nEs una partida ambientada en la guerra colonial.", users, gameSessions, 10, 5, 3, 4);
    		cliente.createGame(session,g1);
    		fail("Se espera InvalidGameInfoException");
    		
    		
    	} catch (RemoteException re) {
			fail(re.toString());
		} catch (InvalidGameInfoException iie) {
			try{
				juegos = cliente.listGames(session);
				juegos.trimToSize();
				int size2 = juegos.size();
				assertTrue(size == size2); //Se espera que no se aumente
										   //el tamao de la lista de juegos
			}catch (Exception ee){
				fail(ee.toString());
			}
		}catch (ConnectException e) {
			fail(e.toString());
		} catch (InvalidSessionException e) {
			fail(e.toString());
		} catch (Exception e){
			fail(e.toString());
		}
    }
    
    public void testTc3(){ //GameInfo vlido y sessin no vlida
    	ArrayList<GameInfo> juegos = null;
    	try{
    		users.add("edu");
        	gameSessions.add(Calendar.getInstance());
        	juegos = cliente.listGames(session);
    		g1 = new GameInfo(uuidInexistente(juegos), "Partida 1", 
    				"La descripcin de la partida 1\nEs una partida ambientada en la guerra colonial.", users, gameSessions, 10, 5, 3, 4);
    		cliente.createGame(sessionnovalida,g1);
    		fail("Se espera InvalidSessionException");
    		
    		
    	} catch (RemoteException re) {
			fail(re.toString());
		} catch (InvalidGameInfoException iie) {
			fail(iie.toString());
		}catch (ConnectException e) {
			fail(e.toString());
		} catch (InvalidSessionException e) {
			try{
				juegos = cliente.listGames(session);
				assertFalse(buscarUUID(g1.getId(),juegos)); //Comprobamos que el
															//uuid no este en la lista de juegos
			}catch (Exception ee){
				fail(ee.toString());
			}
		} catch (Exception e){
			fail(e.toString());
		}
    }
    
    public void testTc4(){ //GameInfo no vlido y sessin no vlida
    	ArrayList<GameInfo> juegos = null;
    	int size = 0;
    	try{
    		users.add("edu");
        	gameSessions.add(Calendar.getInstance());
        	juegos = cliente.listGames(session);
        	juegos.trimToSize();
        	size = juegos.size();
    		g1 = new GameInfo(juegos.get(1).getId(), "Partida 1", 
    				"La descripcin de la partida 1\nEs una partida ambientada en la guerra colonial.", users, gameSessions, 10, 5, 3, 4);
    		cliente.createGame(sessionnovalida,g1);
    		fail("Se espera InvalidGameInfoException y/o InvalidGameInfoException");
    		
    	} catch (RemoteException re) {
			fail(re.toString());
		} catch (InvalidGameInfoException iie) {
			try{
				juegos = cliente.listGames(session);
				juegos.trimToSize();
				int size2 = juegos.size();
				assertTrue(size == size2);
			}catch (Exception ee){
				fail(ee.toString());
			}
		}catch (ConnectException e) {
			fail(e.toString());
		} catch (InvalidSessionException e) {
			try{
				juegos = cliente.listGames(session);
				assertFalse(buscarUUID(g1.getId(),juegos));
			}catch (Exception ee){
				fail(ee.toString());
			}
		} catch (Exception e){
			fail(e.toString());
		}
    }

	public void tearDown() throws Exception {
		cliente = null;
		System.gc();
	}

}
