package ea.test;

import java.net.ConnectException;
import java.rmi.RemoteException;
import java.util.ArrayList;
import java.util.UUID;

import domain.Game;
import domain.GameInfo;
import domain.Player;

import ea.domain.Connection;
import ea.exceptions.NullInformationException;

import exceptions.*;

import junit.framework.TestCase;

public class TestCaseExitGame extends TestCase
{

	// fixtures
	UUID idValidSession, idInvalidSession, idValidGame, idInvalidGame;
	GameInfo gameInfo;
	ArrayList<Player> players;
	Player p;
	ArrayList<String> playersStr;
	Game game;
	ArrayList<GameInfo> gamesInfo;
	ArrayList<String> ps;
	
	public void setUp() 
	{
		this.idValidGame = UUID.randomUUID();
		this.idInvalidGame = UUID.fromString("no valid");
		this.idInvalidSession = UUID.fromString("no valid");
		this.playersStr.add("Luis");
		this.p = new Player();
		this.p.setName("Luis");
		this.gameInfo = new GameInfo(this.idValidGame, "testGame", "testGame", 
				this.playersStr, null, 1, 1, 1, 1);
		
		try {
			Connection.getConnection().registerPlayer("Luis", "1234", "luis@alu");
			this.idValidSession = Connection.getConnection().loginPlayer("Luis", "1234");
			Connection.getConnection().createGame(this.gameInfo);
			Connection.getConnection().joinGame(this.idValidSession, this.idValidGame);
			game = Connection.getConnection().playGame(this.idValidSession, this.idValidGame);
			p.setOnline(true);
		} catch (RemoteException e) {
			e.printStackTrace();
		} catch (InvalidGameInfoException e) {
			e.printStackTrace();
		} catch (InvalidSessionException e) {
			e.printStackTrace();
		} catch (ConnectException e) {
			e.printStackTrace();
		} catch (WrongLoginException e) {
			e.printStackTrace();
		} catch (NullInformationException e) {
			e.printStackTrace();
		} catch (UserAlreadyExistsException e) {
			e.printStackTrace();
		} catch (FullGameException e) {
			e.printStackTrace();
		} catch (GameNotFoundException e) {
			e.printStackTrace();
		} catch (AlreadyInGameException e) {
			e.printStackTrace();
		} catch (InvalidTimeException e) {
			e.printStackTrace();
		}		
	}

	public void testCase1() 
	{
		try {
			// jj = jugadores en juego
			this.gamesInfo = Connection.getConnection().listGames(this.idValidSession);
			for (int i=0; i<this.gamesInfo.size(); i++)
			{
				GameInfo gi = this.gamesInfo.get(i);				
				if (gi.getId() == this.idValidGame) {
					ps = gi.getPlayers();					
				}
			}
			
			assertTrue(ps.contains("Luis"));
			Connection.getConnection().quitGame(idValidSession, idValidGame);
			
			// obtener del cliente el jugador "Luis" en p
			assertTrue(!p.isOnline());
		} catch (RemoteException e) {
			fail("No se esperaba la excepcin: ");
			e.printStackTrace();
		} catch (GameNotFoundException e) {
			fail("No se esperaba la excepcin: ");
			e.printStackTrace();
		} catch (InvalidSessionException e) {
			fail("No se esperaba la excepcin: ");
			e.printStackTrace();
		} catch (InvalidTimeException e) {
			fail("No se esperaba la excepcin: ");
			e.printStackTrace();
		}
	}

	public void testCase2() 
	{
		try {
			Connection.getConnection().quitGame(idValidSession, idInvalidGame);
			
			fail("Se esperaba la exception GameNotFoundException");
		} catch (RemoteException e) {
			fail("No se esperaba la excepcin: ");
			e.printStackTrace();
		} catch (GameNotFoundException e) {
			e.printStackTrace();
		} catch (InvalidSessionException e) {
			fail("No se esperaba la excepcin: ");
			e.printStackTrace();
		} catch (InvalidTimeException e) {
			fail("No se esperaba la excepcin: ");
			e.printStackTrace();
		}
	}

	public void testCase3() 
	{		
		try {
			Connection.getConnection().quitGame(idInvalidSession, idValidGame);
			
			fail("Se esperaba la exception InvalidSessionException");
		} catch (RemoteException e) {
			fail("No se esperaba la excepcin: ");
			e.printStackTrace();
		} catch (GameNotFoundException e) {
			fail("No se esperaba la excepcin: ");
			e.printStackTrace();
		} catch (InvalidSessionException e) {
			e.printStackTrace();
		} catch (InvalidTimeException e) {
			fail("No se esperaba la excepcin: ");
			e.printStackTrace();
		}
	}

	public void testCase4() 
	{		
		try {
			Connection.getConnection().quitGame(idInvalidSession, idInvalidGame);
			
			fail("Se esperaba GameNotFoundException o InvalidSessionException");
		} catch (RemoteException e) {
			fail("No se esperaba la excepcin: ");
			e.printStackTrace();
		} catch (GameNotFoundException e) {
			e.printStackTrace();
		} catch (InvalidSessionException e) {
			e.printStackTrace();
		} catch (InvalidTimeException e) {
			fail("No se esperaba la excepcin: ");
			e.printStackTrace();
		}
	}

	public void tearDown() 
	{
		try {
			Connection.getConnection().logoutUser(this.idValidSession);
		} catch (RemoteException e) {
			e.printStackTrace();
		} catch (InvalidSessionException e) {
			e.printStackTrace();
		}
	}

}
