package ea.test;

import java.rmi.RemoteException;
import java.util.ArrayList;
import java.util.UUID;

import junit.framework.TestCase;

import communications.EventType;

import domain.Game;
import domain.GameInfo;
import domain.Player;
import domain.Territory;
import domain.Territory.Continent;
import ea.libs.domain.Connection;
import ea.libs.exceptions.EmptyInformationException;
import ea.libs.exceptions.NullInformationException;
import ea.server.Server;
import exceptions.GameNotFoundException;
import exceptions.InvalidGameInfoException;
import exceptions.InvalidSessionException;
import exceptions.InvalidTimeException;
import exceptions.NotCurrentPlayerGameException;
import exceptions.UserAlreadyExistsException;
import exceptions.WrongLoginException;

public class TestCaseFinalishedTurn extends TestCase
{
	// fixtures

    Game game;
    Territory territory1;
    int[] canons = {1,0,0};
    ArrayList<Territory> map;
    GameInfo gameInfo;
    Player player1;
    ArrayList<String> playersName;
    ArrayList<Player> players;
    Long idGame, idPlayer;
    UUID session, gameUUID;
    UUID invalidSession, invalidGameUUID;
    EventType event;
  
    
    public void setUp()
    {
    	gameUUID = UUID.randomUUID();
    	invalidSession = UUID.randomUUID();
    	invalidGameUUID = UUID.randomUUID();
    	
    	player1 = new Player("testPlayer", 0, true, true, null);
    	playersName = new ArrayList<String>();
    	playersName.add(player1.getName());
    	
    	gameInfo = new GameInfo(gameUUID, "testGame", "testGame", playersName, null, 1, 1, 1, 1);
    	
    	map = new ArrayList<Territory>();
    	territory1 = new Territory(1, Continent.Africa, player1, 1, canons, 1, 1, 1);
    	map.add(territory1);
 
    	players = new ArrayList<Player>();
    	players.add(player1);
    	game = new Game(gameInfo, map, players);
    	
    	try {   		
    		
    		Connection.getConnection().createGame(gameInfo);  		
    		Connection.getConnection().registerUser("testPlayer", "0000", "testEmail@test.com");
    		session = Connection.getConnection().loginPlayer("testPlayer", "0000");
    		Connection.getConnection().joinGame(session, gameUUID); 
    		game = Connection.getConnection().playGame(session, gameUUID);
    		    		
		} catch (RemoteException e) {
			e.printStackTrace();
		} catch (NullInformationException e) {
			e.printStackTrace();
		} catch (InvalidGameInfoException e) {
			e.printStackTrace();
		} catch (UserAlreadyExistsException e) {
			e.printStackTrace();
		} catch (WrongLoginException e) {
			e.printStackTrace();
		} catch (Exception e) {
			e.printStackTrace();
		}   	
    }

    public void testFinalishedTurn()
    {
        try
        {              	
            //Comprobamos que se termina el turno y pasa a otro jugador
        	//TestCase 1
        	Connection.getConnection().endTurn(session, gameUUID);
        	assertTrue(!game.getPlayers().get(0).isHasTurn());
        }
        catch (RemoteException re) {
        	fail(re.toString());
        } catch (GameNotFoundException e) {
			fail();
		} catch (InvalidSessionException e) {
			fail();
		} catch (NotCurrentPlayerGameException e) {
			fail();
		} catch (Exception e) {
			e.printStackTrace();
		}        
    }
    
    public void testFinalishedTurnInvalidGame() {
    	
        try{   
        	
        	//TestCase 2
        	Connection.getConnection().endTurn(session, invalidGameUUID);
        	
        	fail("Se esperaba excepcin");        	
        } catch (GameNotFoundException e) {
			
		} catch (InvalidSessionException e) {
			fail();
		} catch (InvalidTimeException e) {
			fail();
		} catch (RemoteException e) {
			e.printStackTrace();
		} catch (Exception e) {
			e.printStackTrace();
		}
    }
    
    public void testFinalishedTurnInvalidSession() {
    	
        try{   
        	
        	//TestCase 2
        	Connection.getConnection().endTurn(invalidSession, gameUUID);
        	
        	fail("Se esperaba excepcin");        	
        } catch (GameNotFoundException e) {
			fail();
		} catch (InvalidSessionException e) {
			
		} catch (InvalidTimeException e) {
			
		} catch (RemoteException e) {
			e.printStackTrace();
		} catch (Exception e) {
			e.printStackTrace();
		}
    }
    
    public void testFinalishedTurnInvalidAll() {
    	
        try{   
        
        	//TestCase 2
        	Connection.getConnection().endTurn(invalidSession, invalidGameUUID);
        	
        	fail("Se esperaba excepcin");        	
        } catch (GameNotFoundException e) {
			
		} catch (InvalidSessionException e) {
		
		} catch (InvalidTimeException e) {
			
		} catch (RemoteException e) {
			e.printStackTrace();
		} catch (Exception e) {
			e.printStackTrace();
		}
    }

    public void tearDown()
    {
    	try {
    		Connection.getConnection().logoutUser(session);
    		//Connection.getConnection().deleteUser();
			//Connection.getConnection().deleteGame(game);
		} catch (RemoteException e) {
			e.printStackTrace();
		} catch (InvalidSessionException e) {
			e.printStackTrace();
		}
    }
}