package ea.test;


import java.net.ConnectException;
import java.rmi.RemoteException;
import java.util.ArrayList;
import java.util.UUID;

import junit.framework.TestCase;
import domain.GameInfo;
import ea.client.Client;
import exceptions.AlreadyInGameException;
import exceptions.FullGameException;
import exceptions.GameNotFoundException;
import exceptions.InvalidSessionException;
import exceptions.UserAlreadyExistsException;
import exceptions.WrongLoginException;
import ea.libs.domain.Connection;
import ea.libs.exceptions.NullInformationException;
import ea.server.Server;

public class TestCaseJoinGame extends TestCase{

	Server server;
    Connection conn;
    UUID session, sessionnovalida = null;
    UUID game, gamenoexiste;
    Client cliente;
    GameInfo gi;

    public void setUp()
    {
    	try {
    		cliente = Connection.getConnection();
    		cliente.registerPlayer("edu", "12345", "edseal20@hotmail.com");
    		session = cliente.loginPlayer("edu", "12345");
    		ArrayList<GameInfo> juegos = cliente.listGames(session);
    		gi = juegos.get(1); 	//Se saca la informacin del primer juego
    		game = gi.getId();		//Se saca el id del juego
    		gamenoexiste = uuidInexistente(juegos);	//Se saca un uuid de un juego que no existe
    		
    		
    	} catch (UserAlreadyExistsException e) {
			e.printStackTrace();
		} catch (RemoteException e) {
			e.printStackTrace();
		} catch (ConnectException e) {
			e.printStackTrace();
		} catch (WrongLoginException e) {
			e.printStackTrace();
		} catch (NullInformationException e) {
			e.printStackTrace();
		} catch (Exception e){
			e.printStackTrace();
		}
    }
    
    //Se busca un uuid en la lista de juegos
    private boolean buscarUUID(UUID valor, GameInfo[] ajuegos){
    	for(GameInfo g:ajuegos){
    		if(g.getId().equals(valor)) return true;
    	}
    	return false;
    }
    
    //Se genera un uuid no existente en la lista de juegos
    private UUID uuidInexistente(ArrayList<GameInfo> juegos){
    	GameInfo[] ajuegos = (GameInfo[])juegos.toArray();
    	UUID valor = UUID.randomUUID();
    	while(buscarUUID(valor,ajuegos)){
    		valor = UUID.randomUUID();
    	}
    	return valor;
    }
    
    //Para saber si un juego existe
    private boolean existeJuego(UUID id) throws RemoteException{
    	ArrayList<GameInfo> juegos = cliente.listGames(session);
    	GameInfo[] ajuegos = (GameInfo[])juegos.toArray();
    	return buscarUUID(id,ajuegos);
    }
    
    //Para saber si un juego determinado tena un jugador
    private boolean existeJugador(ArrayList<String> jugadores, String jugador){
    	String[] ajugadores = (String[])jugadores.toArray();
    	for(String j:ajugadores){
    		if(j.equalsIgnoreCase(jugador)) return true;
    	}
    	return false;
    }
    
    public void testTc1(){ //Sesin y juegos vlidos
    	try{
    		cliente.joinGame(session, game);
    		ArrayList<GameInfo> juegos = cliente.listGames(session);
    		gi = juegos.get(1);
    		ArrayList<String> jugadores = gi.getPlayers();
    		assertTrue(existeJugador(jugadores,"edu"));	//El jugador edu unido al juego
    		
    	} catch (RemoteException re) {
			fail(re.toString());
		} catch (FullGameException fge) {
			fail(fge.toString());
		} catch (GameNotFoundException gnfe) {
			fail(gnfe.toString());
		} catch (InvalidSessionException ise){
			fail(ise.toString());
		} catch (AlreadyInGameException e) {
			fail(e.toString());
		} catch (Exception e){
			fail(e.toString());
		}
    }
    
    public void testTc2() throws RemoteException{ //Sesion valida y juego no existente
    	try{
    		cliente.joinGame(session, gamenoexiste);
    		fail("Se espera GameNotFoundException");
    				
    	} catch (RemoteException re) {
			fail(re.toString());
		} catch (FullGameException fge) {
			fail(fge.toString());
		} catch (GameNotFoundException gnfe) {
			assertFalse(existeJuego(gamenoexiste)); //Se espera que el juego no exista
		} catch (InvalidSessionException ise){
			fail(ise.toString());
		} catch (AlreadyInGameException e) {
			fail(e.toString());
		} catch (Exception e){
			fail(e.toString());
		}
    }
    
    public void testTc3(){ //Sesin no vlida y juego existente
    	try{
    		cliente.joinGame(sessionnovalida, game);
    		fail("Se espera InvalidSessionException.");
    		
    	} catch (RemoteException re) {
			fail(re.toString());
		} catch (FullGameException fge) {
			fail(fge.toString());
		} catch (GameNotFoundException gnfe) {
			fail(gnfe.toString());
		} catch (InvalidSessionException ise){
			assertNull(sessionnovalida);
		} catch (AlreadyInGameException e) {
			fail(e.toString());
		} catch (Exception e){
			fail(e.toString());
		}
    }
    
    public void testTc4() throws RemoteException{ //Sesin y juego no vlidos
    	try{
    		cliente.joinGame(sessionnovalida, gamenoexiste);
    		fail("Se espera InvalidSessionException y GameNotFoundException");
    		
    	} catch (RemoteException re) {
			fail(re.toString());
		} catch (FullGameException fge) {
			fail(fge.toString());
		} catch (GameNotFoundException gnfe) {
			assertFalse(existeJuego(gamenoexiste));
		} catch (InvalidSessionException ise){
			assertNull(sessionnovalida);
		} catch (AlreadyInGameException e) {
			fail(e.toString());
		} catch (Exception e){
			fail(e.toString());
		}
    }

	public void tearDown() throws Exception {
		cliente = null;
        System.gc();
	}

}
