package ea.test;

import java.rmi.RemoteException;
import java.util.ArrayList;
import java.util.UUID;

import junit.framework.TestCase;

import communications.EventType;

import domain.Arsenal;
import domain.Game;
import domain.GameInfo;
import domain.Player;
import domain.Territory;
import domain.Territory.Continent;
import ea.libs.domain.Connection;
import ea.libs.exceptions.EmptyInformationException;
import ea.libs.exceptions.NullInformationException;
import ea.server.Server;
import exceptions.GameNotFoundException;
import exceptions.InvalidGameInfoException;
import exceptions.InvalidSessionException;
import exceptions.InvalidTerritoryException;
import exceptions.InvalidTimeException;
import exceptions.NotCurrentPlayerGameException;
import exceptions.UserAlreadyExistsException;
import exceptions.WrongLoginException;

public class TestCaseMakeAttack extends TestCase
{
	// fixtures

    Game game;
    Territory territory1, territory2, territoryInvalid;
    int[] canons = {1,0,0};
    ArrayList<Territory> map;
    GameInfo gameInfo;
    Player player1, player2;
    ArrayList<String> playersName;
    ArrayList<Player> players;
    Long idGame, idPlayer;
    UUID session, session2, gameUUID;
    UUID invalidSession, invalidGameUUID;
    EventType event;
  
    
    public void setUp()
    {
    	gameUUID = UUID.randomUUID();
    	invalidSession = UUID.randomUUID();
    	invalidGameUUID = UUID.randomUUID();
    	
    	player1 = new Player("testPlayer", 0, true, true, null);
    	player2 = new Player("testPlayer2", 0, true, true, null);
    	playersName = new ArrayList<String>();
    	playersName.add(player1.getName());
    	playersName.add(player2.getName());
    	
    	gameInfo = new GameInfo(gameUUID, "testGame", "testGame", playersName, null, 1, 1, 1, 1);
    	
    	map = new ArrayList<Territory>();
    	territory1 = new Territory(1, Continent.Africa, player1, 1, canons, 1, 1, 1);
    	territory2 = new Territory(2, Continent.Africa, player2, 1, canons, 1, 1, 1);
    	territoryInvalid = new Territory(1, Continent.NorthAmerica, player1, 1, canons, 1, 1, 1);
    	map.add(territory1);
    	map.add(territory2);
    	map.add(territoryInvalid);
 
    	players = new ArrayList<Player>();
    	players.add(player1);
    	players.add(player2);
    	game = new Game(gameInfo, map, players);
    	
    	try {   		
    		Connection.getConnection().createGame(gameInfo);  		
    		Connection.getConnection().registerUser("testPlayer", "0000", "testEmail@test.com");
    		Connection.getConnection().registerUser("testPlayer2", "1111", "test2Email@test.com");
    		session = Connection.getConnection().loginPlayer("testPlayer", "0000");
    		session2 = Connection.getConnection().loginPlayer("testPlayer2", "1111");
    		Connection.getConnection().joinGame(session, gameUUID); 
    		Connection.getConnection().joinGame(session2, gameUUID); 
    		game = Connection.getConnection().playGame(session, gameUUID);
    		game = Connection.getConnection().playGame(session2, gameUUID);
    		    		
		} catch (RemoteException e) {
			e.printStackTrace();
		} catch (NullInformationException e) {
			e.printStackTrace();
		} catch (InvalidGameInfoException e) {
			e.printStackTrace();
		} catch (UserAlreadyExistsException e) {
			e.printStackTrace();
		} catch (WrongLoginException e) {
			e.printStackTrace();
		} catch (Exception e) {
			e.printStackTrace();
		}   	
    }

    public void testMakeAttack()
    {
        try
        {              	
            //Comprobamos que se ejecuta Atacar en el servidor
        	//TestCase 1
        	Connection.getConnection().attackTerritory(session, gameUUID, territory1, territory2, new Arsenal(1,0,0,0));
        	
        } catch (RemoteException re) {
        	fail(re.toString());
        } catch (GameNotFoundException e) {
			fail(e.toString());
		} catch (InvalidSessionException e) {
			fail();
		} catch (InvalidTerritoryException e) {
			fail();
		}        
    }
    
    public void testMakeAttackInvalid() {
    	
        try{   
        	
        	//TestCase 2
        	Connection.getConnection().attackTerritory(session, gameUUID, territory1, territory2, new Arsenal(100,0,0,0));
        	fail("Se esperaba excepcin Arsenal errneo");        	
        } catch (InvalidTerritoryException e) {
			
        } catch (RemoteException e) {
			
		} catch (GameNotFoundException e) {
			
		} catch (InvalidSessionException e) {
			
		}
    }
    
    public void testMakeAttackInvalid2() {
    	
        try{   
        	
        	//TestCase 3
        	Connection.getConnection().attackTerritory(session, gameUUID, territoryInvalid, territoryInvalid, new Arsenal(1,0,0,0));
        	fail("Se esperaba excepcin");        	
        } catch (InvalidTerritoryException e) {
			
        } catch (RemoteException e) {
			
		} catch (GameNotFoundException e) {
			fail();
		} catch (InvalidSessionException e) {
			fail();
		}
    }
    
    public void testMakeAttackInvalid3() {
    	
        try{   
        	
        	//TestCase 4
        	Connection.getConnection().attackTerritory(session, invalidGameUUID, territory1, territory2, new Arsenal(1,0,0,0));
        	fail("Se esperaba excepcin GameNotFound");        	
        } catch (InvalidTerritoryException e) {
			fail();
        } catch (RemoteException e) {
			
		} catch (GameNotFoundException e) {
			
		} catch (InvalidSessionException e) {
			fail();
		}
    }
    
    public void testMakeAttackInvalid4() {
    	
        try{   
        	
        	//TestCase 5
        	Connection.getConnection().attackTerritory(session, invalidGameUUID, territoryInvalid, territory2, null);
        	fail("Se esperaba excepcin ");        	
        } catch (InvalidTerritoryException e) {
			
        } catch (RemoteException e) {
			
		} catch (GameNotFoundException e) {
			
		} catch (InvalidSessionException e) {
			fail();
		}
    }
    
    public void testMakeAttackInvalid5() {
    	
        try{   
        	
        	//TestCase 6
        	Connection.getConnection().attackTerritory(invalidSession, gameUUID, territory1, territory2, new Arsenal(1,0,0,0));
        	fail("Se esperaba excepcin InvalidSession");        	
        } catch (InvalidTerritoryException e) {
			fail();
        } catch (RemoteException e) {
			
		} catch (GameNotFoundException e) {
			fail();
		} catch (InvalidSessionException e) {
			
		}
    }
    
    public void testMakeAttackInvalid6() {
    	
        try{   
        	
        	//TestCase 7
        	Connection.getConnection().attackTerritory(invalidSession, gameUUID, territoryInvalid, territory2, null);
        	fail("Se esperaba excepcin");        	
        } catch (InvalidTerritoryException e) {
			
        } catch (RemoteException e) {
			
		} catch (GameNotFoundException e) {
			fail();
		} catch (InvalidSessionException e) {
			
		}
    }
    
    public void testMakeAttackInvalid7() {
    	
        try{   
        	
        	//TestCase 8
        	Connection.getConnection().attackTerritory(invalidSession, invalidGameUUID, territory1, territoryInvalid, null);
        	fail("Se esperaba excepcin");        	
        } catch (InvalidTerritoryException e) {
			
        } catch (RemoteException e) {
			
		} catch (GameNotFoundException e) {
			
		} catch (InvalidSessionException e) {
			
		}
    }
    
    public void testMakeAttackInvalid8() {
    	
        try{   
        	
        	//TestCase 9
        	Connection.getConnection().attackTerritory(invalidSession, invalidGameUUID, territoryInvalid, territoryInvalid, new Arsenal(100,0,0,0));
        	fail("Se esperaba excepcin Arsenal errneo");        	
        } catch (InvalidTerritoryException e) {
			
        } catch (RemoteException e) {
			
		} catch (GameNotFoundException e) {
			
		} catch (InvalidSessionException e) {
			
		}
    }

    public void tearDown()
    {
    	try {
    		Connection.getConnection().logoutUser(session);
    		//conn.getConnection().deleteUser();
			//conn.getConnection().deleteGame(game);
		} catch (RemoteException e) {
			e.printStackTrace();
		} catch (InvalidSessionException e) {
			e.printStackTrace();
		}
    }
}