package ea.test;

import java.rmi.RemoteException;
import java.util.ArrayList;
import java.util.UUID;

import junit.framework.TestCase;

import communications.EventType;

import domain.Arsenal;
import domain.Game;
import domain.GameInfo;
import domain.Player;
import domain.Territory;
import domain.Territory.Continent;
import ea.libs.domain.Connection;
import ea.libs.exceptions.EmptyInformationException;
import ea.libs.exceptions.NullInformationException;
import ea.server.Server;
import exceptions.GameNotFoundException;
import exceptions.InvalidGameInfoException;
import exceptions.InvalidSessionException;
import exceptions.InvalidTimeException;
import exceptions.NotCurrentPlayerGameException;
import exceptions.UserAlreadyExistsException;
import exceptions.WrongLoginException;

public class TestCaseNegotiate extends TestCase
{
	// fixtures
	
    Game game;
    Territory territory1, territory2;
    int[] canons = {1,0,0};
    ArrayList<Territory> map;
    GameInfo gameInfo;
    Player player1, player2;
    ArrayList<String> playersName;
    ArrayList<Player> players;
    Long idGame, idPlayer;
    UUID session, session2, gameUUID;
    UUID invalidSession, invalidGameUUID;
    EventType event;
  
    
    public void setUp()
    {
    	gameUUID = UUID.randomUUID();
    	invalidSession = UUID.randomUUID();
    	invalidGameUUID = UUID.randomUUID();
    	
    	player1 = new Player("testPlayer", 0, true, true, null);
    	player2 = new Player("testPlayer2", 0, true, true, null);
    	playersName = new ArrayList<String>();
    	playersName.add(player1.getName());
    	playersName.add(player2.getName());
    	
    	gameInfo = new GameInfo(gameUUID, "testGame", "testGame", playersName, null, 1, 1, 1, 1);
    	
    	map = new ArrayList<Territory>();
    	territory1 = new Territory(1, Continent.Africa, player1, 1, canons, 1, 1, 1);
    	territory2 = new Territory(2, Continent.Africa, player2, 1, canons, 1, 1, 1);
    	map.add(territory1);
    	map.add(territory2);
 
    	players = new ArrayList<Player>();
    	players.add(player1);
    	players.add(player2);
    	game = new Game(gameInfo, map, players);
    	
    	try {   		
    		
    		Connection.getConnection().createGame(gameInfo);  		
    		Connection.getConnection().registerUser("testPlayer", "0000", "testEmail@test.com");
    		Connection.getConnection().registerUser("testPlayer2", "1111", "test2Email@test.com");
    		session = Connection.getConnection().loginPlayer("testPlayer", "0000");
    		session2 = Connection.getConnection().loginPlayer("testPlayer2", "1111");
    		Connection.getConnection().joinGame(session, gameUUID); 
    		Connection.getConnection().joinGame(session2, gameUUID); 
    		game = Connection.getConnection().playGame(session, gameUUID);
    		game = Connection.getConnection().playGame(session2, gameUUID);
    		if(game.getPlayers().get(0).isHasTurn()){
    			Connection.getConnection().attackTerritory(session, gameUUID, territory1, territory2, new Arsenal(1,0,0,0));
    		} else {
    			Connection.getConnection().attackTerritory(session, gameUUID, territory2, territory1, new Arsenal(1,0,0,0));
    		}
    		    		
		} catch (RemoteException e) {
			e.printStackTrace();
		} catch (NullInformationException e) {
			e.printStackTrace();
		} catch (InvalidGameInfoException e) {
			e.printStackTrace();
		} catch (UserAlreadyExistsException e) {
			e.printStackTrace();
		} catch (WrongLoginException e) {
			e.printStackTrace();
		} catch (Exception e) {
			e.printStackTrace();
		}   	
    }

    public void testDefense()
    {
        try
        {              	
            //Comprobamos que se ejecuta Defender en el servidor
        	//TestCase 1
        	if(game.getPlayers().get(0).isHasTurn()){
        		Connection.getConnection().
        	}else{
        		Connection.getConnection()
        	}
        	
        } catch (RemoteException e) {
			fail();
		} catch (GameNotFoundException e) {
			fail();
		} catch (InvalidSessionException e) {
			fail();
		} catch (InvalidTimeException e) {
			fail();
		}	
    }
    
    public void testDefenseInvalidGame() {
    	
    	try
        {    
    		setUp();
        	//TestCase 2
        	if(game.getPlayers().get(0).isHasTurn()){
        		Connection.getConnection().acceptAttack(session, invalidGameUUID);
        	}else{
        		Connection.getConnection().acceptAttack(session2, invalidGameUUID);
        	}
        	fail("Se esperaba GameNotFoundException");
        	
        } catch (RemoteException e) {
			fail();
		} catch (GameNotFoundException e) {
			fail();
		} catch (InvalidSessionException e) {
			
		} catch (InvalidTimeException e) {
			
		}	
    }
    
    public void testDefenseInvalidSession() {
    	
    	try
        {    
    		setUp();
        	//TestCase 2
        	if(game.getPlayers().get(0).isHasTurn()){
        		Connection.getConnection().acceptAttack(invalidSession, gameUUID);
        	}else{
        		Connection.getConnection().acceptAttack(invalidSession, gameUUID);
        	}
        	fail("Se esperaba SessionInvalidException");
        	
        } catch (RemoteException e) {
			
		} catch (GameNotFoundException e) {
			
		} catch (InvalidSessionException e) {
			fail();
		} catch (InvalidTimeException e) {
			
		}	
    }
    
    public void testDefenseInvalidAll() {
    	
    	try
        {    
    		setUp();
        	//TestCase 4
        	if(game.getPlayers().get(0).isHasTurn()){
        		Connection.getConnection().acceptAttack(invalidSession, invalidGameUUID);
        	}else{
        		Connection.getConnection().acceptAttack(invalidSession, invalidGameUUID);
        	}
        	fail("Se espearaba excepcin.");
        	
        } catch (RemoteException e) {
			fail();
		} catch (GameNotFoundException e) {
			
		} catch (InvalidSessionException e) {
			
		} catch (InvalidTimeException e) {
			fail();
		}	
    }

    public void tearDown()
    {
    	try {
    		Connection.getConnection().logoutUser(session);
    		//conn.getConnection().deleteUser();
			//conn.getConnection().deleteGame(game);
		} catch (RemoteException e) {
			e.printStackTrace();
		} catch (InvalidSessionException e) {
			e.printStackTrace();
		}
    }
}