package ea.test;


import java.net.ConnectException;
import java.rmi.RemoteException;
import java.util.ArrayList;
import java.util.UUID;

import junit.framework.TestCase;
import domain.Game;
import domain.GameInfo;
import exceptions.AlreadyInGameException;
import exceptions.FullGameException;
import exceptions.GameNotFoundException;
import exceptions.InvalidSessionException;
import exceptions.InvalidTimeException;
import exceptions.UserAlreadyExistsException;
import exceptions.WrongLoginException;
import ea.client.Client;
import ea.libs.domain.Connection;
import ea.libs.exceptions.NullInformationException;
import ea.server.Server;

public class TestCasePlayGame extends TestCase{

	Server server;
    Connection conn;
    UUID session, sessionnovalida = null;
    UUID game, gamenoexiste;
    Client cliente;
    GameInfo gi;

    public void setUp()
    {
    	try {
    		cliente = Connection.getConnection();
    		cliente.registerPlayer("edu", "12345", "edseal20@hotmail.com");
    		session = cliente.loginPlayer("edu", "12345");
    		ArrayList<GameInfo> juegos = cliente.listGames(session);
    		gi = juegos.get(1);
    		game = gi.getId();
    		cliente.joinGame(session, game);
    		gamenoexiste = uuidInexistente(juegos);
    		
		} catch (RemoteException e) {
			e.printStackTrace();
		} catch (UserAlreadyExistsException e) {
			e.printStackTrace();
		} catch (ConnectException e) {
			e.printStackTrace();
		} catch (WrongLoginException e) {
			e.printStackTrace();
		} catch (NullInformationException e) {
			e.printStackTrace();
		} catch (FullGameException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (GameNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (InvalidSessionException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (AlreadyInGameException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}  
    }
    
    private boolean buscarUUID(UUID valor, GameInfo[] ajuegos){
    	for(GameInfo g:ajuegos){
    		if(g.getId().equals(valor)) return true;
    	}
    	return false;
    }
    
    private UUID uuidInexistente(ArrayList<GameInfo> juegos){
    	GameInfo[] ajuegos = (GameInfo[])juegos.toArray();
    	UUID valor = UUID.randomUUID();
    	while(buscarUUID(valor,ajuegos)){
    		valor = UUID.randomUUID();
    	}
    	return valor;
    }
    
    private boolean existeJuego(UUID id) throws RemoteException{
    	ArrayList<GameInfo> juegos = cliente.listGames(session);
    	GameInfo[] ajuegos = (GameInfo[])juegos.toArray();
    	return buscarUUID(id,ajuegos);
    }
    
    public void testTc1(){ //Sesin y juego vlidos
    	try{
    		Game g = cliente.playGame(session, game);
    		assertNotNull(g); //Se espera que el juego no sea nulo
    		
    	} catch (RemoteException re) {
			fail(re.toString());
		} catch (InvalidTimeException ite) {
			fail(ite.toString());
		} catch (GameNotFoundException e) {
			fail(e.toString());
		} catch (InvalidSessionException e) {
			fail(e.toString());
		}catch (Exception e){
			fail(e.toString());
		}
    }
    
    public void testTc2() throws RemoteException{ //Sesin vlida y juego no existe
    	try{
    		cliente.playGame(session, gamenoexiste);
    		fail("Se espera GameNotFoundException");
    		
    	} catch (RemoteException re) {
			fail(re.toString());
		} catch (InvalidTimeException ite) {
			fail(ite.toString());
		} catch (GameNotFoundException e) {
			assertFalse(existeJuego(gamenoexiste));
		} catch (InvalidSessionException e) {
			fail(e.toString());
		}catch (Exception e){
			fail(e.toString());
		}
    }
    
    public void testTc3(){ //Sesin invlida y juego existe
    	try{
    		cliente.playGame(sessionnovalida, game);
    		fail("Se espera InvalidSessionException");
    		
    	} catch (RemoteException re) {
			fail(re.toString());
		} catch (InvalidTimeException ite) {
			fail(ite.toString());
		} catch (GameNotFoundException e) {
			fail(e.toString());
		} catch (InvalidSessionException e) {
			assertNull(sessionnovalida);
		}catch (Exception e){
			fail(e.toString());
		}
    }
    
    public void testTc4() throws RemoteException{ //Sesin y juego invlidos
    	try{
    		cliente.playGame(sessionnovalida, gamenoexiste);
    		fail("Se espera InvalidSessionException y GameNotFoundException");
    		
    	} catch (RemoteException re) {
			fail(re.toString());
		} catch (InvalidTimeException ite) {
			fail(ite.toString());
		} catch (GameNotFoundException e) {
			assertFalse(existeJuego(gamenoexiste));
		} catch (InvalidSessionException e) {
			assertNull(sessionnovalida);
		}catch (Exception e){
			fail(e.toString());
		}
    }

	public void tearDown() throws Exception {
		cliente = null;
        System.gc();
	}
}
