package ea.test;

import java.rmi.RemoteException;
import java.util.ArrayList;
import java.util.UUID;

import junit.framework.TestCase;
import domain.Game;
import domain.GameInfo;
import domain.Player;
import domain.Territory;
import ea.libs.domain.Connection;
import ea.libs.exceptions.EmptyInformationException;
import ea.libs.exceptions.NullInformationException;
import ea.server.Server;
import exceptions.GameNotFoundException;
import exceptions.InvalidGameInfoException;
import exceptions.InvalidSessionException;
import exceptions.InvalidTimeException;
import exceptions.UserAlreadyExistsException;
import exceptions.WrongLoginException;

public class TestCaseResignGame extends TestCase
{
	// fixtures

    Game game;
    ArrayList<Territory> map;
    GameInfo gameInfo;
    Player player;
    ArrayList<String> players;
    ArrayList<Player> playersGame;
    Long idGame, idPlayer;
    UUID session, gameUUID;
    UUID invalidSession, invalidGameUUID;
  
    
    public void setUp()
    {
    	gameUUID = UUID.randomUUID();
    	invalidGameUUID = UUID.randomUUID();
    	invalidSession = UUID.randomUUID();
    	
    	player = new Player("testPlayer", 0, true, true, null);
    	players = new ArrayList<String>();
    	players.add(player.getName());
    	
    	gameInfo = new GameInfo(gameUUID, "testGame", "testGame", players, null, 1, 1, 1, 1);
    	
    	map = new ArrayList<Territory>();
 
    	playersGame = new ArrayList<Player>();
    	playersGame.add(player);
    	game = new Game(gameInfo, map, playersGame);
    	
    	try {
    		
    		Connection.getConnection().createGame(gameInfo);  		
    		Connection.getConnection().registerUser("testPlayer", "0000", "testEmail@test.com");
    		session = Connection.getConnection().loginPlayer("testPlayer", "0000");
    		Connection.getConnection().joinGame(session, gameUUID);    
    		game = Connection.getConnection().playGame(session, gameUUID);
    		    		
		} catch (RemoteException e) {
			e.printStackTrace();
		} catch (NullInformationException e) {
			e.printStackTrace();
		} catch (InvalidGameInfoException e) {
			e.printStackTrace();
		} catch (UserAlreadyExistsException e) {
			e.printStackTrace();
		} catch (WrongLoginException e) {
			e.printStackTrace();
		} catch (Exception e) {
			e.printStackTrace();
		}   	
    }

    public void testResignGame()
    {
        try
        {              	
            //Comprobamos que el jugador se desvincula de la partida
        	//TestCase 1
        	Connection.getConnection().resignGame(session, gameUUID);        	
        	assertTrue(!game.getPlayers().contains(player));
        }
        catch (RemoteException re) {
        	fail();
        } catch (GameNotFoundException e) {
			fail();
		} catch (InvalidSessionException e) {
			fail();
		}        
    }
    
    public void testResignGameInvalidGame() {
    	
        try{   
        
        	//TestCase 2
        	Connection.getConnection().quitGame(session, invalidGameUUID);
        	fail("Se esperaba excepcin GameNotFound");        	
        } catch (GameNotFoundException e) {
			
		} catch (InvalidSessionException e) {
			e.printStackTrace();
		} catch (InvalidTimeException e) {
			e.printStackTrace();
		} catch (RemoteException e) {
			e.printStackTrace();
		}
    }
    
    public void testResignGameInvalidSession() {
        
        try{
        	
        	//TestCase 3
        	Connection.getConnection().quitGame(invalidSession, gameUUID);
        	fail("Se esperaba excepcin InvalidSession");
        }catch (InvalidSessionException e) {
    		
    	} catch (RemoteException e) {
			e.printStackTrace();
		} catch (GameNotFoundException e) {
			e.printStackTrace();
		} catch (InvalidTimeException e) {
			e.printStackTrace();
		}
        
    }
    
    public void testResignGameAllInvalid() {
        
        try{
        	
        	//TestCase 4
        	Connection.getConnection().quitGame(invalidSession, invalidGameUUID);
        	fail("Se esperaba excepcin InvalidSession o GameNotFound");
        } catch (GameNotFoundException e) {
			
		} catch (InvalidSessionException e) {
			 
		} catch (InvalidTimeException e) {
			e.printStackTrace();
		} catch (RemoteException e) {
			e.printStackTrace();
		}
    }

    public void tearDown()
    {
    	try {
    		Connection.getConnection().logoutUser(session);
    		//Connection.getConnection().deleteUser();
    		//Connection.getConnection().deleteGame(game);
		} catch (RemoteException e) {
			e.printStackTrace();
		} catch (InvalidSessionException e) {
			e.printStackTrace();
		}
    }
}