package ea.test;

import java.rmi.RemoteException;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.UUID;

import junit.framework.TestCase;
import domain.Game;
import domain.GameInfo;
import domain.Player;
import domain.Territory;
import domain.Territory.Continent;
import ea.libs.domain.Connection;
import ea.libs.exceptions.EmptyInformationException;
import ea.libs.exceptions.NullInformationException;
import ea.server.Server;
import exceptions.InvalidGameInfoException;
import exceptions.InvalidSessionException;
import exceptions.UserAlreadyExistsException;
import exceptions.WrongLoginException;

public class TestCaseUpdateResources extends TestCase
{
    // fixtures
	
    Connection conn;
    Game game;
    ArrayList<Territory> map;
    Territory territory1, territory2, territory3;
    GameInfo gameInfo;
    Player player;
    ArrayList<String> players;
    ArrayList<Player> playersGame;
    Long idGame, idPlayer;
    UUID session, gameUUID;
    int idOriginTerritory, idDestinyTerritory;
    int[] units;
    int[] canons = {1,0,0};
    
    //Para el test
    Iterator<GameInfo> gamesInfo;
    ArrayList<GameInfo> gameList;
    int soldiersOriginBefore;
    int soldiersDestinyBefore;
    int missilesOriginBefore;
    int missilesDestinyBefore;
    int[] canonsOriginBefore;
    int[] canonsDestinyBefore;
    
    public void setUp()
    {
    	gameUUID = UUID.randomUUID();
    	units = new int[3];
    	
    	player = new Player("testPlayer", 0, true, true, null);
    	players = new ArrayList<String>();
    	players.add(player.getName());
    	
    	gameInfo = new GameInfo(gameUUID, "testGame", "testGame", players, null, 1, 1, 1, 1);
    	
    	map = new ArrayList<Territory>();
    	territory1 = new Territory(1, Continent.Africa, player, 1, canons, 1, 1, 1);
    	territory2 = new Territory(2, Continent.Africa, player, 1, canons, 1, 1, 1);
    	territory3 = new Territory(3, Continent.Africa, player, 1, canons, 1, 1, 1);
    	map.add(territory1);
    	map.add(territory2);
    	map.add(territory3);
    	
    	playersGame = new ArrayList<Player>();
    	playersGame.add(player);
    	game = new Game(gameInfo, map, playersGame);
    	
    	try {	
    		conn = new Connection("localhost");
    		conn.getConnection().createGame(gameInfo);  		
    		conn.getConnection().registerUser("testPlayer", "0000", "testEmail@test.com");
    		session = conn.getConnection().loginPlayer("testPlayer", "0000");
    		conn.getConnection().joinGame(session, gameUUID);    
    		game = Connection.getConnection().playGame(session, gameUUID);
    		    		
		} catch (RemoteException e) {
			e.printStackTrace();
		} catch (NullInformationException e) {
			e.printStackTrace();
		} catch (EmptyInformationException e) {
			e.printStackTrace();
		} catch (InvalidGameInfoException e) {
			e.printStackTrace();
		} catch (UserAlreadyExistsException e) {
			e.printStackTrace();
		} catch (WrongLoginException e) {
			e.printStackTrace();
		} catch (Exception e) {
			e.printStackTrace();
		}   
		
    }

    public void testConfigurationTerritories()
    {
        try
        {      
        	int galliAnt = player.getMoney();
        	Connection.getConnection().endTurn(session, gameUUID);
        	//Ahora habra que comprobar que se asignan soldados por tener tres territorios
        	
            assertTrue(player.getMoney() == galliAnt + 1);
        }
        catch(Exception e)
        {
            fail(e.toString());
        }
    }

    public void tearDown()
    {
    	try {
    		conn.getConnection().logoutUser(session);
		} catch (RemoteException e) {
			e.printStackTrace();
		} catch (InvalidSessionException e) {
			e.printStackTrace();
		}
    }
}