package ea.test;

import java.net.ConnectException;
import java.rmi.RemoteException;

import java.util.ArrayList;
import java.util.UUID;

import communications.EventType;

import domain.Game;
import domain.GameInfo;
import domain.Player;
import domain.Spy;
import domain.Territory;
import domain.Territory.Continent;
import ea.domain.Connection;
import ea.exceptions.NullInformationException;
import exceptions.AlreadyInGameException;
import exceptions.FullGameException;
import exceptions.GameNotFoundException;
import exceptions.InvalidGameInfoException;
import exceptions.InvalidSessionException;
import exceptions.InvalidTimeException;
import exceptions.NotCurrentPlayerGameException;
import exceptions.UserAlreadyExistsException;
import exceptions.WrongLoginException;

import junit.framework.TestCase;

public class TestCaseUpdateStateGame extends TestCase
{
	// fixtures
	UUID idValidSession, idInvalidSession;
	UUID idValidGame, idInvalidGame;
	ArrayList<Player> validPlayers, invalidPlayers;
	ArrayList<Territory> validTerritories, invalidTerritories;
	EventType validEvent, invalidEvent;
	ArrayList<String> playersStr;
	GameInfo gameInfo;
	Game game;
	Player p;
	ArrayList<Spy> spies;
	int[] numCannons;

    public void setUp()
    {
    	this.idValidGame = UUID.randomUUID();
		this.idInvalidGame = UUID.fromString("no valid");
		this.idInvalidSession = UUID.fromString("no valid");
		this.playersStr.add("Luis");	
		this.gameInfo = new GameInfo(this.idValidGame, "testGame", "testGame", 
				this.playersStr, null, 1, 1, 1, 1);
		
		try {
			Connection.getConnection().registerPlayer("Luis", "1234", "luis@alu");
			this.idValidSession = Connection.getConnection().loginPlayer("Luis", "1234");
			Connection.getConnection().createGame(this.gameInfo);
			this.spies = new ArrayList<Spy>(); 
			p = new Player("Luis", 30000, false, false, spies);
			this.numCannons[0] = 0;
			Territory t = new Territory(1, Continent.Europe, p, 10, this.numCannons, 0, 0, 0);				
			this.validTerritories.add(t);
			this.validPlayers.add(p);
			this.invalidTerritories = null;
			this.invalidPlayers = null;
			this.invalidEvent = EventType.UnknownEvent;
			Connection.getConnection().joinGame(this.idValidSession, this.idValidGame);
			game = Connection.getConnection().playGame(this.idValidSession, this.idValidGame);
		} catch (RemoteException e) {
			e.printStackTrace();
		} catch (InvalidGameInfoException e) {
			e.printStackTrace();
		} catch (InvalidSessionException e) {
			e.printStackTrace();
		} catch (ConnectException e) {
			e.printStackTrace();
		} catch (WrongLoginException e) {
			e.printStackTrace();
		} catch (NullInformationException e) {
			e.printStackTrace();
		} catch (UserAlreadyExistsException e) {
			e.printStackTrace();
		} catch (FullGameException e) {
			e.printStackTrace();
		} catch (GameNotFoundException e) {
			e.printStackTrace();
		} catch (AlreadyInGameException e) {
			e.printStackTrace();
		} catch (InvalidTimeException e) {
			e.printStackTrace();
		}			
    }

    public void testCase0()
    // Escenario normal
    {		
    	try {
    		Connection.getConnection().updateGame(this.idValidSession, this.idValidGame,
    				this.validPlayers, this.validTerritories,     				
    				EventType.NegotiationEvent);
    				
			
		} catch (RemoteException e) {
			fail("No se esperaba la excepcin: ");
			e.printStackTrace();
		} catch (GameNotFoundException e) {			
			fail("No se esperaba la excepcin: ");
			e.printStackTrace();
		} catch (InvalidSessionException e) {			
			fail("No se esperaba la excepcin: ");
			e.printStackTrace();
		} catch (NotCurrentPlayerGameException e) {
			fail("No se esperaba la excepcin: ");
			e.printStackTrace();
		}        
    }
    
    public void testCase1()
    {		
    	try {
    		Connection.getConnection().updateGame(idInvalidSession, idInvalidGame, 
    				invalidPlayers, invalidTerritories, 
    				EventType.UnknownEvent);
			fail("Se esperaba exception");
		} catch (RemoteException e) {
			fail("No se esperaba la excepcin: ");
			e.printStackTrace();
		} catch (GameNotFoundException e) {
			e.printStackTrace();
		} catch (InvalidSessionException e) {
			e.printStackTrace();
		} catch (NotCurrentPlayerGameException e) {
			e.printStackTrace();
		}        
    }
    
    public void testCase2()
    {
    	try {
    		Connection.getConnection().updateGame(idInvalidSession, idInvalidGame, 
    				validPlayers, invalidTerritories, 
    				EventType.NegotiationEvent);
			fail("Se esperaba exception");
		} catch (RemoteException e) {
			fail("No se esperaba la excepcin: ");
			e.printStackTrace();
		} catch (GameNotFoundException e) {
			e.printStackTrace();
		} catch (InvalidSessionException e) {
			e.printStackTrace();
		} catch (NotCurrentPlayerGameException e) {
			fail("No se esperaba la excepcin: ");
			e.printStackTrace();
		}
    }
    
    public void testCase3()
    {
     	try {
     		Connection.getConnection().updateGame(idInvalidSession, idValidGame, 
     				invalidPlayers, validTerritories, 
     				EventType.UnknownEvent);			
			fail("Se esperaba exception");
		} catch (RemoteException e) {
			fail("No se esperaba la excepcin: ");
			e.printStackTrace();
		} catch (GameNotFoundException e) {
			fail("No se esperaba la excepcin: ");
			e.printStackTrace();
		} catch (InvalidSessionException e) {
			e.printStackTrace();
		} catch (NotCurrentPlayerGameException e) {
			e.printStackTrace();
		}
    }
    
    public void testCase4()
    {
     	try {
     		Connection.getConnection().updateGame(idInvalidSession, idValidGame, 
     				validPlayers, validTerritories, 
     				EventType.NegotiationEvent);
     		fail("Se esperaba exception");
		} catch (RemoteException e) {
			fail("No se esperaba la excepcin: ");
			e.printStackTrace();
		} catch (GameNotFoundException e) {
			fail("No se esperaba la excepcin: ");
			e.printStackTrace();
		} catch (InvalidSessionException e) {
			e.printStackTrace();
		} catch (NotCurrentPlayerGameException e) {
			fail("No se esperaba la excepcin: ");
			e.printStackTrace();
		}
    }
    
    public void testCase5()
    {
    	try {
    		Connection.getConnection().updateGame(idValidSession, idInvalidGame, 
    				invalidPlayers, validTerritories, 
    				EventType.NegotiationEvent);
			fail("Se esperaba exception");
		} catch (RemoteException e) {
			fail("No se esperaba la excepcin: ");
			e.printStackTrace();
		} catch (GameNotFoundException e) {
			e.printStackTrace();
		} catch (InvalidSessionException e) {
			fail("No se esperaba la excepcin: ");
			e.printStackTrace();
		} catch (NotCurrentPlayerGameException e) {
			e.printStackTrace();
		}
    }
    
    public void testCase6()
    {
    	try {
    		Connection.getConnection().updateGame(idValidSession, idValidGame, 
    				validPlayers, invalidTerritories, 
    				EventType.UnknownEvent);
			fail("Se esperaba exception");
		} catch (RemoteException e) {
			fail("No se esperaba la excepcin: ");
			e.printStackTrace();
		} catch (GameNotFoundException e) {
			fail("No se esperaba la excepcin: ");
			e.printStackTrace();
		} catch (InvalidSessionException e) {
			fail("No se esperaba la excepcin: ");
			e.printStackTrace();
		} catch (NotCurrentPlayerGameException e) {
			fail("No se esperaba la excepcin: ");
			e.printStackTrace();
		}        
    }

    public void tearDown()
    {
    	try {
			Connection.getConnection().logoutUser(this.idValidSession);
		} catch (RemoteException e) {
			e.printStackTrace();
		} catch (InvalidSessionException e) {
			e.printStackTrace();
		}
    }
}