package ea.test;

import java.rmi.RemoteException;
import java.util.ArrayList;
import java.util.UUID;

import junit.framework.TestCase;

import communications.EventType;

import domain.Game;
import domain.GameInfo;
import domain.Player;
import domain.Territory;
import domain.Territory.Continent;
import ea.libs.domain.Connection;
import ea.libs.exceptions.EmptyInformationException;
import ea.libs.exceptions.NullInformationException;
import ea.server.Server;
import exceptions.GameNotFoundException;
import exceptions.InvalidGameInfoException;
import exceptions.InvalidSessionException;
import exceptions.InvalidTimeException;
import exceptions.NotCurrentPlayerGameException;
import exceptions.UserAlreadyExistsException;
import exceptions.WrongLoginException;

public class TestCaseUpdateTurn extends TestCase
{
	// fixtures
    Game game;
    Territory territory1, territory2;
    int[] canons = {1,0,0};
    ArrayList<Territory> territoryUpdate;
    ArrayList<Territory> invalidTerritory;
    GameInfo gameInfo;
    Player player;
    ArrayList<String> players;
    ArrayList<Player> playerUpdate;
    ArrayList<Player> invalidPlayer;
    Long idGame, idPlayer;
    UUID session, gameUUID;
    UUID invalidSession, invalidGameUUID;
    EventType event, invalidEvent;
  
    
    public void setUp()
    {
    	gameUUID = UUID.randomUUID();
    	invalidSession = UUID.randomUUID();
    	invalidGameUUID =UUID.randomUUID();
    	
    	player = new Player("testPlayer", 100, true, true, null);
    	players = new ArrayList<String>();
    	players.add(player.getName());
    	
    	gameInfo = new GameInfo(gameUUID, "testGame", "testGame", players, null, 1, 1, 1, 1);
    	
    	territoryUpdate = new ArrayList<Territory>();
    	territory1 = new Territory(1, Continent.Africa, player, 100, canons, 1, 1, 1);
    	territory2 = new Territory(2, Continent.Africa, player, 1, canons, 1, 1, 1);
    	territoryUpdate.add(territory1);
    	territoryUpdate.add(territory2);
 
    	playerUpdate = new ArrayList<Player>();
    	playerUpdate.add(player);
    	game = new Game(gameInfo, territoryUpdate, playerUpdate);
    	
    	try {
    		    		
    		Connection.getConnection().createGame(gameInfo);  		
    		Connection.getConnection().registerUser("testPlayer", "0000", "testEmail@test.com");
    		session = Connection.getConnection().loginPlayer("testPlayer", "0000");
    		Connection.getConnection().joinGame(session, gameUUID);    
    		game = Connection.getConnection().playGame(session, gameUUID);
    		playerUpdate = game.getPlayers();
    		territoryUpdate = game.getMap();
    		playerUpdate.get(0).setMoney(50);
    		territoryUpdate.get(0).setNumSoldiers(50);
    		    		
		} catch (RemoteException e) {
			e.printStackTrace();
		} catch (NullInformationException e) {
			e.printStackTrace();
		} catch (InvalidGameInfoException e) {
			e.printStackTrace();
		} catch (UserAlreadyExistsException e) {
			e.printStackTrace();
		} catch (WrongLoginException e) {
			e.printStackTrace();
		} catch (Exception e) {
			e.printStackTrace();
		}   	
    }

    public void testUpdateTurn()
    {
        try
        {              	
            //Comprobamos que se actualiza el juego correctamente 
        	//Valid TestCase 
        	
        	Connection.getConnection().updateGame(session, gameUUID, playerUpdate, territoryUpdate, event.UnknownEvent);
        	Connection.getConnection().endTurn(session, gameUUID);
        	assertTrue(game.getMap().get(0).getNumSoldiers()==50 && game.getPlayers().get(0).getMoney()==50);
        	
        }
        catch (RemoteException re) {
        	fail(re.toString());
        } catch (GameNotFoundException e) {
			fail();
		} catch (InvalidSessionException e) {
			fail();
		} catch (NotCurrentPlayerGameException e) {
			fail();
		} catch (Exception e) {
			e.printStackTrace();
		}        
    }
    
    public void testUpdateTurnInvalid() {
    	
        try{   
        
        	//TestCase 1
        	Connection.getConnection().updateGame(invalidSession, invalidGameUUID, null, null, null);
        	
        	fail("Se esperaba excepcin ");        	
        } catch (GameNotFoundException e) {
			
		} catch (InvalidSessionException e) {
			
		} catch (RemoteException e) {
			e.printStackTrace();
		} catch (NotCurrentPlayerGameException e) {
			
		} catch (NullInformationException e) {
			
		}
    }
    
    public void testUpdateTurnInvalid2() {
    	
        try{   
        	
        	//TestCase 2
        	Connection.getConnection().updateGame(invalidSession, invalidGameUUID, null, invalidTerritory, event.AttackEvent);
        	
        	fail("Se esperaba excepcin ");        	
        } catch (GameNotFoundException e) {
			
		} catch (InvalidSessionException e) {
			
		} catch (RemoteException e) {
			e.printStackTrace();
		} catch (NotCurrentPlayerGameException e) {
			
		} catch (NullInformationException e) {
			
		}
    }
    
    public void testUpdateTurnInvalid3() {
    	
        try{   
        	
        	//TestCase 3
        	Connection.getConnection().updateGame(invalidSession, invalidGameUUID, invalidPlayer, territoryUpdate, null);
        	
        	fail("Se esperaba excepcin ");        	
        } catch (GameNotFoundException e) {
			
		} catch (InvalidSessionException e) {
			e.printStackTrace();
		} catch (RemoteException e) {
			e.printStackTrace();
		} catch (NotCurrentPlayerGameException e) {
			
		} catch (NullInformationException e) {
			
		}
    }
    
    public void testUpdateTurnInvalid4() {
    	
        try{   
        	
        	//TestCase 4
        	Connection.getConnection().updateGame(invalidSession, gameUUID, invalidPlayer, invalidTerritory, event.AttackEvent);
        	
        	fail("Se esperaba excepcin ");        	
        } catch (GameNotFoundException e) {
			
		} catch (InvalidSessionException e) {
			
		} catch (RemoteException e) {
			e.printStackTrace();
		} catch (NotCurrentPlayerGameException e) {
			
		} catch (NullInformationException e) {
			
		}
    }
    
    public void testUpdateTurnInvalid5() {
    	
        try{   
        	
        	//TestCase 5
        	Connection.getConnection().updateGame(invalidSession, gameUUID, playerUpdate, territoryUpdate, null);
        	
        	fail("Se esperaba excepcin ");        	
        } catch (GameNotFoundException e) {
			
		} catch (InvalidSessionException e) {
			
		} catch (RemoteException e) {
			e.printStackTrace();
		} catch (NotCurrentPlayerGameException e) {
			
		} catch (NullInformationException e) {
			
		}
    }
    
    public void testUpdateTurnInvalid6() {
    	
        try{   
        	
        	//TestCase 6
        	Connection.getConnection().updateGame(session, invalidGameUUID, invalidPlayer, null, null);
        	
        	fail("Se esperaba excepcin ");        	
        } catch (GameNotFoundException e) {
			
		} catch (InvalidSessionException e) {
			fail();
		} catch (RemoteException e) {
			e.printStackTrace();
		} catch (NotCurrentPlayerGameException e) {
			
		} catch (NullInformationException e) {
			
		}
    }
    
    public void testUpdateTurnInvalid7() {
    	
        try{   
        	
        	//TestCase 7
        	Connection.getConnection().updateGame(session, invalidGameUUID, playerUpdate, invalidTerritory, null);
        	fail("Se esperaba excepcin ");        	
        } catch (GameNotFoundException e) {
			
		} catch (InvalidSessionException e) {
			fail();
		} catch (RemoteException e) {
			e.printStackTrace();
		} catch (NotCurrentPlayerGameException e) {
			
		} catch (NullInformationException e) {
			
		}
    }
    
    public void testUpdateTurnInvalid8() {
    	
        try{   
        	
        	//TestCase 8
        	Connection.getConnection().updateGame(session, invalidGameUUID, playerUpdate, territoryUpdate, event.UnknownEvent);
        	
        	fail("Se esperaba excepcin ");        	
        } catch (GameNotFoundException e) {
			
		} catch (InvalidSessionException e) {
			fail();
		} catch (RemoteException e) {
			e.printStackTrace();
		} catch (NotCurrentPlayerGameException e) {
			
		} catch (NullInformationException e) {
			
		}
    }
    
    public void testUpdateTurnInvalid9() {
    	
        try{   
        	
        	//TestCase 9
        	Connection.getConnection().updateGame(session, gameUUID, null, invalidTerritory, null);
        	
        	fail("Se esperaba excepcin ");        	
        } catch (GameNotFoundException e) {
			fail();
		} catch (InvalidSessionException e) {
			fail();
		} catch (RemoteException e) {
			e.printStackTrace();
		} catch (NotCurrentPlayerGameException e) {
			
		} catch (NullInformationException e) {
			
		}
    }
    
    public void testUpdateTurnInvalid10() {
    	
        try{   
        	
        	//TestCase 10
        	Connection.getConnection().updateGame(session, gameUUID, null, territoryUpdate, event.AttackEvent);
        	        	
        	fail("Se esperaba excepcin ");        	
        } catch (GameNotFoundException e) {
			fail();
		} catch (InvalidSessionException e) {
			fail();
		} catch (RemoteException e) {
			e.printStackTrace();
		} catch (NotCurrentPlayerGameException e) {
			
		} catch (NullInformationException e) {
			
		}
    }
    
    public void testUpdateTurnInvalid11() {
    	
        try{   
        	
        	//TestCase 11
        	Connection.getConnection().updateGame(session, gameUUID, playerUpdate, null, null);
        	fail("Se esperaba excepcin ");        	
        } catch (GameNotFoundException e) {
			fail();
		} catch (InvalidSessionException e) {
			fail();
		} catch (NullInformationException e) {
			
		} catch (RemoteException e) {
			e.printStackTrace();
		} catch (NotCurrentPlayerGameException e) {
			
		}
    }

    public void tearDown()
    {
    	try {
    		Connection.getConnection().logoutUser(session);
    		//Connection.getConnection().deleteUser();
			//Connection.getConnection().deleteGame(game);
		} catch (RemoteException e) {
			e.printStackTrace();
		} catch (InvalidSessionException e) {
			e.printStackTrace();
		}
    }
}