package ea.test;

import java.rmi.Remote;
import java.rmi.RemoteException;
import java.util.UUID;

import junit.framework.TestCase;
import ea.communication.IServer;
import ea.communication.exceptions.UserAlreadyExistsException;
import ea.communication.exceptions.WrongLoginException;
import ea.domain.Connection;
import ea.exceptions.EmptyInformationException;
import ea.server.Server;

public class TestLogin extends TestCase
{
    // fixtures
    String userName, password;
    Server server;
    Connection conn;
    UUID session;
    Remote remote;

    public void setUp()
    {
    	try 
    	{
    		server = new Server();
    		server.startServer();
    		server.registerUser("luis", "1234", "xluis2055@hotmail.com");
    	} 
    	catch(RemoteException e) 
    	{
    		e.printStackTrace();
    	}
    	catch(UserAlreadyExistsException e) 
    	{
    		e.printStackTrace();
    	} 
    	catch(EmptyInformationException e) 
    	{
    		e.printStackTrace();
    	}

    public void testLoginPlayer()
    {
        try
        {
        	userName = "luis";
        	password = "1234";
            session = null;
            conn = new Connection("localhost");
            
            session = conn.getConnection().loginPlayer(userName, password);     
            
            assertNotNull(session);
            
            session = conn.getConnection().loginPlayer(userName, password);            
            fail("Se esperaba UserAlreadyExistsException()");
        }
        catch(RemoteException re)
        {
        	fail(re.toString());
        }
        catch(UserAlreadyExistsException ue)
        {
        	assertTrue(true);
        }
        catch(Exception e)
        {
            fail(e.toString());
        }
    }

    public void testLoginPlayerNeg()
    {
        // Test if the connection is not establish
    	// when the login and password are empty
        try
        {
            userName = "";
            password = "";
            session = null;
            conn = new Connection("localhost");
                        
            session = conn.getConnection().loginPlayer(userName, password);
              
            assertNull(session);

            userName = "defaultUser";
            password = "";
            session = null;
            remote = rmiInt;
            
            session = conn.getConnection().loginPlayer(userName, password);
            assertNull(session);

            userName = "";
            password = "defaultPassword";
            session = null;
            session = conn.getConnection().loginPlayer(userName, password);
            assertNull(session);
                        
            userName = "defaultUser";
            password = "defaultPassword";
            session = null;
                   
            session = conn.getConnection().loginPlayer(userName, password);
            fail("Se esperaba UsuarioInexistenteException()");
        }
        catch(RemoteException re)
        {
        	fail(re.toString());
        }
        catch(WrongLoginException ue)
        {
        	assertTrue(true);
        }
        catch(Exception e)
        {
            fail(e.toString());
        }
    }

    public void tearDown()
    {
    	//[TODO] Delete user registered for test
    	server.stopServer();
    }
}